<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: ajax.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 */

// include base api
include 'includes/pnAPI.php';

// start PN
pnInit(PN_CORE_ALL & ~PN_CORE_TOOLS & ~PN_CORE_DECODEURLS);

// Get variables
$module = FormUtil::getPassedValue('module', '', 'GETPOST');
$type   = FormUtil::getPassedValue('type', 'ajax', 'GETPOST');
$func   = FormUtil::getPassedValue('func', '', 'GETPOST');

// Check for site closed
if (pnConfigGetVar('siteoff') && !SecurityUtil::checkPermission('Settings::', 'SiteOff::', ACCESS_ADMIN) && !($module == 'Users' && $func == 'siteofflogin')) {
    if (SecurityUtil::checkPermission('Users::', '::', ACCESS_OVERVIEW) && pnUserLoggedIn()){
        pnUserLogOut();
    }
    AjaxUtil::error('Site is currently disabled');
}

if (empty($func)) {
    AjaxUtil::error('missing parameter \'func\'');
}

// get module information
$modinfo = pnModGetInfo(pnModGetIDFromName($module));
if ($modinfo == false) {
    AjaxUtil::error('unknown module ' . DataUtil::formatForDisplay($module));
}

if (!pnModAvailable($modinfo['name'])) {
    AjaxUtil::error('module ' . DataUtil::formatForDisplay($module) . ' not available');
}

if ($modinfo['type'] == 2 || $modinfo['type'] == 3)
{
    // New-new style of loading modules
    if (!isset($arguments)) {
        $arguments = array();
    }

    if (pnModLoad($modinfo['name'], $type)) {
        if (pnConfigGetVar('PN_CONFIG_USE_TRANSACTIONS')) {
                $dbConn = pnDBGetConn(true);
                $dbConn->StartTrans();
        }

        // Run the function
        $return = pnModFunc($modinfo['name'], $type, $func, $arguments);

        if (pnConfigGetVar('PN_CONFIG_USE_TRANSACTIONS')) {
            if ($dbConn->HasFailedTrans()) {
                $return = pnML('_TRANSACTIONFAILED') . '\n' . $return;
            	AjaxUtil::error($return);
            	$return == true;
            }
            $dbConn->CompleteTrans();
        }
    } else {
        $return = false;
    }

    // Sort out return of function.  Can be
    // true - finished
    // false - display error msg
    // text - return information
    if ($return === true) {
        // Nothing to do here everything was done in the module
    } elseif ($return === false) {
        // Failed to load the module
        AjaxUtil::error('Failed to load module <strong>' . DataUtil::formatForDisplay($module) .'</strong> (at function: "<strong>'. DataUtil::formatForDisplay($func).'</strong>")');
    } else {
        AjaxUtil::output($return, true, false);
    }
} else {
    // Old-old style of loading modules not supported with Ajax
    AjaxUtil::error('ajax support for old style modules not supported');
}

pnShutDown();
