<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: HtmlUtil.class.php 25336 2009-01-21 11:35:51Z rgasch $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @package Zikula_Core
 */

/**
 * HTMLUtil
 *
 * @package Zikula_Core
 * @subpackage HTMLUtil
 */
class HtmlUtil
{
    /**
     * Return the HTML code for the specified date selector input box
     *
     * @param objectname    The name of the object the field will be placed in
     * @param htmlname      The html fieldname under which the date value will be submitted
     * @param dateFormat    The dateformat to use for displaying the chosen date
     * @param defaultString The String to display before a value has been selected
     * @param defaultDate   The Date the calendar should to default to
     *
     * @return The resulting HTML string
     */
    function buildCalendarInputBox ($objectname, $htmlname, $dateFormat, $defaultString='', $defaultDate='')
    {
        $html = '';

        if (!$htmlname) {
            return pn_exit ('HtmlUtil::buildCalendarInputBox: Missing htmlname ...');
        }

        if (!$dateFormat) {
            return pn_exit ('HtmlUtil::buildCalendarInputBox: Missing dateFormat...');
        }

        $fieldKey    = $htmlname;
        if ($objectname) {
            $fieldKey  = $objectname.'['.$htmlname.']';
        }

        $triggerName = 'trigger_'.$htmlname;
        $displayName = 'display_'.$htmlname;
        //$daFormat    = preg_replace ('/([a-z|A-Z])/', '%$1', $dateFormat); // replace 'x' -> '%x'

        $html .= '<span id="'.$displayName.'">'.$defaultString.'</span>';
        $html .= '&nbsp;';
        $html .= '<input type="hidden" name="'.$fieldKey.'" id="'.$htmlname.'" value="'.$defaultDate.'" />';
        $html .= '<img src="javascript/jscalendar/img.gif" id="'.$triggerName.'" style="cursor: pointer; border: 0px solid blue;" title="Date selector" alt="Date selector" onmouseover="this.style.background=\'blue\';" onmouseout="this.style.background=\'\'" />';

        $html .= '<script type="text/javascript"> Calendar.setup({';
        $html .= 'ifFormat    : "%Y-%m-%d %H:%M:00",'; // universal format, don't change this!
        $html .= 'inputField  : "'.$htmlname.'",';
        $html .= 'displayArea : "'.$displayName.'",';
        $html .= 'daFormat    : "'.$dateFormat.'",';
        $html .= 'button      : "'.$triggerName.'",';
        $html .= 'align       : "Tl",';

        if ($defaultDate) {
            $d = strtotime ($defaultDate);
            $d = date ('Y/m/d', $d);
            $html .= 'date : "'.$d.'",';
        }

        $html .= 'singleClick : true }); </script>';

        return $html;
    }


    function getSelector_ObjectArray ($modname, $objectType, $name, $field='id', $displayField='name',
                                      $where='', $sort='', $selectedValue='', $defaultValue=0, $defaultText='', $allValue=0, $allText='',
                                      $displayField2=null, $submit=true, $disabled=false, $fieldSeparator=', ', $multipleSize=1)
    {
        if (!$modname) {
            return pn_exit ('Invalid modname passed to HtmlUtil::getSelector_ObjectArray ...');
        }

        if (!$objectType) {
            return pn_exit ('Invalid object name passed to HtmlUtil::getSelector_ObjectArray ...');
        }

        if (!pnModDBInfoLoad($modname)) {
            return "Unavailable/Invalid modulename [$modname] passed to HtmlUtil::getSelector_ObjectArray";
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        if ($allText) {
            $sel = ((string)$allValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$allValue\" $sel>$allText</option>";
        }

        if (!SecurityUtil::checkPermission("$objectType::", '::', ACCESS_OVERVIEW)) {
            return "Security Check failed for modulename [$modname] passed to HtmlUtil::getSelector_ObjectArray";
        }

        $cacheKey = md5("$modname|$objectType|$where|$sort");
        if (isset($cache[$cacheKey])) {
            $dataArray = $cache[$cacheKey];
        } else {
            $classname = Loader::loadClassFromModule ($modname, $objectType, true);
            if (!$classname) {
                return "Unable to load class [$objectType] for module [$modname]";
            }

            $class = new $classname ();
            $dataArray = $class->get ($where, $sort, -1, -1, '', false/*, $distinct*/);
            $cache[$cacheKey] = $dataArray;
        }

        foreach ($dataArray as $object) {
            $val   = $object[$field];
            $sel   = ((string)$val==(string)$selectedValue ? 'selected="selected"' : '');
            $disp  = DataUtil::formatForDisplayHTML($object[$displayField]);

            $disp2 = '';
            if ($displayField2) {
                $disp2 = $fieldSeparator . DataUtil::formatForDisplayHTML($object[$displayField2]);
            }

            $html .= "<option value=\"$val\" $sel>$disp $disp2</option>";
        }

        $html .= '</select>';
        return $html;
    }


    function getSelector_FieldArray ($modname, $tablekey, $name, $field='id', $where='', $sort='', $selectedValue='',
                                     $defaultValue=0, $defaultText='', $allValue=0, $allText='', $assocKey='', $distinct=false, 
                                     $submit=true, $disabled=false, $truncate=0, $multipleSize=1)
    {
        if (!$tablekey) {
            return pn_exit ("Invalid tablekey [$tablekey] passed to HtmlUtil::getSelector_FieldArray ...");
        }

        if (!$name) {
            return pn_exit ("Invalid name [$name] passed to HtmlUtil::getSelector_FieldArray ...");
        }

        if ($modname) {
            pnModDBInfoLoad($modname, '', true);
        }

        if ($truncate > 0) {
            Loader::loadClass ('StringUtil');
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? ' disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? ' multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? " size=\"$multipleSize\"" : '';
        $submit       = $submit ? ' onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\"{$multipleSize}{$multiple}{$submit}{$disabled}>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? ' selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\"{$sel}>$defaultText</option>";
        }

        if ($allText) {
            $sel = ((string)$allValue==(string)$selectedValue ? ' selected="selected"' : '');
            $html .= "<option value=\"$allValue\"{$sel}>$allText</option>";
        }

        $fa = DBUtil::selectFieldArray ($tablekey, $field, $where, $sort, $distinct, $assocKey);

        foreach ($fa as $k=>$v) {
            if ($v) {
                if ($truncate > 0 && strlen ($v) > $truncate) {
                    $v = StringUtil::getTruncatedString ($v, $truncate);
                }
                $v = DataUtil::formatForDisplayHTML($v);
                $sel   = ((string)$k==(string)$selectedValue ? ' selected="selected"' : '');
                $html .= "<option value=\"$k\"{$sel}>$v</option>";
            }
        }

        $html .= '</select>';
        return $html;
    }


    /**
     * Return the HTML selector code for the given category hierarchy, maps to CategoryUtil::getSelector_Categories()
     *
     * @param cats              The category hierarchy to generate a HTML selector for
     * @param name              The name of the selector field to generate (optional) (default='category[parent_id]')
     * @param field             The field value to return (optional) (default='id')
     * @param selectedValue     The selected category (optional) (default=0)
     * @param defaultValue      The default value to present to the user (optional) (default=0)
     * @param defaultText       The default text to present to the user (optional) (default='')
     * @param allValue          The value to assign for the "All" choice (optional) (default=0)
     * @param allText           The text to display for the "All" choice (optional) (default='')
     * @param submit            whether or not to submit the form upon change (optional) (default=false)
     * @param displayPath       If false, the path is simulated, if true, the full path is shown (optional) (default=false)
     * @param doReplaceRootCat  Whether or not to replace the root category with a localized string (optional) (default=true)
     * @param multipleSize      If > 1, a multiple selector box is built, otherwise a normal/single selector box is build (optional) (default=1)
     *
     * @return The HTML selector code for the given category hierarchy
     */
    function getSelector_Categories ($cats, $name, $field='id', $selectedValue='0', $defaultValue=0, $defaultText='', $allValue=0, $allText='',
                                     $submit=false, $displayPath=false, $doReplaceRootCat=true, $multipleSize=1)
    {
        Loader::loadClass ('CategoryUtil');
        return CategoryUtil::getSelector_Categories ($cats, $field, $selectedValue, $name, $defaultValue, $defaultText, $allValue, $allText,
                                                     $submit, $displayPath, $doReplaceRootCat, $multipleSize);
    }


    /**
     * Return the HTML code for the values in a given category
     *
     * @param categoryPath  The identifying category path
     * @param values        The values used to populate the defautl states (optional) (default=array())
     * @param namePrefix    The path/object prefix to apply to the field name (optional) (default='')
     * @param excludeList   A (string) list of IDs to exclude (optional) (default=null)
     * @param disabled      whether or not the checkboxes are to be disabled (optional) (default=false)
     *
     * @return The resulting dropdown data
     */
    function getCheckboxes_CategoryField ($categoryPath, $values=array(), $namePrefix='',
                                          $excludeList=null, $disabled=false)
    {
        Loader::loadClass ('CategoryUtil');

        if (!$categoryPath) {
            return pn_exit ('Invalid category name passed to HtmlUtil::getSelector_Category ...');
        }

        if (!$lang) {
            $lang = SessionUtil::getVar('lang');
        }

        //if ($_SESSION['pnCache']['categoryselectorbypath'][$categoryPath] && !$force)
        //return $_SESSION['pnCache']['categoryselectorbypath'][$categoryPath];

        $cats = CategoryUtil::getSubCategoriesByPath ($categoryPath, 'path', false, true, false, true, false, '', 'value');

        foreach ($cats as $k => $v)
        {
            $val   = $k;
            $fname = $val;
            if ($namePrefix) {
                $fname = $namePrefix . '[' . $k . ']' ;
            }

            if (strpos($excludeList, ','.$k.',') === false) {
                $disp  = $v['display_name'][$lang];
                if (!$disp) {
                    $disp = $v['name'];
                }

                $html .= "<input type=\"checkbox\" name=\"$fname\" " . ( $values[$k] ? ' checked="checked" ' : '') . ( $disabled ? ' disabled="disabled" ' : '') . " />&nbsp;&nbsp;&nbsp;&nbsp;$disp<br />";
            }
        }

        return $html;
    }


    function getSelector_ModuleTables ($modname, $name, $selectedValue='', $defaultValue=0, $defaultText='',
                                       $submit=false, $remove='', $disabled=false, $nStripChars=0, $multipleSize=1)
    {
        if (!$modname) {
            return pn_exit ('Invalid modname passed to HtmlUtil::getSelector_ModuleTables ...');
        }

        if (!$name) {
            return pn_exit ('Invalid name passed to HtmlUtil::getSelector_ModuleTables ...');
        }

        $m = strtolower (str_replace ('pn', '', $modname));
        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        $tables = pnModDBInfoLoad($modname, '', true);

        if (is_array($tables) && $tables) {
            foreach ($tables as $k => $v) {
                if (strpos($k, '_column') === false &&
                    strpos($k, '_db_extra_enable') === false &&
                    strpos($k, '_primary_key_column') === false) {
                    if (strpos($k, 'pn_') === 0) {
                        $k = substr ($k, 4);
                    }

                    if ($remove) {
                        $k2= str_replace ($remove, '', $k);
                    } else {
                        $k2 = $k;
                    }

                    if ($nStripChars) {
                        $k2 = ucfirst(substr($k2, $nStripChars));
                    }

                    $sel   = ($k2==$selectedValue ? 'selected="selected"' : '');
                    // Use $k2 for display also (instead of showing the internal table name)
                    // See http://noc.postnuke.com/tracker/?func=detail&aid=16110&group_id=5&atid=101
                    $html .= "<option value=\"$k2\" $sel>$k2</option>";
                }
            }
        }

        $html .= '</select>';
        return $html;
    }


    function getSelector_TableFields ($modname, $tablename, $name, $selectedValue='', $defaultValue=0, $defaultText='', $submit=false,
                                      $showSystemColumns=false, $disabled=false, $multipleSize=1)
    {
        if (!$modname) {
            return pn_exit ('Invalid modname passed to HtmlUtil::getSelector_TableFields ...');
        }

        if (!$tablename) {
            return pn_exit ('Invalid tablename passed to HtmlUtil::getSelector_TableFields ...');
        }

        if (!$name) {
            return pn_exit ('Invalid name passed to HtmlUtil::getSelector_TableFields ...');
        }

        $m = strtolower (str_replace ('pn', '', $modname));
        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selected ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        $tables = pnModDBInfoLoad($modname, '', true);
        $colkey = $tablename . '_column';
        $cols   = $tables[$colkey];

        if (!$cols) {
            return pn_exit ("Invalid column key [$colkey] in HtmlUtil::getSelector_TableField ...");
        }

        if (!$showSystemColumns) {
            $filtercols = array();
            ObjectUtil::addStandardFieldsToTableDefinition ($filtercols, '');
        }

        foreach ($cols as $k => $v) {
            if ($showSystemColumns) {
                $sel   = ((string)$v==(string)$selectedValue ? 'selected="selected"' : '');
                $html .= "<option value=\"$v\" $sel>$k</option>";
            } else {
                if (!$filtercols[$k]) {
                    $sel   = ((string)$v==(string)$selectedValue ? 'selected="selected"' : '');
                    $html .= "<option value=\"$v\" $sel>$k</option>";
                }
            }
        }

        $html .= '</select>';
        return $html;
    }


    /**
     * Return the HTML code for the Yes/No dropdown
     *
     * @param selected    The value which should be selected (default=1) (optional)
     * @param name        The name of the generated selector (optional)
     *
     * @return The resulting HTML string
     */
    function getSelector_YesNo ($selected='1', $name='')
    {
        if (!$name) {
            $name='permission';
        }

        $vals = array ();
        $vals[0] = _NO;
        $vals[1] = _YES;
        $id = strtr ($name, '[]', '__');

        $html = '<select name="'.$name.'"'
        .' id="'.DataUtil::formatForDisplay($id).'">';
        for ($i=0; $i<sizeof($vals); $i++) {
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$i\" $sel>$vals[$i]</option>";
        }
        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the localized string for the specified yes/no value
     *
     * @param val        The value for which we wish to obtain the string representation
     *
     * @return The string representation for the selected value
     */
    function getSelectorValue_YesNo ($val)
    {
        $vals = array ();
        $vals[0] = _NO;
        $vals[1] = _YES;

        return $vals[$val];
    }


    /**
     * Return the dropdown data for the language selector
     *
     * @param includeAll     whether or not to include the 'All' choice
     *
     * @return The string representation for the selected value
     */
    function getSelectorData_Language ($includeAll=true)
    {
        $lang = languagelist();
        $langlist = array ();
        $dropdown = array ();

        if ($includeAll) {
            $dropdown[] = array ('id'=>'', 'name'=>_ALL);
        }

        $handle = opendir('language');
        while ($f = readdir($handle)) {
            if (is_dir("language/$f") && @$lang[$f]) {
                $langlist[$f] = $lang[$f];
            }
        }

        asort($langlist);

        foreach ($langlist as $k=>$v) {
            $dropdown[] = array ('id'=>$k, 'name'=>$v);
        }

        return $dropdown;
    }


    /**
     * Return the localized string for the given value
     *
     * @param value        The currently active/selected value
     *
     * @return The resulting HTML string
     */
    function getSelectorValue_Permission ($value)
    {
        $perms = array ();
        $perms[_PN_PERMISSION_BASIC_PRIVATE] = _PN_TEXT_PERMISSION_BASIC_PRIVATE;
        $perms[_PN_PERMISSION_BASIC_GROUP] = _PN_TEXT_PERMISSION_BASIC_GROUP;
        $perms[_PN_PERMISSION_BASIC_USER] = _PN_TEXT_PERMISSION_BASIC_USER;
        $perms[_PN_PERMISSION_BASIC_PUBLIC] = _PN_TEXT_PERMISSION_BASIC_PUBLIC;

        return $perms[$value];
    }


    /**
     * Return the HTML code for the Permission dropdown
     *
     * @param name        The name of the generated selector (optional) (default='permission')
     * @param selected    The value which should be selected (optional) (default=2)
     *
     * @return The resulting HTML string
     */
    function getSelector_Permission ($name='permission', $selected='U')
    {
        if (!$name) {
            $name='permission';
        }

        $perms = array ();
        $perms[_PN_PERMISSION_BASIC_PRIVATE] = _PN_TEXT_PERMISSION_BASIC_PRIVATE;
        $perms[_PN_PERMISSION_BASIC_GROUP] = _PN_TEXT_PERMISSION_BASIC_GROUP;
        $perms[_PN_PERMISSION_BASIC_USER] = _PN_TEXT_PERMISSION_BASIC_USER;
        $perms[_PN_PERMISSION_BASIC_PUBLIC] = _PN_TEXT_PERMISSION_BASIC_PUBLIC;
        //$perms[0] = _PN_TEXT_PERMISSION_BASIC_PRIVATE;
        //$perms[1] = _PN_TEXT_PERMISSION_BASIC_GROUP;
        //$perms[2] = _PN_TEXT_PERMISSION_BASIC_PUBLIC;

        $id = strtr ($name, '[]', '__');

        $html = '<select name="'.$name.'"'
        .' id="'.DataUtil::formatForDisplay($id).'">';
        foreach ($perms as $k => $v) {
            $sel = ($k==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$k\" $sel>$v</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML code for the Permission Level dropdown
     *
     * @param name        The name of the generated selector (optional) (default='permission')
     * @param selected    The value which should be selected (optional) (default=2)
     *
     * @return The resulting HTML string
     */
    function getSelector_PermissionLevel ($name='permission', $selected='0')
    {
        $perms = array ();
        $perms[_PN_PERMISSION_LEVEL_NONE] = _PN_TEXT_PERMISSION_LEVEL_NONE;
        $perms[_PN_PERMISSION_LEVEL_READ] = _PN_TEXT_PERMISSION_LEVEL_READ;
        $perms[_PN_PERMISSION_LEVEL_WRITE] = _PN_TEXT_PERMISSION_LEVEL_WRITE;

        $id = strtr ($name, '[]', '__');

        $html = '<select name="'.$name.'"'
        .' id="'.DataUtil::formatForDisplay($id).'">';
        foreach ($perms as $k => $v) {
            $sel = ($k==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$k\" $sel>$v</option>";
        }
        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the html for the PN user group selector
     *
     * @param name           The selector name
     * @param selectedValue  The currently selected value of the selector (optional) (default=0)
     * @param defaultValue   The default value of the selector (optional) (default=0)
     * @param defaultText    The text of the default value (optional) (default='')
     * @param allValue       The value to assign for the "All" choice (optional) (default=0)
     * @param allText        The text to display for the "All" choice (optional) (default='')
     * @param excludeList    A (string) list of IDs to exclude (optional) (default=null)
     * @param submit         Whether or not to auto-submit the selector (optional) (default=false)
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The html for the user group selector
     */
    function getSelector_PNGroup ($name='groupid', $selectedValue=0, $defaultValue=0, $defaultText='',
                                  $allValue=0, $allText='', $excludeList='', $submit=false, $disabled=false, $multipleSize=1)
    {
        Loader::loadClass ('UserUtil');

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        if ($allText) {
            $sel = ((string)$allValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$allValue\" $sel>$allText</option>";
        }

        $where = '';
        $grouplist = UserUtil::getPNGroups ('', 'ORDER BY pn_name');
        foreach ($grouplist as $k => $v) {
            $id   = $v['gid'];
            $disp = $v['name'];
            if (strpos($excludeList, ",$id,") === false) {
                $sel = ((string)$selectedValue==(string)$id ? 'selected="selected"' : '');
                $html .= "<option value=\"$id\" $sel>$disp</option>";
            }
        }

        $html .= '</select>';

        return $html;
    }


    /**
     * Return a PN array strcuture for the PN user dropdown box
     *
     * @param name           The selector name
     * @param gid            The group ID to get users for (optional) (default=null)
     * @param selectedValue  The currently selected value of the selector (optional) (default=0)
     * @param defaultValue   The default value of the selector (optional) (default=0)
     * @param defaultText    The text of the default value (optional) (default='')
     * @param allValue       The value to assign for the "All" choice (optional) (default='')
     * @param allText        The text to display for the "All" choice (optional) (default='')
     * @param excludeList    A (string) list of IDs to exclude (optional) (default=null)
     * @param submit         Whether or not to auto-submit the selector (optional) (default=false)
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The string for the user group selector
     */
    function getSelector_PNUser ($name='userid', $gid=null, $selectedValue=0, $defaultValue=0, $defaultText='',
                                 $allValue=0, $allText='', $excludeList='', $submit=false, $disabled=false, $multipleSize=1)
    {
        Loader::loadClass ('UserUtil');

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        if ($allText) {
            $sel = ((string)$allValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$allValue\" $sel>$allText</option>";
        }

        $where = '';
        if ($excludeList) {
            $where = "WHERE pn_uid NOT IN ($excludeList)";
        }

        if ($gid) {
            $users  = UserUtil::getUsersForGroup ($gid);
            if ($users) {
                $and    = $where ? ' AND ' : '';
                $where .= $and .  'pn_uid IN (' . implode(',', $users) . ')';
            }
        }

        $userlist = UserUtil::getPNUsers($where, 'ORDER BY pn_uname');
        foreach ($userlist as $k => $v) {
            $sel = ((string)$selectedValue==(string)$k ? 'selected="selected"' : '');
            $html .= "<option value=\"$k\" $sel>$v[uname]</option>";
        }

        $html .= '</select>';

        return $html;
    }


    /**
     * Return the html for the PNModule selector
     *
     * @param name           The selector name
     * @param selectedValue  The currently selected value of the selector (optional) (default=0)
     * @param defaultValue   The default value of the selector (optional) (default=0)
     * @param defaultText    The text of the default value (optional) (default='')
     * @param allValue       The value to assign the "All" choice (optional) (default=0)
     * @param allText        The text to display for the "All" choice (optional) (default='')
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The string for the user group selector
     */
    function getSelector_PNModule ($name='moduleName', $selectedValue=0, $defaultValue=0, $defaultText='',
                                   $allValue=0, $allText='', $submit=false, $disabled=false, $multipleSize=1)
    {
        Loader::loadClass ('ModuleUtil');

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        if ($allValue) {
            $sel = ((string)$allValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$allValue\" $sel>$allText</option>";
        }

        $where = '';
        $modlist = ModuleUtil::getModulesByState (3, 'displayname');
        foreach ($modlist as $v) {
            $name = $v['name'];
            $displayname = $v['displayname'];
            $sel  = ((string)$selectedValue==(string)$name ? 'selected="selected"' : '');
            $html .= "<option value=\"$name\" $sel>$displayname</option>";
        }

        $html .= '</select>';

        return $html;
    }


    /**
     * Return the HTML for the date day selector
     *
     * @param selected       The value which should be selected (default=0) (optional)
     * @param name           The name of the generated selector (default='day') (optional)
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_DatetimeDay ($selected=0, $name='day', $submit=false, $disabled=false, $multipleSize=1)
    {
        if (!$name) {
            $name='day';
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        for ($i=1; $i<32; $i++) {
            $val = sprintf ("%02d", $i);
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$val\" $sel>$val</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML for the date hour selector
     *
     * @param selected       The value which should be selected (default=0) (optional)
     * @param name           The name of the generated selector (default='hour') (optional)
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_DatetimeHour($selected=0, $name='hour', $submit=false, $disabled=false, $multipleSize=1)
    {
        if (!$name) {
            $name='hour';
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        for ($i=0; $i<24; $i++) {
            $val = sprintf ("%02d", $i);
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$val\" $sel>$val</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML for the date minute selector
     *
     * @param selected       The value which should be selected (default=0) (optional)
     * @param name           The name of the generated selector (default='minute') (optional)
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_DatetimeMinute($selected=0, $name='minute', $submit=false, $disabled=false, $multipleSize=1)
    {
        if (!$name) {
            $name='minute';
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        for ($i=0; $i<60; $i+=5) {
            $val = sprintf ("%02d", $i);
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$val\" $sel>$val</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML for the date month selector
     *
     * @param selected       The value which should be selected (default=0) (optional)
     * @param name           The name of the generated selector (default='month') (optional)
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_DatetimeMonth ($selected=0, $name='month', $submit=false, $disabled=false, $multipleSize=1)
    {
        if (!$name) {
            $name='month';
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        for ($i=1; $i<13; $i++) {
            $val = sprintf ("%02d", $i);
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$val\" $sel>$val</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML for the date year selector
     *
     * @param selected       The value which should be selected (default=2003) (optional)
     * @param name           The name of the generated selector (default='year') (optional)
     * @param first          The start year for the selector (default=2003) (optional)
     * @param last           The name of the generated selector (default=2007) (optional)
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_DatetimeYear ($selected=2007, $name='year', $first=2003, $last=2008, $submit=false, $disabled=false, $multipleSize=1)
    {
        if (!$name) {
            $name='year';
        }

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        for ($i=$first; $i<$last; $i++) {
            $sel = ($i==$selected ? 'selected="selected"' : '');
            $html = $html . "<option value=\"$i\" $sel>$i</option>";
        }

        $html = $html . '</select>';

        return $html;
    }


    /**
     * Return the HTML for the country selector
     *
     * @param name           The name of the generated selector (default='countries') (optional)
     * @param selectedValue  The value which is currently selected (default='') (optional)
     * @param defaultValue   The default value to select (default='') (optional)
     * @param defaultText    The text for the default value (default='') (optional)
     * @param allValue       The value to assign for the "All" choice (optional) (default=0)
     * @param allText        The text to display for the "All" choice (optional) (default='')
     * @param submit         Whether or not to auto-submit the selector
     * @param disabled       Whether or not to disable selector (optional) (default=false)
     * @param multipleSize   The size to use for a multiple selector, 1 produces a normal/single selector (optional (default=1)
     *
     * @return The generated HTML for the selector
     */
    function getSelector_Countries ($name='countries', $selectedValue='', $defaultValue=0, $defaultText='', 
                                    $allValue=0, $allText='', $submit=false, $disabled=false, $multipleSize=1)
    {
        $countries = countrylist();
        asort ($countries);

        $id = strtr ($name, '[]', '__');
        $disabled     = $disabled ? 'disabled="disabled"' : '';
        $multiple     = $multipleSize > 1 ? 'multiple="multiple"' : '';
        $multipleSize = $multipleSize > 1 ? "size=\"$multipleSize\"" : '';
        $submit       = $submit ? 'onchange="this.form.submit();"' : '';

        $html = "<select name=\"$name\" id=\"$id\" $multipleSize $multiple $submit $disabled>";

        if ($defaultText && !$selectedValue) {
            $sel = ((string)$defaultValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$defaultValue\" $sel>$defaultText</option>";
        }

        if ($allText) {
            $sel = ((string)$allValue==(string)$selectedValue ? 'selected="selected"' : '');
            $html .= "<option value=\"$allValue\" $sel>$allText</option>";
        }

        $ak = array_keys($countries);
        foreach ($ak as $k) {
            $sel = ((string)$selectedValue==(string)$k ? 'selected="selected"' : '');
            $html .= "<option value=\"$k\" $sel>" . DataUtil::formatForDisplay($countries[$k]) . '</option>';
        }

        $html .= '</select>';

        return $html;
    }


    /**
     * Same as PN HTMLApi function but adds javascript form submit code to selector
     */
    function FormSelectMultipleSubmit($fieldname, $data, $multiple=0, $size=1, $selected = '', $accesskey='', $onchange='')
    {
        if (empty ($fieldname))  {
            return;
        }

        // Set up selected if required
        if (!empty($selected)) {
            for ($i=0; !empty($data[$i]); $i++) {
                if ($data[$i]['id'] == $selected) {
                    $data[$i]['selected'] = 1;
                }
            }
        }

        $c = count($data);
        if ($c < $size) {
            $size = $c;
        }

        $idname = strtr ($fieldname, '[]', '__');

        $output = '<select'
        .' name="'.DataUtil::formatForDisplay($fieldname).'"'
        .' id="'.DataUtil::formatForDisplay($idname).'"'
        .' size="'.DataUtil::formatForDisplay($size).'"'
        .(($multiple == 1) ? ' multiple="multiple"' : '')
        .((empty ($accesskey)) ? '' : ' accesskey="'.DataUtil::formatForDisplay($accesskey).'"')
        //.' tabindex="'.$this->tabindex.'"'
        .($onchange ? " onchange=\"$onchange\"" : '')
        .'>';

        foreach ($data as $datum) {
            $output .= '<option'
            .' value="'.DataUtil::formatForDisplay($datum['id']).'"'
            .((empty ($datum['selected'])) ? '' : " selected='$datum[selected]'")
            .'>'
            .DataUtil::formatForDisplay($datum['name'])
            .'</option>';
        }

        $output .= '</select>';
        return $output;
    }
}
