<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnUser.php 25330 2009-01-20 18:52:00Z Landseer $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Core
 * @subpackage pnAPI
*/

/**
 * Defines
 */

/**
 * Data types for User Properties
 */
define('UDCONST_MANDATORY', -1); // indicates a core field that can't be removed
define('UDCONST_CORE', 0); // indicates a core field (HACK, to be removed?)
define('UDCONST_STRING', 1);
define('UDCONST_TEXT', 2);
define('UDCONST_FLOAT', 3);
define('UDCONST_INTEGER', 4);

/**
 * Log the user in
 *
 * @param uname $ the name of the user logging in
 * @param pass $ the password of the user logging in
 * @param rememberme whether $ or not to remember this login
 * @param checkPassword bool true whether or not to check the password
 * @return bool true if the user successfully logged in, false otherwise
 */
function pnUserLogIn($uname, $pass, $rememberme=false, $checkPassword=true)
{
    if (pnUserLoggedIn()) {
        return true;
    }

    $uservars = pnModGetVar('Users');

    if (!pnVarValidate($uname, (($uservars['loginviaoption']==1) ? 'email' : 'uname'))) {
        return false;
    }

    // get the database connection
    pnModDBInfoLoad('Users', 'Users');
    pnModAPILoad('Users', 'user', true);

    $uname = strtolower ($uname);
    if (!pnModAvailable('AuthPN')) {
        if (!isset($uservars['loginviaoption']) || $uservars['loginviaoption'] == 0) {
            $user = DBUtil::selectObjectByID('users', $uname, 'uname', null, null, null, false, 'lower');
        } else {
            $user = DBUtil::selectObjectByID('users', $uname, 'email', null, null, null, false, 'lower');
        }

        if (!$user) {
            return false;
        }

        // check if the account is active
        if (isset($user['activated']) && $user['activated']=='0') {
            // account inactive, deny login
            return false;
        } else if ($user['activated']=='2') {
            // we need a session var here that can have 3 states
            // 0: account needs to be activated, this is the value after
            //    we detected this
            // 1: account needs to activated, user check the accept checkbox
            // 2: everything is ok
            // have we been here before?
            $confirmtou = SessionUtil::getVar('confirmtou', 0);
            switch($confirmtou) {
                case 0:
                    // continue if legal module is active and and configured to
                    // use the terms of use
                    if (pnModAvailable('legal')) {
                        $tou = pnModGetVar('legal', 'termsofuse');
                        if ($tou==1) {
                            // users must confirm terms of use before before he can continue
                            // we redirect him to the login screen
                            // to ensure that he reads this reminder
                            SessionUtil::setVar('confirmtou', 0);
                            return false;
                        }
                    }
                    break;
                case 1: // user has accepted the terms of use - continue
                    break;
                case 2:
                default:
            }
        }

        $uid = $user['uid'];

        // password check doesn't apply to HTTP(S) based login
        if ($checkPassword) {
            $upass = $user['pass'];
            $pnuser_hash_number = $user['hash_method'];
            $hashmethodsarray   = pnModAPIFunc('Users', 'user', 'gethashmethods', array('reverse' => true));

            $hpass = DataUtil::hash($pass, $hashmethodsarray[$pnuser_hash_number]);
            if ($hpass != $upass) {
                return false;
            }

            // Check stored hash matches the current system type, if not convert it.
            $system_hash_method = $uservars['hash_method'];
            if ($system_hash_method != $hashmethodsarray[$pnuser_hash_number]) {
                $newhash = DataUtil::hash($pass, $system_hash_method);
                $hashtonumberarray = pnModAPIFunc('Users', 'user', 'gethashmethods');

                $obj = array('uid'         => $uid,
                             'pass'        => $newhash,
                             'hash_method' => $hashtonumberarray[$system_hash_method]);
                $result = DBUtil::updateObject($obj, 'users', '', 'uid');

                if (!$result) {
                    return false;
                }
            }
        }

        // Storing Last Login date
        if ($uservars['savelastlogindate']) {
            if (!pnUserSetVar('lastlogin', date("Y-m-d H:i:s", time()), $uid)) {
                // show messages but continue
                LogUtil::registerError(_UNABLETOSAVELOGINDATE);
            }
        }
    } else {
        $authmodules = explode(',', pnModGetVar('AuthPN','authmodules'));
        foreach ($authmodules as $authmodule) {
            $authmodule = trim($authmodule);
            if (pnModAvailable($authmodule) && pnModAPILoad($authmodule, 'user')) {
                $uid = pnModAPIFunc($authmodule, 'user', 'login',
                        array('uname' => $uname, 'pass' => $pass, 'rememberme' => $rememberme, 'checkPassword' => $checkPassword));
                if ($uid) {
                    break;
                }
            }
        }
        if (!$uid) {
            return false;
        }
    }

    if (!defined('_PNINSTALLVER')) {
        SessionUtil::requireSession();
    }

    // Set session variables
    SessionUtil::setVar('uid', (int)$uid);
    if (!empty($rememberme)) {
        SessionUtil::setVar('rememberme', 1);
    }

    if (isset($confirmtou) && $confirmtou==1) {
        // if we get here, the user did accept the terms of use
        // now update the status
        pnUserSetVar('activated', 1, (int)$uid);
        SessionUtil::delVar('confirmtou');
    }

    // now we've logged in the permissions previously calculated are invalid
    $GLOBALS['authinfogathered'][$uid] = 0;

    return true;
}

/**
 * Log the user in via the REMOTE_USER SERVER property. This routine simply
 * checks if the REMOTE_USER exists in the PN environment: if he does a
 * session is created for him, regardless of the password being used.
 *
 * @return bool true if the user successfully logged in, false otherwise
 */
function pnUserLogInHTTP()
{
    $uname = pnServerGetVar('REMOTE_USER');
    $hSec  = pnConfigGetVar('session_http_login_high_security', true);
    $rc    = pnUserLogIn($uname, null, false, false);
    if ($rc && $hSec) {
        pnConfigSetVar('seclevel', 'High');
    }

    return $rc;
}

/**
 * Log the user out
 *
 * @public
 * @return bool true if the user successfully logged out, false otherwise
 */
function pnUserLogOut()
{
    if (pnUserLoggedIn()) {
        if (pnModAvailable('AuthPN')) {
            $authmodules = explode(',', pnModGetVar('AuthPN','authmodules'));
            foreach ($authmodules as $authmodule) {
                $authmodule = trim($authmodule);
                if (pnModAvailable($authmodule) && pnModAPILoad($authmodule, 'user')) {
                    if (!$result = pnModAPIFunc($authmodule, 'user', 'logout')) {
                        return false;
                    }
                }
            }
        }

        // delete logged on user the session
        // SessionUtil::delVar('rememberme');
        // SessionUtil::delVar('uid');
        session_destroy();
    }

    return true;
}

/**
 * is the user logged in?
 *
 * @public
 * @returns bool true if the user is logged in, false if they are not
 */
function pnUserLoggedIn()
{
    return (bool)SessionUtil::getVar('uid');
}

/**
 * get all user variables
 *
 * @access public
 * @author Gregor J. Rothfuss
 * @since 1.33 - 2002/02/07
 * @param uid $ the user id of the user
 * @return array an associative array with all variables for a user
 */
function pnUserGetVars($uid, $force = false)
{
    if (!is_numeric($uid)) {
        return false;
    }

    static $cache = array();
    static $props;

    // caching
    if (isset($cache[$uid]) && !$force) {
        return $cache[$uid];
    }

    // load the Users database information
    pnModDBInfoLoad('Users', 'Users');

    // get user info
    $user = DBUtil::selectObjectByID('users', $uid, 'uid');
    // user can be false (error) or empty array (no such user)
    if (empty($user)) {
        return false;
    }       
    if ($user == false) {
        return LogUtil::registerError(_GETFAILED);
    }

    // duplicate column info with 'pn_' prefix
    foreach ($user as $k => $v) {
        // special case: do not duplicate attributes, meta data and categories information
        if (($k <> '__ATTRIBUTES__') && ($k <> '__META__') && ($k <> '__CATEGORIES__')) {
            $k2 = 'pn_' . $k;
            $user[$k2] = $v;
        }
    }

    // create an empty array to hold the user vars
    $vars = array();

    // decide if we have to use the (obsolete) DUDs from the Profile module
    $profileModule = pnConfigGetVar('profilemodule', '');
    $useOldProfileDUD = (!empty($profileModule) && $profileModule == 'Profile' && pnModAvailable($profileModule));

    if ($useOldProfileDUD) {
        // load the Profile database information
        pnModDBInfoLoad('Profile', 'Profile');

        // get table/column info
        $pntable     = pnDBGetTables();
        $propcolumn  = $pntable['user_property_column'];
        $datacolumn  = $pntable['user_data_column'];

        // get dynamic properties
        if (!isset($props)) {
            $where = "$propcolumn[prop_dtype] >= 1 ";
            $sort  = "$propcolumn[prop_weight] ASC";
            $props = DBUtil::selectObjectArray('user_property', $where, $sort);
        }

        // get user dynamic properties
        $where = "$datacolumn[uda_uid]=" . DataUtil::formatForStore($uid);
        $userprops = DBUtil::selectObjectArray('user_data', $where, '', -1, -1, 'uda_propid');

        // assign dynamic property data
        foreach ($props as $prop) {
            $id     = $prop['prop_id'];
            $label  = $prop['prop_label'];
            $vars[$label]  = isset($userprops[$id]) ? $userprops[$id]['uda_value'] : null;
        }

        $vars = array_merge($user, $vars);
    }
    else {
        $vars = $user;
    }

    $cache[$uid] = $vars;
    return($vars);
}

/**
 * get a user variable
 *
 * @public
 * @author Jim McDonald
 * @param name $ the name of the variable
 * @param uid $ the user to get the variable for
 * @param default $ the default value to return if the specified variable doesn't exist
 * @return string the value of the user variable if successful, false otherwise
 */
function pnUserGetVar($name, $uid=-1, $default=false)
{
    static $vars = array();

    if (empty($name)) {
        return;
    }

    // bug fix #1311 [landseer]
    if (isset($uid) && !is_numeric($uid) ) {
        return;
    }

    if ($uid == -1) {
        $uid = SessionUtil::getVar('uid');
    }
    if (empty($uid)) {
        return;
    }

    // Get this user's variables if not already obtained
    if (!isset($vars[$uid])) {
        $vars[$uid] = pnUserGetVars($uid);
    }

    // Return the variable
    if (isset($vars[$uid][$name])) {
        return $vars[$uid][$name];
    }

    return $default;
}

/**
 * set a user variable
 *
 * @access public
 * @author Gregor J. Rothfuss
 * @since 1.23 - 2002/02/01
 * @param name $ the name of the variable
 * @param value $ the value of the variable
 * @param uid $ the user to set the variable for
 * @return bool true if the set was successful, false otherwise
 */
function pnUserSetVar($name, $value, $uid = -1)
{
    $pntable = pnDBGetTables();

    if (empty($name)) {
        return false;
    }
    if (!isset($value)) {
        return false;
    }

    if ($uid == -1) {
        $uid = SessionUtil::getVar('uid');
    }
    if (empty($uid)) {
        return false;
    }

    if (pnUserFieldAlias($name)) {
        // this value comes from the users table
        $obj = array('uid' => $uid,
                     $name => $value);
        return (bool)DBUtil::updateObject($obj, 'users', '', 'uid');
    }

    // decide if we have to use the (obsolete) DUDs from the Profile module
    $profileModule = pnConfigGetVar('profilemodule', '');
    $useOldProfileDUD = (!empty($profileModule) && $profileModule == 'Profile' && pnModAvailable($profileModule));
    if (!$useOldProfileDUD) {
        // no dynamic data, we can finish here
        return true;
    }

    $propertiestable = $pntable['user_property'];
    $datatable = $pntable['user_data'];
    $propcolumns = $pntable['user_property_column'];
    $datacolumns = $pntable['user_data_column'];

    $name = pnUserDynamicAlias($name);

    // Confirm that this is a known value
    $property = DBUtil::selectObjectByID('user_property', $name, 'prop_label');

    if (!$property) {
        return false;
    }

    $id = $property['prop_id'];
    $type = $property['prop_dtype'];
    $validation = unserialize($property['prop_validation']);

    // Combining fields, TODO: Extend to other types than only EXTDATE
    // Must check type, if EXTDATE { implode } else { serialize }
    if (is_array($value)) {
        if ($validation['displaytype'] == 6) {
            $value = implode('-', $value);
        } else {
            $value = serialize(array_values($value));
        }
    }

    // check for existence of the variable in user data
    $where = "WHERE $datacolumns[uda_propid] = '" . DataUtil::formatForStore($id) . "'
              AND   $datacolumns[uda_uid] = '" . DataUtil::formatForStore($uid) . "'";
    $userdata = DBUtil::selectObject('user_data', $where);

    // jgm - this won't work in databases that care about typing
    // but this should get fixed when we move to the dynamic user
    // variables setup
    // TODO: do some checking with $type to maybe do conditional sql
    if (!$userdata) {
        // record does not exist
        $obj = array('uda_propid' => $id,
                     'uda_uid'    => $uid,
                     'uda_value'  => $value);
        $result = DBUtil::insertObject($obj, 'user_data', 'uda_id');
    } else {
        // existing record
        $obj = array('uda_value' => $value);
        $result = DBUtil::updateObject($obj, 'user_data', $where);
    }

    return (bool)$result;
}


function pnUserSetPassword($pass)
{
    $method = pnModGetVar('Users', 'hash_method');
    $hashmethodsarray = pnModAPIFunc('Users', 'user', 'gethashmethods');
    pnUserSetVar('pass', DataUtil::hash($pass, $method));
    pnUserSetVar('hash_method', $hashmethodsarray[$method]);
}


/**
 * delete the contents of a user variable
 *
 * @access public
 * @author Gregor J. Rothfuss
 * @since 1.23 - 2002/02/01
 * @param name $ the name of the variable
 * @param uid $ the user to delete the variable for
 * @return string true on success, false on failure
 */
function pnUserDelVar($name, $uid = -1)
{
    $pntable = pnDBGetTables();

    // Prevent deletion of core fields (duh)
    if (empty($name) || ($name == 'uid') || ($name == 'email') ||
       ($name == 'password') || ($name == 'uname')) {
        return false;
    }

    if ($uid == -1) {
        $uid = SessionUtil::getVar('uid');
    }
    if (empty($uid)) {
        return false;
    }

    if (pnUserFieldAlias($name)) {
        // this value comes from the users table
        $obj = array('uid' => $uid,
                     $name => '');
        return (bool)DBUtil::updateObject($obj, 'users', '', 'uid');
    }

    // decide if we have to use the (obsolete) DUDs from the Profile module
    $profileModule = pnConfigGetVar('profilemodule', '');
    $useOldProfileDUD = (!empty($profileModule) && $profileModule == 'Profile' && pnModAvailable($profileModule));
    if (!$useOldProfileDUD) {
        return true;
    }

    $datacolumns = $tables['user_data_column'];

    // get property id for cascading delete later
    $result = DBUtil::selectObjectByID('user_property', $name, 'prop_label');
    if ($result) {
        $id = $result['prop_id'];
        // delete variable from user data for all users
        $where = "WHERE $datacolumns[uda_propid] = '" . DataUtil::formatForStore($id)  . "'
                  AND   $datacolumns[uda_uid]    = '" . DataUtil::formatForStore($uid) . "'";
        $result = DBUtil::deleteWhere('user_data', $where);
    }

    return (bool)$result;
}

/**
 * get the user's theme
 * <br />
 * This function will return the current theme for the user.
 * Order of theme priority:
 *  - page-specific
 *  - category
 *  - user
 *  - system
 *
 * @public
 * @return string the name of the user's theme
 **/
function pnUserGetTheme($force = false)
{
    static $theme;
    if (isset($theme) && !$force) {
        return $theme;
    }

    // Page-specific theme
    $pagetheme = FormUtil::getPassedValue('theme', null, 'GETPOST');
    $type      = FormUtil::getPassedValue('type', null, 'GETPOST');
    $qstring   = pnServerGetVar('QUERY_STRING');
    if (!empty($pagetheme)) {
        $themeinfo = ThemeUtil::getInfo(ThemeUtil::getIDFromName($pagetheme));
        if ($themeinfo['state'] == PNTHEME_STATE_ACTIVE &&
             ($themeinfo['user'] || $themeinfo['system'] ||
              ($themeinfo['admin'] && ($type == 'admin' || stristr($qstring, 'admin.php'))))
            && is_dir('themes/'.DataUtil::formatForOS($themeinfo['directory']))) {
            $theme = $themeinfo['name'];
            return $themeinfo['name'];
        }
    }

    // check for an admin theme
    if (($type == 'admin' || stristr($qstring, 'admin.php')) && SecurityUtil::checkPermission('::', '::', ACCESS_EDIT)) {
        $admintheme = pnModGetVar('Admin', 'admintheme');
        if (!empty($admintheme)) {
            $themeinfo = ThemeUtil::getInfo(ThemeUtil::getIDFromName($admintheme));
            if ($themeinfo && $themeinfo['state'] == PNTHEME_STATE_ACTIVE && is_dir('themes/'.DataUtil::formatForOS($themeinfo['directory']))) {
                $theme = $themeinfo['name'];
                return $themeinfo['name'];
            }
        }
    }

    // set a new theme for the user
    $newtheme = FormUtil::getPassedValue('newtheme', null, 'GETPOST');
    if (!empty($newtheme) && pnConfigGetVar('theme_change')) {
        $themeinfo = ThemeUtil::getInfo(ThemeUtil::getIDFromName($newtheme));
        if ($themeinfo && $themeinfo['state'] == PNTHEME_STATE_ACTIVE && is_dir('themes/'.DataUtil::formatForOS($themeinfo['directory']))) {
            if (pnUserLoggedIn()) {
                pnUserSetVar('theme', $newtheme);
            } else {
                SessionUtil::setVar('theme', $newtheme);
            }
            $theme = $themeinfo['name'];
            return $themeinfo['name'];
        }
    }

    // User theme
    if (pnConfigGetVar('theme_change')) {
        if ((pnUserLoggedIn())) {
            $usertheme = pnUserGetVar('theme');
        } else {
            $usertheme = SessionUtil::getVar('theme');
        }
        $themeinfo = ThemeUtil::getInfo(ThemeUtil::getIDFromName($usertheme));
        if ($themeinfo && $themeinfo['state'] == PNTHEME_STATE_ACTIVE && is_dir('themes/'.DataUtil::formatForOS($themeinfo['directory']))) {
            $theme = $themeinfo['name'];
            return $themeinfo['name'];
        }
    }

    // default site theme
    $defaulttheme = pnConfigGetVar('Default_Theme');
    $themeinfo = ThemeUtil::getInfo(ThemeUtil::getIDFromName($defaulttheme));
    if ($themeinfo && $themeinfo['state'] == PNTHEME_STATE_ACTIVE && is_dir('themes/'.DataUtil::formatForOS($themeinfo['directory']))) {
        $theme = $themeinfo['name'];
        return $themeinfo['name'];
    }

    $theme = 'ExtraLite';
    return $theme;
}

/**
 * get the user's language
 *
 * @public <br>
 * jgm - the language parameter should be a user variable, not a
 *        session variable
 * @return string the name of the user's language
 */
function pnUserGetLang()
{
    $lang = SessionUtil::getVar('lang');
    if (!empty($lang)) {
        return $lang;
    }

    return pnConfigGetVar('language');
}

/**
 * get the options for commenting
 * <br>
 * This function is deprecated, use <code>pnUserGetcommentArray()</code> in
 * conjunction with <code>pnModURL()</code> to produce relevant URLs
 *
 * @deprecated
 * @public
 * @return string the comment options string
 */
function pnUserGetCommentOptions($implode=true)
{
    if (pnUserLoggedIn()) {
        $mode = pnUserGetVar('umode');
        $order = pnUserGetVar('uorder');
        $thold = pnUserGetVar('thold');
    }

    if (empty($mode)) {
        $mode = 'thread';
    }

    if (empty($order)) {
        $order = 0;
    }

    if (empty($thold)) {
        $thold = 0;
    }

    if ($implode) {
        return("mode=$mode&amp;order=$order&amp;thold=$thold");
    }

    $array = array('mode' => $mode, 'order' => $order, 'thold' => $thold);
    return $array;
}

/**
 * get the options for commenting
 *
 * @public
 * @return array the comment options array
 */
function pnUserGetCommentOptionsArray()
{
    if (pnUserLoggedIn()) {
        $mode = pnUserGetVar('umode');
        $order = pnUserGetVar('uorder');
        $thold = pnUserGetVar('thold');
    }

    if (empty($mode)) {
        $mode = 'thread';
    }

    if (empty($order)) {
    $order = 0;
    }

    if (empty($thold)) {
        $thold = 0;
    }

    $array = array('mode' => $mode, 'order' => $order, 'thold' => $thold);
    return $array;
}

/**
 * get a list of user information
 *
 * @public
 * @return array array of user arrays
 */
function pnUserGetAll($sortby = 'uname', $sortorder = 'ASC', $limit = 0, $startnum = 1, $activated = '', $regexpfield = '', $regexpression ='')
{
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // decide if we have to load the (obsolete) DUDs from the Profile module
    $profileModule = pnConfigGetVar('profilemodule', '');
    $useOldProfileDUD = (!empty($profileModule) && $profileModule == 'Profile' && pnModAvailable($profileModule));

    pnModDBInfoLoad('Users', 'Users');
    $userstable  =  $pntable['users'];
    $userscolumn = &$pntable['users_column'];

    // SQL part storing the DUD parts
    $dynsql = '';

    if ($useOldProfileDUD) {
        pnModDBInfoLoad('Profile', 'Profile');
        $propcolumn  = $pntable['user_property_column'];
        $datacolumn  = $pntable['user_data_column'];

        $where = "$propcolumn[prop_dtype] >= '1'";
        $sort  = "$propcolumn[prop_weight] ASC";
        $props = DBUtil::selectObjectArray ('user_property', $where, $sort);

        foreach ($props as $prop) {
            $dynsql .= "MAX(IF($datacolumn[uda_propid] = '$prop[prop_id]', $datacolumn[uda_value], NULL)) as '$prop[prop_label]', ";
        }
    }

    // needs changes before being converted to DBUtil
    $sql = "SELECT     $dynsql
                       $userscolumn[uid]             as uid,
                       $userscolumn[uname]           as uname,
                       $userscolumn[email]           as email,
                       $userscolumn[storynum]        as storynum,
                       $userscolumn[theme]           as theme,
                       $userscolumn[user_theme]      as user_theme,
                       $userscolumn[ublock]          as ublock,
                       $userscolumn[ublockon]        as ublockon,
                       $userscolumn[counter]         as counter,
                       $userscolumn[activated]       as activated,
                       $userscolumn[lastlogin]       as lastlogin,
                       $userscolumn[validfrom]       as validfrom,
                       $userscolumn[validuntil]      as validuntil,
                       $userscolumn[hash_method]     as hash_method
            FROM       $pntable[users] ";
    if ($useOldProfileDUD) {
        $sql .= "
            RIGHT JOIN $pntable[user_data]
            ON         $datacolumn[uda_uid] = $userscolumn[uid] ";
    }

    $where = '';
    if (!empty($regexpfield) && (array_key_exists($regexpfield, $userscolumn)) && !empty($regexpression)) {
        $where = 'WHERE ' . $userscolumn[$regexpfield]. ' REGEXP "' . DataUtil::formatForStore($regexpression) . '"';
    }
    if (!empty($activated) && is_numeric($activated) && array_key_exists('activated', $userscolumn)) {
        if (!empty($where)) {
            $where .= ' AND ';
        } else {
            $where = ' WHERE ';
        }
        $where .= "$userscolumn[activated] != '" . DataUtil::formatForStore($activated) . "'";
    }

    $groupby = " GROUP BY $userscolumn[uname] ";

    $sort = '';
    if (!empty($sortby)) {
        if (array_key_exists($sortby, $userscolumn)) {
            $sort = "ORDER BY $userscolumn[$sortby] " . $sortorder; //sorty by .....
        } else {
            $sort = "ORDER BY $sortby " . $sortorder; //sorty by dynamic.....
        }
        if ($sortby != 'uname') {
            $sort .= ", $userscolumn[uname] ASC ";
        }
    }

    $sql = $sql . $where . $groupby . $sort;

    if (is_numeric($limit) && $limit != 0) {
        $result = $dbconn->SelectLimit($sql, $numitems, $startnum - 1);
    } else {
        $result = $dbconn->Execute($sql);
    }

    if ($dbconn->ErrorNo() != 0) {
        return;
    }

    if ($result->EOF) {
        $return = false;
        return $return;
    }

    $resarray = array();
    $vars = array('_UREALNAME'      => 'name',
                  '_UREALEMAIL'     => 'email',
                  '_UFAKEMAIL'      => 'femail',
                  '_YOURHOMEPAGE'   => 'url',
                  '_TIMEZONEOFFSET' => 'timezone_offset',
                  '_YOURAVATAR'     => 'user_avatar',
                  '_YICQ'           => 'user_icq',
                  '_YAIM'           => 'user_aim',
                  '_YYIM'           => 'user_yim',
                  '_YMSNM'          => 'user_msnm',
                  '_YLOCATION'      => 'user_from',
                  '_YOCCUPATION'    => 'user_occ',
                  '_YINTERESTS'     => 'user_intrest',
                  '_SIGNATURE'      => 'user_sig',
                  '_EXTRAINFO'      => 'bio');

    for(; !$result->EOF; $result->MoveNext()) {

        $item = $result->GetRowAssoc(2); // WARNING: NEED 2 = user-defined fieldnames
        foreach($item as $key => $value) {
            if (array_key_exists($key, $vars)) {
                $item[$vars[$key]] = $value;
            }
        }

        $resarray[$item['uid']] = $item;

    }

    $result->Close();

    return $resarray;

}

/**
 * Get the uid of a user from the username
 *
 * @access public
 * @author Michael Halbrook
 * @since 1.18 - 19/04/2004
 * @param uname $ the username
 * @return mixed userid if found, false if not
 */
function pnUserGetIDFromName($uname)
{
    if (empty($uname)) {
        return false;
    }

    $result = DBUtil::selectObjectByID('users', $uname, 'uname');
    return (isset($result['uid']) ? $result['uid'] : false);
}

/**
 * Checks the alias and returns if we save the data in the
 * Profile module's user_data table or the users table.
 * This should be removed if we ever go fully dynamic
 *
 * @access private
 * @author F. Chestnut
 * @since 1.26 - 19/04/2004
 * @param label $ the alias of the field to check
 * @return true if found, false if not, void upon error
 */
function pnUserFieldAlias($label)
{

    if (empty($label)) {
        return false;
    }

    $vars = array('email',
                  'user_regdate',
                  'user_viewemail',
                  'user_theme',
                  'pass',
                  'storynum',
                  'umode',
                  'uorder',
                  'thold',
                  'noscore',
                  'ublockon',
                  'ublock',
                  'theme',
                  'commentmax',
                  'counter',
                  'activated',
                  'lastlogin',
                  'validfrom',
                  'validuntil',
                  'hash_method');

    return in_array($label, $vars);
}

/** TEMPORARY FIX
 * Checks the alias and returns PROP_ID.
 *
 * @access private
 * @author F. Chestnut
 * @since 1.26 - 19/04/2004
 * @param label $ the alias of the field to check
 * @return true if found, false if not, void upon error
 */
function pnUserDynamicAlias($label)
{

    if (empty($label)) {
        return false;
    }

    $vars = array('name'            => '_UREALNAME',
                  'email'           => '_UREALEMAIL',
                  'femail'          => '_UFAKEMAIL',
                  'url'             => '_YOURHOMEPAGE',
                  'timezone_offset' => '_TIMEZONEOFFSET',
                  'user_avatar'     => '_YOURAVATAR',
                  'user_icq'        => '_YICQ',
                  'user_aim'        => '_YAIM',
                  'user_yim'        => '_YYIM',
                  'user_msnm'       => '_YMSNM',
                  'user_from'       => '_YLOCATION',
                  'user_occ'        => '_YOCCUPATION',
                  'user_intrest'    => '_YINTERESTS',
                  'user_sig'        => '_SIGNATURE',
                  'bio'             => '_EXTRAINFO');

    if (array_key_exists($label, $vars)) {
        return $vars[$label];
    }

    return $label;

}

/* Checks the PROP_ID and returns the alias.
 *
 * @access private
 * @author F. Chestnut
 * @since 1.26 - 19/04/2004
 * @param label $ the alias of the field to check
 * @return true if found, false if not, void upon error
 */
function pnUserAliasFromID($propid='')
{
    if (empty($propid)) {
        return false;
    }

    $vars = array('_UREALNAME'      => 'name',
                  '_UREALEMAIL'     => 'email',
                  '_UFAKEMAIL'      => 'femail',
                  '_YOURHOMEPAGE'   => 'url',
                  '_TIMEZONEOFFSET' => 'timezone_offset',
                  '_YOURAVATAR'     => 'user_avatar',
                  '_YICQ'           => 'user_icq',
                  '_YAIM'           => 'user_aim',
                  '_YYIM'           => 'user_yim',
                  '_YMSNM'          => 'user_msnm',
                  '_YLOCATION'      => 'user_from',
                  '_YOCCUPATION'    => 'user_occ',
                  '_YINTERESTS'     => 'user_intrest',
                  '_SIGNATURE'      => 'user_sig',
                  '_EXTRAINFO'      => 'bio');

    if (array_key_exists($propid, $vars)) {
        return $vars[$propid];
    }

    return false;

}
