<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id:$
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Profile
 */
 
// Ported from : i-Block [Featured User 1.5] - Alexander Graef aka MagicX - http://www.portalzine.de 

/**
 * initialise block
 *
 * @author       The Zikula Development Team
 */
function Profile_featureduserblock_init()
{
    // Security
    pnSecAddSchema('Profile:FeaturedUserblock:', 'Block title::');
}

/**
 * get information on block
 *
 * @author       The Zikula Development Team
 * @return       array       The block information
 */
function Profile_featureduserblock_info()
{
    return array('text_type'       => DataUtil::formatForDisplay(_PROFILE_FEATUREDUSER_BLOCKNAME),
                 'text_type_long'  => DataUtil::formatForDisplay(_PROFILE_FEATUREDUSER_BLOCKDESC),
                 'module'          => 'Profile',
                 'allow_multiple'  => true,
                 'form_content'    => false,
                 'form_refresh'    => false,
                 'show_preview'    => true,
                 'admin_tableless' => true);
}

/**
 * display block
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the rendered bock
 */
function Profile_featureduserblock_display($blockinfo)
{
    // Check if the Profile module is available.
    if (!pnModAvailable('Profile')) {
        return false;
    }

    // Security check
    if (!SecurityUtil::checkPermission('Profile:FeaturedUserblock:', "$blockinfo[title]::", ACCESS_READ)) {
        return false;
    }

    // Get variables from content block
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    $pnRender = pnRender::getInstance('Profile');

    // get all active profile fields
    $activeduds  = pnModAPIfunc('Profile', 'memberslist', 'getallactive');
    foreach ($activeduds as $activedud) {
        $dudarray[] = $activedud['prop_label'];
    }
    $pnRender->assign('dudarray',  $dudarray);

    $pnRender->cacheid = $vars['username'];
    $pnRender->assign(pnUserGetVars(pnUserGetIDFromName($vars['username'])));
    
    $blockinfo['content'] = $pnRender->fetch('profile_block_featureduser.htm');

    return pnBlockThemeBlock($blockinfo);
}

/**
 * modify block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the bock form
 */
function Profile_featureduserblock_modify($blockinfo)
{
    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['username'])) {
        $vars['username'] = '';
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile', false);

    // assign the approriate values
    $pnRender->assign('username', $vars['username']);

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_block_featureduser_modify.htm');
}


/**
 * update block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       $blockinfo  the modified blockinfo structure
 */
function Profile_featureduserblock_update($blockinfo)
{
    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // alter the corresponding variable
    $vars['username'] = FormUtil::getPassedValue('username', null, 'REQUEST');

    // write back the new contents
    $blockinfo['content'] = pnBlockVarsToContent($vars);

    // clear the block cache
    $pnRender = pnRender::getInstance('Profile');
    $pnRender->clear_cache('profile_block_featureduser.htm');

    return $blockinfo;
}
