<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: membersonline.php 24733 2008-10-17 11:44:47Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Profile
 */

// Ported from : i-Block [Members Online 2.5] - MagicX - Portalzine.de

/**
 * initialise block
 *
 * @author       The Zikula Development Team
 */
function Profile_membersonlineblock_init()
{
    // Security
    pnSecAddSchema('Profile:MembersOnlineblock:', 'Block title::');
}

/**
 * get information on block
 *
 * @author       The Zikula Development Team
 * @return       array       The block information
 */
function Profile_membersonlineblock_info()
{
    return array('text_type'       => DataUtil::formatForDisplay(_PROFILE_MEMBERSONLINE_BLOCKNAME),
                 'text_type_long'  => DataUtil::formatForDisplay(_PROFILE_MEMBERSONLINE_BLOCKDESC),
                 'module'          => 'Profile',
                 'allow_multiple'  => true,
                 'form_content'    => false,
                 'form_refresh'    => false,
                 'show_preview'    => true,
                 'admin_tableless' => true);
}

/**
 * display block
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the rendered bock
 */
function Profile_membersonlineblock_display($blockinfo)
{
    // Check if the Profile module is available.
    if (!pnModAvailable('Profile')) {
        return false;
    }

    // Security check
    if (!SecurityUtil::checkPermission('Profile:MembersOnlineblock:', "$blockinfo[title]::", ACCESS_READ)) {
        return false;
    }

    // Get variables from content block
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['lengthmax'])) {
        $vars['lengthmax'] = 30;
    }

    $membonline = 0;
    $anononline = 0;
    $uid = pnUserGetVar('uid');

    $users = pnModAPIFunc('Profile', 'memberslist', 'getallonline');

    $usersonline = array();

    if ($users) {
        foreach($users['unames'] as $user) {
            if ($user['uid'] <> 1) {
                if (strlen($user['uname']) > $vars['lengthmax']) {
                    $user['unameshort'] = substr($user['uname'], 0, $vars['lengthmax']-3) . '...';
                } else {
                    $user['unameshort'] = $user['uname'];
                }
            }
            $usersonline[] = $user;
        }
    }

    $pnRender = pnRender::getInstance('Profile', false);

    // check which messaging module is available and add the necessary info
    $msgmodule = pnModAPIFunc('Profile', 'memberslist', 'getmessagingmodule');
    $pnRender->assign('msgmodule', $msgmodule);
    if (!empty($msgmodule) && pnUserLoggedIn()) {
        $pnRender->assign('messages', pnModAPIFunc($msgmodule, 'user', 'getmessagecount'));
    }

    $pnRender->assign('usersonline',       $usersonline);
    $pnRender->assign('membonline',        $users['numusers']);
    $pnRender->assign('anononline',        $users['numguests']);
    $pnRender->assign('uid',               $uid);
    $pnRender->assign('anonymoussessions', pnConfigGetVar('anonymoussessions'));

    $blockinfo['content'] = $pnRender->fetch('profile_block_membersonline.htm', $membonline . $anononline);

    return pnBlockThemeBlock($blockinfo);
}

/**
 * modify block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       output      the bock form
 */
function Profile_membersonlineblock_modify($blockinfo)
{
    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['lengthmax'])) {
        $vars['lengthmax'] = 30;
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile', false);

    // assign the approriate values
    $pnRender->assign('lengthmax', $vars['lengthmax']);

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_block_membersonline_modify.htm');
}


/**
 * update block settings
 *
 * @author       The Zikula Development Team
 * @param        array       $blockinfo     a blockinfo structure
 * @return       $blockinfo  the modified blockinfo structure
 */
function Profile_membersonlineblock_update($blockinfo)
{
    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // alter the corresponding variable
    $vars['lengthmax'] = (int)FormUtil::getPassedValue('lengthmax', null, 'REQUEST');

    // write back the new contents
    $blockinfo['content'] = pnBlockVarsToContent($vars);

    // clear the block cache
    $pnRender = pnRender::getInstance('Profile');
    $pnRender->clear_cache('profile_block_membersonline.htm');

    return $blockinfo;
}
