<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnmemberslistapi.php 24733 2008-10-17 11:44:47Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Profile
 */

/**
 * Get all users
 * This API function returns all users ids. This function allows for
 * filtering and for paged selection
 * 
 * @author Mark West
 * @param 'startnum' start number for recordset
 * @param 'numitems' number of items to return
 * @param 'letter' letter to filter by
 * @param 'sortby' attribute to sort by
 * @param 'sortorder' sort order ascending/descending
 * @return array matching user ids
 */
function Profile_memberslistapi_getall($args)
{
    // Optional arguments.
    if (!isset($args['startnum'])) {
        $args['startnum'] = 1;
    }
    if (!isset($args['numitems'])) {
        $args['numitems'] = -1;
    }
    if (!isset($args['sortby'])) {
        $args['sortby'] = 'uname';
    }
    if (!isset($args['sortorder']) || empty($args['sortorder'])) {
        $args['sortorder'] = 'ASC';
    }
    if (!isset($args['sorting']) || empty($args['sorting'])) {
        $args['sorting'] = 0;
    }
    if (!isset($args['searchby']) || empty($args['searchby'])) {
        $args['searchby'] = 'uname';
    }

    // define the array to hold the result items
    $items = array();

    // Security check
    if (!SecurityUtil::checkPermission('Profile:Members:', '::', ACCESS_READ)) {
        return $items;
    }

    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // load the database information for the users module
    pnModDBInfoLoad('Users');

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $userscolumn = $pntable['users_column'];
    $datacolumn  = $pntable['user_data_column'];

    // Get items
    $sql = "SELECT     $userscolumn[uid] as uid
            FROM       $pntable[user_data]
            RIGHT JOIN $pntable[users]
            ON         $datacolumn[uda_uid] = $userscolumn[uid] ";

    // treat a single character as from the alpha filter and everything else as from the search input
    if (strlen($args['letter']) > 1) {
        $args['letter'] = "%{$args['letter']}";
    }

    if ($args['searchby'] == 'uname'){
        if (isset($args['letter']) && eregi("[a-zA-Z]", $args['letter'])) {
            // are we listing all or "other" ?
            $where = "WHERE UPPER($userscolumn[uname]) LIKE UPPER('" . DataUtil::formatForStore($args['letter']) . "%') AND $userscolumn[uid] != '1' ";
            // I guess we are not..
        } else if (isset($args['letter'])) {
            // But other is numbers ?
            $where = "WHERE ($userscolumn[uname] LIKE '0%'
                       OR $userscolumn[uname] LIKE '1%'
                       OR $userscolumn[uname] LIKE '2%'
                       OR $userscolumn[uname] LIKE '3%'
                       OR $userscolumn[uname] LIKE '4%'
                       OR $userscolumn[uname] LIKE '5%'
                       OR $userscolumn[uname] LIKE '6%'
                       OR $userscolumn[uname] LIKE '7%'
                       OR $userscolumn[uname] LIKE '8%'
                       OR $userscolumn[uname] LIKE '9%'
                       OR $userscolumn[uname] LIKE '-%'
                       OR $userscolumn[uname] LIKE '.%'
                       OR $userscolumn[uname] LIKE '@%'
                       OR $userscolumn[uname] LIKE '$%') ";
            // fifers: while this is not the most eloquent solution, it is
            // cross database compatible.  We could do an if dbtype is mysql
            // then do the regexp.  consider for performance enhancement.
            //
            // "WHERE $column[uname] REGEXP \"^\[1-9]\" "
            // REGEX :D, although i think its MySQL only
            // Will have to change this later.
            // if you know a better way to match only the first char
            // to be a number in uname, please change it and email
            // sweede@gallatinriver.net the correction
            // or go to post-nuke project page and post
            // your correction there. Thanks, Bjorn.
        } else { // or we are unknown or all..
            $where = "WHERE $userscolumn[uid] != '1' "; // this is to get rid of anoying "undefinied variable" message
        }
    } else if ($args['searchby'] == 'name') {
        $where = "WHERE ".$datacolumn['uda_propid']."=".$realname_prop_id." AND ".$datacolumn['uda_value']." LIKE '$startbyjocker" . DataUtil::formatForStore($args['letter'])."%' ";        
    } else if ($args['searchby'] == 'url') {
        $where = "WHERE ".$datacolumn['uda_propid']."=".$homepage_prop_id." AND ".$datacolumn['uda_value']." LIKE '$startbyjocker".DataUtil::formatForStore($args['letter'])."%' ";        
    } else {
        $where = "WHERE ".$userscolumn[DataUtil::formatForStore($args['searchby'])]." LIKE '$startbyjocker".DataUtil::formatForStore($args['letter'])."%' ";
    }

    if (!$args['sorting'] && pnModGetVar('Profile', 'filterunverified')) {
        $where .= "AND $userscolumn[activated] != '0'";
    }

    $groupby = " GROUP BY $userscolumn[uname] ";

    if (array_key_exists($args['sortby'], $userscolumn)) {
        $sort = 'ORDER BY ' . DataUtil::formatForStore($userscolumn[$args['sortby']]) . ' ' . $args['sortorder']; //sort by .....
    } else {
        $sort = "ORDER BY $args[sortby] " . $args['sortorder']; //sort by dynamic.....
    }
    if ($args['sortby'] != 'uname') {
        $sort .= ", $userscolumn[uname] ASC ";
    }

    $sql .= $where . $groupby . $sort;

    $result = $dbconn->SelectLimit($sql, $args['numitems'], $args['startnum']-1);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Put items into result array
    for (; !$result->EOF; $result->MoveNext()) {
        list($uid) = $result->fields;
        if (SecurityUtil::checkPermission('Profile:Members:', "::", ACCESS_READ)) {
            $items[$uid] = pnUserGetVars($uid);
        }
    }

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the items
    return $items;
}

/**
 * Utility function to count the number of users
 * This function allows for filtering by letter
 * 
 * @author Mark West
 * @param 'letter' letter to filter by
 * @return integer count of matching users
 */
function Profile_memberslistapi_countitems($args)
{
    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // load the database information for the users module
    pnModDBInfoLoad('Users');

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $userscolumn = $pntable['users_column'];

    // Get item
    $select = "SELECT COUNT(1) FROM $pntable[users] ";
    if (isset($args['letter']) && !empty($args['letter'])) {
        // are we listing all or "other" ?
        $where = "WHERE UPPER($userscolumn[uname]) LIKE UPPER('".DataUtil::formatForStore($args['letter'])."%') AND $userscolumn[uname] NOT LIKE 'Anonymous' ";
        // I guess we are not..
    } else if (isset($args['letter'])) {
        $where = "WHERE ($userscolumn[uname] LIKE '0%'
                  OR $userscolumn[uname] LIKE '1%'
                  OR $userscolumn[uname] LIKE '2%'
                  OR $userscolumn[uname] LIKE '3%'
                  OR $userscolumn[uname] LIKE '4%'
                  OR $userscolumn[uname] LIKE '5%'
                  OR $userscolumn[uname] LIKE '6%'
                  OR $userscolumn[uname] LIKE '7%'
                  OR $userscolumn[uname] LIKE '8%'
                  OR $userscolumn[uname] LIKE '9%'
                  OR $userscolumn[uname] LIKE '-%'
                  OR $userscolumn[uname] LIKE '.%'
                  OR $userscolumn[uname] LIKE '@%'
                  OR $userscolumn[uname] LIKE '$%') ";
    } else {
        $where = "WHERE $userscolumn[uid] != '1'";
    }

    if (pnModGetVar('Profile', 'filterunverified')) {
        $where .= " AND $userscolumn[activated] != '0'";
    }

    $sql = $select . $where;
    $result = $dbconn->Execute($sql);

    // Check for an error with the database code
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Obtain the number of items
    list($numitems) = $result->fields;

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the number of items
    return $numitems;
}

/**
 * Utility function to count the number of users online
 * 
 * @author Mark West
 * @return integer count of registered users online
 */
function Profile_memberslistapi_getregisteredonline($args)
{
    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $sessioninfocolumn = $pntable['session_info_column'];
    $sessioninfotable = $pntable['session_info'];

    $activetime = adodb_strftime('%Y-%m-%d %H:%M:%S', time() - (pnConfigGetVar('secinactivemins') * 60));

    // Get items
    $sql = "SELECT DISTINCT $sessioninfocolumn[uid] FROM $sessioninfotable
            WHERE $sessioninfocolumn[uid] != 0 AND $sessioninfocolumn[lastused] > '$activetime'
            GROUP BY $sessioninfocolumn[uid]";
    $result = $dbconn->Execute($sql);

     // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Obtain the number of items
    $numusers = $result->RecordCount();

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the number of items
    return $numusers;
}

/**
 * Utility function get the latest registered user
 * 
 * @author Mark West
 * @return integer latest registered user id
 */
function Profile_memberslistapi_getlatestuser($args)
{
    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // load the database information for the users module
    pnModDBInfoLoad('Users');

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $userscolumn = $pntable['users_column'];

    // filter out unverified users
    $where = '';
    if (pnModGetVar('Profile', 'filterunverified')) {
         $where = " AND $userscolumn[activated] = '1'";
    }

    // Get items
    $sql = "SELECT $userscolumn[uid]
            FROM $pntable[users]
            WHERE $userscolumn[uname] NOT LIKE 'Anonymous' $where
            ORDER BY $userscolumn[uid] DESC";
    $result = $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Obtain the number of items
    list($lastuser) = $result->fields;

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the number of items
    return $lastuser;

}

/**
 * Utility function to decide if a user is online
 * 
 * @author Mark West
 * @return bool true if online, false otherwise
 */
function Profile_memberslistapi_isonline($args)
{
    // check arguments
    if (!isset($args['userid']) || empty($args['userid']) || !is_numeric($args['userid'])) {
        return false;
    }

    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // get active time based on security settings
    $activetime = adodb_strftime('%Y-%m-%d %H:%M:%S', time() - (pnConfigGetVar('secinactivemins') * 60));

    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $sessioninfocolumn = $pntable['session_info_column'];
    $sessioninfotable = $pntable['session_info'];

    // Get items
    $sql = "SELECT DISTINCT $sessioninfocolumn[uid]
            FROM $sessioninfotable
            WHERE $sessioninfocolumn[uid] = $args[userid] and $sessioninfocolumn[lastused] > '$activetime'";
    $result = $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Obtain the item
    list($online) = $result->fields;

    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return if the user is online
    if ($online > 0) {
        return true;
    } else {
        return false;
    }

}

/**
 * Utility function return registered users online
 * 
 * @author Mark West
 * @return array registered users
 */
function Profile_memberslistapi_whosonline($args)
{
    // Get database setup
    $dbconn = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // define the array to hold the resultant items
    $items = array();
    // It's good practice to name the table and column definitions you are
    // getting - $table and $column don't cut it in more complex modules
    $sessioninfocolumn = $pntable['session_info_column'];
    $sessioninfotable = $pntable['session_info'];

    // get active time based on security settings
    $activetime = adodb_strftime('%Y-%m-%d %H:%M:%S', time() - (pnConfigGetVar('secinactivemins') * 60));

    // Get items
    $sql = "SELECT DISTINCT $sessioninfocolumn[uid]
            FROM $sessioninfotable
            WHERE $sessioninfocolumn[uid] != 0
            AND $sessioninfocolumn[lastused] > '$activetime'
            GROUP BY $sessioninfocolumn[uid]";
    $result = $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    // Obtain the number of items
    list($numitems) = $result->fields;

    // Put items into result array.
    for (; !$result->EOF; $result->MoveNext()) {
        list($uid) = $result->fields;
        $items[$uid] = pnUserGetVars($uid);
    }
    // All successful database queries produce a result set, and that result
    // set should be closed when it has been finished with
    $result->Close();

    // Return the items
    return $items;
}

/**
 * Utility function returns all users online
 * 
 * @author Frank Chestnut
 * @return array All online visitors (including anonymous)
 */
function Profile_memberslistapi_getallonline($args)
{
    // Get database setup
    $dbconn  = pnDBGetConn(true);
    $pntable = pnDBGetTables();

    // define the array to hold the resultant items
    $items = array();

    $sessioninfotable  = $pntable['session_info'];
    $sessioninfocolumn = &$pntable['session_info_column'];
    $usertbl           = $pntable['users'];
    $usercol           = &$pntable['users_column'];

    // get active time based on security settings
    $activetime = adodb_strftime('%Y-%m-%d %H:%M:%S', time() - (pnConfigGetVar('secinactivemins') * 60));

    // Check if anonymous session are on
    if (pnConfigGetVar('anonymoussessions')) {
        $anonwhere = "AND $sessioninfotable.$sessioninfocolumn[uid] >= '0' ";
    } else {
        $anonwhere = "AND $sessioninfotable.$sessioninfocolumn[uid] > '0'";
    }

    // Get items
    $sql = "SELECT   $sessioninfotable.$sessioninfocolumn[uid],
                     $usertbl.$usercol[uname]
            FROM     $sessioninfotable, $usertbl
            WHERE    $sessioninfocolumn[lastused] > '$activetime'
            $anonwhere
            AND      IF($sessioninfotable.$sessioninfocolumn[uid]='0','1',
                        $sessioninfotable.$sessioninfocolumn[uid]) = $usertbl.$usercol[uid]
            GROUP BY $sessioninfocolumn[ipaddr], $sessioninfotable.$sessioninfocolumn[uid]
            ORDER BY $usercol[uname]";

    $result = $dbconn->Execute($sql);

    // Check for an error with the database code, and if so set an appropriate
    // error message and return
    if ($dbconn->ErrorNo() != 0) {
        return LogUtil::registerError(_GETFAILED);
    }

    $numusers  = 0;
    $numguests = 0;
    $unames = array();
    for(; !$result->EOF; $result->MoveNext()) {
        list($uid, $uname) = $result->fields;

        if ($uid != 0) {
            $unames[] = array('uid'   => $uid,
                              'uname' => $uname);
            $numusers++;
        } else {
            $numguests++;
        }
    }

    $items = array('unames'    => $unames,
                   'numusers'  => $numusers,
                   'numguests' => $numguests,
                   'total'     => $numguests + $numusers);

    $result->Close();

    // Return the items
    return $items;
}

/**
 * Utility function to find out which messages module is installed
 * @author Frank Schmmertz
 * @return string name of the messaging module found, empty if none
 */
function Profile_memberslistapi_getmessagingmodule()
{
    $msgmodule = pnConfigGetVar('messagemodule', '');
    if (pnModAvailable($msgmodule)) {
        pnModLangLoad(msgmodule, 'user');
    } else {
        $msgmodule = '';
    }
    return $msgmodule;    
}
