<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnuser.php 24733 2008-10-17 11:44:47Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Profile
 */

/**
 * the main user function
 * This function is the default function, and is called whenever the module is
 * initiated without defining arguments.  As such it can be used for a number
 * of things, but most commonly it either just shows the module menu and
 * returns or calls whatever the module designer feels should be the default
 * function (often this is the view() function)
 * 
 * @author Mark West
 * @return string HTML string
 */
function Profile_user_main()
{
    // to do: change this comparison to '1.2.0' just before or after the release
    if (version_compare(PN_VERSION_NUM, '1.2.0-dev', '>=')) {
        return pnRedirect(pnModURL('Users', 'user', 'main'));
    }

    // Security check 
    if ((!pnUserLoggedIn()) || (!SecurityUtil::checkPermission('Profile::', '::', ACCESS_READ))) { 
        return LogUtil::registerPermissionError(); 
    } 
    
    // The API function is called. 
    $accountlinks = pnModAPIFunc('Profile', 'user', 'accountlinks'); 
    
    if ($accountlinks == false) { 
        return LogUtil::registerError(_NOITEMSFOUND, 404); 
    } 
    
    // Create output object 
    $pnRender = pnRender::getInstance('Profile', false, null, true); 
    
    // Assign the items to the template 
    $pnRender->assign('accountlinks', $accountlinks); 
    
    // Return the output that has been generated by this function 
    return $pnRender->fetch('profile_user_main.htm'); 
}

/**
 * display item
 * This is a standard function to provide detailed informtion on a single item
 * available from the module.
 * 
 * @author Mark West
 * @return string HTML string
 */
function Profile_user_view($args)
{
    // Get parameters from whatever input we need.
    $uname = FormUtil::getPassedValue('uname', isset($args['uname']) ? $args['uname'] : null, 'GET');
    $uid = (int)FormUtil::getPassedValue('uid', isset($args['uid']) ? $args['uid'] : null, 'GET');
    $page = FormUtil::getPassedValue('page', isset($args['page']) ? $args['page'] : null, 'GET');

    // Security check
    if (!SecurityUtil::checkPermission('Profile::', '::', ACCESS_READ)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile', false);

    // The API function is called.
    $items = pnModAPIFunc('Profile', 'user', 'getallactive');

    // Getting uid by uname
    if (!empty($uname)) {
        $uid = pnUserGetIDFromName($uname);
    } elseif (!empty($uid)) {
    } else {
        $uid = pnUserGetVar('uid');
    }

    // check for an invalid uid (uid = 1 is the anonymous user)
    if ($uid < 2) {
        return LogUtil::registerError(_PROFILE_UNINDENTIFIED, 404);
    }

    // Reducing sql by not using pnusergetvar for each field but working them all
    // at the same time.
    $userinfo = pnUserGetVars($uid);

    if (!$userinfo) {
        return LogUtil::registerError(_PROFILE_UNINDENTIFIED, 404);
    }

    // Check if the user is watching its own profile or if he is admin
    $currentuser = pnUserGetVar('uid');
    $isadmin     = false;
    $ismember    = false;
    $sameuser    = false;

    if ($currentuser >= 2) {
        $ismember = true;
    }
    if ($currentuser == $uid) {
        $sameuser = true;
    }
    if (SecurityUtil::checkPermission('Profile::', '::', ACCESS_ADMIN)) {
        $isadmin = true;
    }

    foreach($items as $field) {
        $alias = pnModAPIFunc('Profile', 'user', 'aliasing', array('label' => $field['prop_label']));

        if (isset($userinfo[$field['prop_label']]) && $alias != $field['prop_label']) {
            $userinfo[$alias] = $userinfo[$field['prop_label']];
        } elseif (isset($userinfo[$alias]) && $alias != $field['prop_label']) {
            $userinfo[$field['prop_label']] = $userinfo[$alias];
        }

        if ($field['prop_displaytype'] > 6) {
            $pnRender->assign($alias, unserialize($userinfo[$alias]));
        } else if (isset($userinfo[$alias])) {
            //$pnRender->assign($alias, @unserialize($userinfo[$alias]));
            $pnRender->assign($alias, $userinfo[$alias]);
        }
    }

    $userinfoTranslate = array();
    foreach ($userinfo as $key => $value){
        $userinfoTranslate[$key] =  defined($value) ? constant($value) : $value;
    }

    // get all active profile fields
    $activeduds  = pnModAPIfunc('Profile', 'user', 'getallactive');
    foreach ($activeduds as $activedud) {
        $dudarray[] = $activedud['prop_label'];
    }
    $pnRender->assign('dudarray',  $dudarray);

    $pnRender->assign('uname',    $userinfo['uname']);
    $pnRender->assign('fields',   $items);
    $pnRender->assign('userinfo', $userinfoTranslate);
    $pnRender->assign('ismember', $ismember);
    $pnRender->assign('isadmin',  $isadmin);
    $pnRender->assign('sameuser', $sameuser);

    // Return the output that has been generated by this function
    if (!empty($page)) {
        if ($pnRender->template_exists("profile_user_view_{$page}.htm")) {
            return $pnRender->fetch("profile_user_view_{$page}.htm", $uid);
        } else {
            return LogUtil::registerError(_PROFILE_UNINDENTIFIED, 404);
        }		
    } else {
        return $pnRender->fetch('profile_user_view.htm', $uid);
    }
}

/**
 * modify a users profile information
 *
 * @author Franky Chestnut
 */
function Profile_user_modify($args)
{
    // Security check
    if ((!pnUserLoggedIn()) || (!SecurityUtil::checkPermission('Profile::', '::', ACCESS_READ))) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile', false);

    // The API function is called.
    $items = pnModAPIFunc('Profile', 'user', 'getall');

    // The return value of the function is checked here
    if ($items == false) {
        return LogUtil::registerError(pnML('_GETITEMSFAILED', array('i' => _PROFILE_PROPERTIES)));
    }

    // check if we get called form the update function in case of an error
    $uname    = FormUtil::getPassedValue('uname',    (isset($args['uname']) ? $args['uname'] : null),    'GET');
    $errors   = FormUtil::getPassedValue('errors',   (isset($args['errors']) ? $args['errors'] : array()), 'GET');
    $dynadata = FormUtil::getPassedValue('dynadata', (isset($args['dynadata']) ? $args['dynadata'] : array()), 'GET');
    //merge this temporary dynadata and the errors into the items array
    foreach($dynadata as $proplabel => $propdata) {
        $items[$proplabel]['temp_propdata'] = $propdata;
        if (array_key_exists($proplabel, $errors)) {
            $items[$proplabel]['prop_error'] = $errors[$proplabel];
        }
    }

    // Assign the items to the template
    $pnRender->assign('duditems', $items);
    $pnRender->assign('uname',    (isset($uname) && !empty($uname)) ? $uname : pnUserGetVar('uname'));

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_user_modify.htm');
}

/**
 * update a users profile
 *
 * @author Franky Chestnut
 */
function Profile_user_update()
{
    // Confirm authorisation code.
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Profile', 'user', 'modify'));
    }

    // Get parameters from whatever input we need.
    $uname    = FormUtil::getPassedValue('uname',    null, 'POST');
    $pass     = FormUtil::getPassedValue('cpass',     null, 'POST');
    $vpass    = FormUtil::getPassedValue('cvpass',    null, 'POST');
    $dynadata = FormUtil::getPassedValue('dynadata', null, 'POST');

    $uid = pnUserGetVar('uid');
    $errors = false;

    // Initializing Check Break
    $errors = array();

    // Email Check
    if (empty($dynadata['_UREALEMAIL'])) {
        LogUtil::registerError(_PROFILE_EMPTYMAIL);
        $errors = true;
    } else if (!pnVarValidate($dynadata['_UREALEMAIL'], 'email')) {
        LogUtil::registerError(_PROFILE_INVALIDMAIL);
        $errors = true;
    } else {
        // The API function is called.
        $mailexists = pnModAPIFunc('Profile', 'user', 'checkmailexists',
                                   array('newmail' => $dynadata['_UREALEMAIL'],
                                         'uid'     => $uid));

        if ($mailexists >= 1) {
            LogUtil::registerError(_PROFILE_MAILEXISTS);
            $errors = true;
        }
    }

    // Password Check
    if (!empty($pass)) {
        if ($pass != $vpass) {
            LogUtil::registerError(_PROFILE_WRONGPASS);
            $errors = true;
        }
    }

    // Check for required fields - The API function is called.
    $checkrequired = pnModAPIFunc('Profile', 'user', 'checkrequired',
                                  array('dynadata' => $dynadata));

    if ($checkrequired['result'] == true) {
        LogUtil::registerError(_PROFILE_REQUIREDMISSING . ' ('.$checkrequired['translatedFieldsStr'].')');
        $errors = true;
    }

    // return stop if not empty
    if ($errors) {
        // wo do not send the pswords here!
        $params = array('uname'    => $uname,
                        'dynadata' => $dynadata);
        return pnRedirect(pnModURL('Profile', 'user', 'modify', $params));
    }

    // Building the sql and saving - The API function is called.
    $save = pnModAPIFunc('Profile', 'user', 'savedata',
                         array('uid'      => $uid,
                               'pass'     => $pass,
                               'vpass'    => $vpass,
                               'dynadata' => $dynadata));

    if ($save != true) {
        return pnRedirect(pnModUrl('Profile', 'user', 'view'));
    }

    // This function generated no output, we redirect the user
    LogUtil::registerStatus (_PROFILE_SAVEOK);
    return pnRedirect(pnModUrl('Profile', 'user', 'view', array('uname' => pnUserGetVar('uname'))));
}

/**
 * user_menu_add_option()
 *
 * This is a legacy function to add an icon to the user's menu.
 * To return these values to the new style API, we need to use
 * a global variable.
 * This is ugly and will be removed
 *
 * @param string $url the URL for the user action
 * @param string $title the description of the user action
 * @param string $image the icon
 **/
function user_menu_add_option($url, $title, $image)
{
    // replace images/menu/ -- the images must be moved to the (pn)images
    // directory of the module
    $image = preg_replace('#^' . pnConfigGetVar('userimg') . '/?#', '', $image);
    $GLOBALS['old_style_links'] = compact('url', 'title', 'image');
}

/**
 * usermenu()
 * This is a legacy function to add an icon to the user's menu.
 * To return these values to the new style API, we need to use
 * a global variable.
 * This is ugly and will be removed
 *
 * @param string $url the URL for the user action
 * @param string $title the description of the user action
 * @param string $image the icon
 **/
function usermenu($url, $title, $image)
{
    user_menu_add_option($url, $title, $image);
}

/**
 * view members list
 * This function provides the main members list view
 * 
 * @author Mark West
 * @return string HTML string
 */
function Profile_user_viewmembers($args)
{
    // Security check
    if (!SecurityUtil::checkPermission('Profile:Members:', '::', ACCESS_READ)) {
        return LogUtil::registerPermissionError();
    }

    // Get parameters from whatever input we need
    $startnum = FormUtil::getPassedValue('startnum', isset($args['startnum']) ? $args['startnum'] : null, 'GET');
    $sortby = FormUtil::getPassedValue('sortby', isset($args['sortby']) ? $args['sortby'] : null, 'GETPOST');
    $searchby = FormUtil::getPassedValue('searchby', isset($args['searchby']) ? $args['searchby'] : null, 'GETPOST');
    $sortorder = FormUtil::getPassedValue('sortorder', isset($args['sortorder']) ? $args['sortorder'] : null, 'GETPOST');
    $letter = FormUtil::getPassedValue('letter', isset($args['letter']) ? $args['letter'] : null, 'GETPOST');

    // Set some defaults
    if (empty($sortby)) {
        $sortby = 'uname';
    }
    if (empty($letter)) {
        $letter = null;
    }
    if (empty($startnum)) {
        $startnum = 1;
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile');

    // get some permissions to use in the cache id and later to filter template output
    if (SecurityUtil::checkPermission('Users::', '::', ACCESS_DELETE) ) {
        $edit=1; $delete=1;
    } elseif (SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT) ) {
        $edit=1; $delete=0;
    } else {
        $edit=0; $delete=0;
    }

    // set the cache id
    $pnRender->cache_id = md5((int)$edit.(int)$delete.$startnum.$letter.$sortby);

    // check out if the contents are cached.
    if ($pnRender->is_cached('profile_user_members_view.htm')) {
        //return $pnRender->fetch('profile_user_members_view.htm');
    }

    // get the number of users to show per page from the module vars
    $itemsperpage = pnModGetVar('Profile', 'memberslistitemsperpage');

    // assign values for header
    $pnRender->assign('memberslistreg', pnModAPIFunc('Profile', 'memberslist', 'countitems'));
    $pnRender->assign('memberslistonline', pnModAPIFunc('Profile', 'memberslist', 'getregisteredonline'));
    $pnRender->assign('memberslistnewest', pnUserGetVar('uname', pnModAPIFunc('Profile', 'memberslist', 'getlatestuser')));

    // get full list of user id's
    $users = pnModAPIFunc('Profile', 'memberslist', 'getall',
                          array('letter'    => $letter,
                                'sortby'    => $sortby,
                                'sortorder' => $sortorder,
                                'searchby'  => $searchby,
                                'startnum'  => $startnum,
                                'numitems'  => $itemsperpage));

    $userscount = pnModAPIFunc('Profile', 'memberslist', 'countitems', array('letter' => $letter));

    // Is current user online
    $pnRender->assign('loggedin', pnUserLoggedIn());

    // check if we should show the extra admin column
    $pnRender->assign('adminedit', $edit);
    $pnRender->assign('admindelete', $delete);

    $pagedusers = array();
    foreach($users as $userid => $user) {
        //$user = array_merge(pnUserGetVars($userid['uid']), $userid);
        $isonline = pnModAPIFunc('Profile', 'memberslist', 'isonline', array('userid' => $userid['uid']));

        // is this user online
        if ($isonline){
            $user['onlinestatus'] = 1;
        } else {
            $user['onlinestatus'] = 0;
        }

        // filter out any dummy url's
        if (isset($user['url']) && (!$user['url'] || $user['url'] == 'http://' || $user['url'] == 'http:///'))  {
            $user['url'] = '';
        }

        // add user to results array
        $pagedusers[] =  $user;
    }
    // get all active profile fields
    $activeduds  = pnModAPIfunc('Profile', 'user', 'getallactive');
    foreach ($activeduds as $activedud) {
        $dudarray[] = $activedud['prop_label'];
    }
    
    $pnRender->assign('dudarray',  $dudarray);
    $pnRender->assign('users',     $pagedusers);
    $pnRender->assign('letter',    $letter);
    $pnRender->assign('sortby',    $sortby);
    $pnRender->assign('sortorder', $sortorder);

    // load profile language file
    pnModLangLoad('Profile', 'user');

    // check which messaging module is available and add the necessary info
    $pnRender->assign('msgmodule', pnModAPIFunc('Profile', 'memberslist', 'getmessagingmodule'));

    // Assign the values for the smarty plugin to produce a pager
    $pnRender->assign('pager', array('numitems' => pnModAPIFunc('Profile', 'memberslist', 'countitems', array('letter' => $letter)),
                                     'itemsperpage' => pnModGetVar('Profile', 'memberslistitemsperpage')));

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_user_members_view.htm');
}

/**
 * Displays last X registered users
 * This function displays the last X users who registered at this site
 * available from the module.
 * 
 * @author Mark West
 * @return string HTML string
 */
function Profile_user_recentmembers()
{
    // Security check
    if (!SecurityUtil::checkPermission('Profile:Members:recent', '::', ACCESS_READ)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile');

    // set the cache id
    $pnRender->cache_id = 'recent' . pnUserLoggedIn();

    // check out if the contents are cached.
    if ($pnRender->is_cached('profile_user_members_recent.htm')) {
        return $pnRender->fetch('profile_user_members_recent.htm');
    }

    $modvars = pnModGetVar('Profile');

    // get last x user id's
    $users = pnModAPIFunc('Profile', 'memberslist', 'getall',
                          array('sortby' => 'user_regdate',
                                'numitems' => $modvars['recentmembersitemsperpage'],
                                'sortorder' => 'DESC'));

    // Is current user online
    $pnRender->assign('loggedin', pnUserLoggedIn());

    // assign all module vars obtained earlier
    $pnRender->assign($modvars);

    // get some permissions to use in the cache id and later to filter template output
    if (SecurityUtil::checkPermission('Users::', '::', ACCESS_DELETE) ) {
        $edit=1; $delete=1;
    } elseif (SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT) ) {
        $edit=1; $delete=0;
    } else {
        $edit=0; $delete=0;
    }

    // check if we should show the extra admin column
    $pnRender->assign('adminedit', $edit);
    $pnRender->assign('admindelete', $delete);

    $pagedusers = array();
    foreach($users as $userid => $user) {
        $isonline = pnModAPIFunc('Profile', 'memberslist', 'isonline', array('userid' => $userid['uid']));

        // display online status
        if ($isonline){
            $user['onlinestatus'] = 1;
        } else {
            $user['onlinestatus'] = 0;
        }
        $pagedusers[] = $user;
    }
    $pnRender->assign('users', $pagedusers);

    // load profile language file
    pnModLangLoad('Profile', 'user');

    // check which messaging module is available and add the necessary info
    $pnRender->assign('msgmodule', pnModAPIFunc('Profile', 'memberslist', 'getmessagingmodule'));

    // get all active profile fields
    $activeduds  = pnModAPIfunc('Profile', 'user', 'getallactive');
    foreach ($activeduds as $activedud) {
        $dudarray[] = $activedud['prop_label'];
    }
    
    $pnRender->assign('dudarray',  $dudarray);

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_user_members_recent.htm');
}

/**
 * View users online
 * This function displays the currently online users
 * 
 * @author Mark West
 * @return string HTML string
 */
function Profile_user_onlinemembers()
{
    // Security check
    if (!SecurityUtil::checkPermission('Profile:Members:online', '::', ACCESS_READ)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Profile');

    // set the cache id
    $pnRender->cache_id = 'online' . pnUserLoggedIn();

    // check out if the contents are cached.
    if ($pnRender->is_cached('profile_user_members_online.htm')) {
       return $pnRender->fetch('profile_user_members_online.htm');
    }

    // get last 10 user id's
    $users = pnModAPIFunc('Profile', 'memberslist', 'whosonline');

    // Current user status
    $pnRender->assign('loggedin', pnUserLoggedIn());

    $pnRender->assign('users', $users);

    // check which messaging module is available and add the necessary info
    $pnRender->assign('msgmodule', pnModAPIFunc('Profile', 'memberslist', 'getmessagingmodule'));

    // get all active profile fields
    $activeduds  = pnModAPIfunc('Profile', 'user', 'getallactive');
    foreach ($activeduds as $activedud) {
        $dudarray[] = $activedud['prop_label'];
    }
    
    $pnRender->assign('dudarray',  $dudarray);

    // Return the output that has been generated by this function
    return $pnRender->fetch('profile_user_members_online.htm');
}
