<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24957 2008-12-05 21:45:13Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Admin
 */

/**
 * initialise the Admin module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if initialisation succcesful, false otherwise
 */
function Admin_init()
{
    if (!DBUtil::createTable('admin_category')) {
        return false;
    }

    if (!DBUtil::createTable('admin_module')) {
        return false;
    }

    pnModSetVar('Admin', 'modulesperrow', 5);
    pnModSetVar('Admin', 'itemsperpage', 25);
    pnModSetVar('Admin', 'defaultcategory', 5);
    pnModSetVar('Admin', 'modulestylesheet', 'navtabs.css');
    pnModSetVar('Admin', 'admingraphic', 1);
    pnModSetVar('Admin', 'startcategory', 1);
    // change below to 0 before release - just makes it easier doing development meantime - drak
    // we can now leave this at 0 since the code also checks the development flag (config.php) - markwest
    pnModSetVar('Admin', 'ignoreinstallercheck', 0);
    pnModSetVar('Admin', 'admintheme', '');
    pnModSetVar('Admin', 'displaynametype', 1);
    
    Admin_defaultdata();

    // Initialisation successful
    return true;
}

/**
 * upgrade the Admin module from an old version
 * This function can be called multiple times
 * @author Mark West
 * @param int $oldversion previous to upgrade from
 * @return bool true if upgrade succcesful, false otherwise
 */
function Admin_upgrade($oldversion)
{
    // Upgrade dependent on old version number
    switch($oldversion) {
        // version 0.1 was supplied with all .7x versions of PN
        case 0.1:
            Admin_init();
            return Admin_upgrade(1.0);
        case 1.0:
            // move the admingraphic config var to a module var
            pnModSetVar('Admin', 'admingraphic', pnConfigGetVar('admingraphic'));
            pnConfigDelVar('admingraphic');
            // set the remaining new vars
            pnModSetVar('Admin', 'modulesperrow', 5);
            pnModSetVar('Admin', 'itemsperpage', 25);
            pnModSetVar('Admin', 'defaultcategory', 5);
            pnModSetVar('Admin', 'modulestylesheet', 'navtabs.css');
            pnModSetVar('Admin', 'startcategory', 1);
            pnModSetVar('Admin', 'ignoreinstallercheck', 0);
            return Admin_upgrade(1.1);
        case 1.1:
            pnModSetVar('Admin', 'admintheme', '');
            return Admin_upgrade(1.2);
        case 1.2:
            //DBUtil::changeTable('admin_module'); // <<<< this doesnt work
            // because adodb doesnt seem to generate valid SQL for a case where you dont already have an primary key already.  Need to drop and recreate instead.

            $pntable = pnDBGetTables();
            $result = DBUtil::executeSQL("SELECT * FROM $pntable[admin_module]");
            $objArray = DBUtil::marshallObjects ($result, array('mid', 'cid'), true);
            DBUtil::dropTable('admin_module');
            DBUtil::createTable('admin_module');
            DBUtil::insertObjectArray($objArray, 'admin_module', 'amid');
            DBUtil::changeTable('admin_category');
            return Admin_upgrade(1.4);
        case 1.4:
            pnModSetVar('Admin', 'displaynametype', 1);
            break;
    }
    // Update successful
    return true;
}

/**
 * delete the Admin module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if deletetion succcesful, false otherwise
 */
function Admin_delete()
{
    if (!DBUtil::dropTable('admin_module')) {
        return false;
    }

    if (!DBUtil::dropTable('admin_category')) {
        return false;
    }

    pnModDelVar('Admin');

    // Deletion successful
    return true;
}

/**
 * create the default data for the modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Mark West
 * @return       bool       false
 */
function Admin_defaultdata()
{
    $record = array(array('catname'     => _ADMIN_CATEGORY_00_a,
                          'description' => _ADMIN_CATEGORY_00_b),
                    array('catname'     => _ADMIN_CATEGORY_01_a,
                          'description' => _ADMIN_CATEGORY_01_b),
                    array('catname'     => _ADMIN_CATEGORY_02_a,
                          'description' => _ADMIN_CATEGORY_02_b),
                    array('catname'     => _ADMIN_CATEGORY_03_a,
                          'description' => _ADMIN_CATEGORY_03_b),
                    array('catname'     => _ADMIN_CATEGORY_04_a,
                          'description' => _ADMIN_CATEGORY_04_b),
                    array('catname'     => _ADMIN_CATEGORY_05_a,
                          'description' => _ADMIN_CATEGORY_05_b),
                    array('catname'     => _ADMIN_CATEGORY_06_a,
                          'description' => _ADMIN_CATEGORY_06_b));

    DBUtil::insertObjectArray($record, 'admin_category', 'cid');
}
