<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Admin_Messages
 */

/**
 * initialise the Admin module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if initialisation succcesful, false otherwise
 */
function Admin_Messages_init()
{
    if (!DBUtil::createTable('message')) {
        return false;
    }

    // Set a default value for a module variable
    pnModSetVar('Admin_Messages', 'itemsperpage', 25);
    pnModSetVar('Admin_Messages', 'allowsearchinactive', false);

    // create the default data for the modules module
    Admin_Messages_defaultdata();

    // Initialisation successful
    return true;
}

/**
 * upgrade the Admin module from an old version
 * This function can be called multiple times
 * @author Mark West
 * @param int $oldversion previous to upgrade from
 * @return bool true if upgrade succcesful, false otherwise
 */
function Admin_Messages_upgrade($oldversion)
{
    // Upgrade dependent on old version number
    switch($oldversion) {
        // version 1.2 was shipped with PN .7x
        case 1.2:
            pnModSetVar('Admin_Messages', 'itemsperpage', 25);
            return Admin_Messages_upgrade(1.5);
        case 1.5:
            pnModSetVar('Admin_Messages', 'allowsearchinactive', false);
            DBUtil::changeTable('message');
            break;
    }

    // Update successful
    return true;
}

/**
 * delete the Admin module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if deletetion succcesful, false otherwise
 */
function Admin_Messages_delete()
{
    if (!DBUtil::dropTable('message')) {
        return false;
    }

    pnModDelVar('Admin_Messages');

    // Deletion successful
    return true;
}

/**
 * create the default data for the modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Mark West
 * @return       bool       false
 */
function Admin_Messages_defaultdata()
{
    $record = array();
    $record['title']    = _MESSAGE_00_a;
    $record['content']  = _MESSAGE_00_b;
    $record['date']     = time();
    $record['expire']   = _MESSAGE_00_d;
    $record['active']   = _MESSAGE_00_e;
    $record['view']     = _MESSAGE_00_f;
    $record['language'] = _MESSAGE_00_g;

    DBUtil::insertObject($record, 'message', 'mid');
}
