<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24813 2008-11-05 14:15:38Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @package Zikula_Core
 */


/**
 * initialise module
*/
function Categories_init()
{
    if (!DBUtil::createTable('categories_category')) {
        return false;
    }

    // Create the index
    if (!DBUtil::createIndex('idx_categories_parent', 'categories_category', 'parent_id') ||
        !DBUtil::createIndex('idx_categories_is_leaf', 'categories_category', 'is_leaf') ||
        !DBUtil::createIndex('idx_categories_name', 'categories_category', 'name') ||
        !DBUtil::createIndex('idx_categories_ipath', 'categories_category', array('ipath', 'is_leaf', 'status')) ||
        !DBUtil::createIndex('idx_categories_status', 'categories_category', 'status') ||
        !DBUtil::createIndex('idx_categories_ipath_status', 'categories_category', array('ipath', 'status'))){
        return false;
    }

    Categories_insertData_10 ();

    // Set autonumber to 10000 (for DB's that support autonumber fields)
    $cat['id'] = 9999;
    DBUtil::insertObject ($cat, 'categories_category', 'id', true);

    // for postgres, we need to explicitly set the sequence value to reflect the inserted data
    $dbType   = DBConnectionStack::getConnectionDBType();
    if ($dbType == 'postgres') {
        $pntables = pnDBGetTables();
        $tab = $pntables['categories_category'];
        $col = $pntables['categories_category_column'];
        $seq = $tab . '_cat_id_seq';
        $sql = "SELECT setval('$seq', (SELECT MAX($col[id]) + 1 FROM $tab))";
        DBUtil::executeSQL($sql);
    }

    DBUtil::deleteObjectByID('categories_category', 9999, 'id');

    Categories_createTables_101 ();

    pnModSetVar ('Categories', 'userrootcat', '/__SYSTEM__/Users');
    pnModSetVar ('Categories', 'allowusercatedit', 0);
    pnModSetVar ('Categories', 'autocreateusercat', 0);
    pnModSetVar ('Categories', 'autocreateuserdefaultcat', 0);
    pnModSetVar ('Categories', 'userdefaultcatname', 'Default');

    // Initialisation successful
    return true;
}

/**
 * create tables
*/
function Categories_createTables_101 ()
{
    if (!DBUtil::createTable('categories_mapmeta')) {
        return false;
    }

    if (!DBUtil::createIndex('idx_categories_mapmeta', 'categories_mapmeta', 'meta_id')) {
        return false;
    }

    if (!DBUtil::createTable('categories_mapobj')) {
        return false;
    }

    if (!DBUtil::createIndex('idx_categories_mapobj', 'categories_mapobj', array('modname', 'table', 'obj_id', 'obj_idcolumn'))) {
        return false;
    }

    if (!DBUtil::createTable('categories_registry')) {
        return false;
    }

    if (!DBUtil::createIndex('idx_categories_registry', 'categories_registry', array('modname', 'table', 'property'))){
        return false;
    }

    return true;
}


/**
 * insert data
*/
function Categories_insertData_10 ()
{
    $objArray = array();
    $objArray[] = array(
        'id'               => 1,
        'parent_id'        => 0,
        'is_locked'        => 1,
        'is_leaf'          => 0,
        'name'             => '__SYSTEM__',
        'display_name'     => 'b:0;',
        'display_desc'     => 'b:0;',
        'path'             => '/__SYSTEM__',
        'ipath'            => '/1',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 2,
        'parent_id'        => 1,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Modules',
        'display_name'     => Categories_makeDisplayName(_CATDISP_MODULES),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules',
        'ipath'            => '/1/2',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 3,
        'parent_id'        => 1,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'General',
        'display_name'     => Categories_makeDisplayName(_CATDISP_GENERAL),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General',
        'ipath'            => '/1/3',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 4,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'YesNo',
        'display_name'     => Categories_makeDisplayName(_CATDISP_YESNO),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/YesNo',
        'ipath'            => '/1/3/4',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 5,
        'parent_id'        => 4,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => '1 - Yes',
        'display_name'     => 'b:0;',
        'display_desc'     => 'b:0;',
        'path'             => '/__SYSTEM__/General/YesNo/1 - Yes',
        'ipath'            => '/1/3/4/5',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'Y'));
    $objArray[] = array(
        'id'               => 6,
        'parent_id'        => 4,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => '2 - No',
        'display_name'     => 'b:0;',
        'display_desc'     => 'b:0;',
        'path'             => '/__SYSTEM__/General/YesNo/2 - No',
        'ipath'            => '/1/3/4/6',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'N'));
    $objArray[] = array(
        'id'               => 10,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Pending Status Extended',
        'display_name'     => Categories_makeDisplayName(_CATDISP_PENDINGSTATUS),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended',
        'ipath'            => '/1/3/10',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 11,
        'parent_id'        => 10,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Pending',
        'display_name'     => Categories_makeDisplayName(_CATDISP_PENDING),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended/Pending',
        'ipath'            => '/1/3/10/11',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'P'));
    $objArray[] = array(
        'id'               => 12,
        'parent_id'        => 10,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Checked',
        'display_name'     => Categories_makeDisplayName(_CATDISP_CHECKED),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended/Checked',
        'ipath'            => '/1/3/10/12',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'C'));
    $objArray[] = array(
        'id'               => 13,
        'parent_id'        => 10,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Approved',
        'display_name'     => Categories_makeDisplayName(_CATDISP_APPROVED),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended/Approved',
        'ipath'            => '/1/3/10/13',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'A'));
    $objArray[] = array(
        'id'               => 14,
        'parent_id'        => 10,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Online',
        'display_name'     => Categories_makeDisplayName(_CATDISP_ONLINE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended/Online',
        'ipath'            => '/1/3/10/14',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'O'));
    $objArray[] = array(
        'id'               => 15,
        'parent_id'        => 10,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Rejected',
        'display_name'     => Categories_makeDisplayName(_CATDISP_REJECTED),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Extended/Rejected',
        'ipath'            => '/1/3/10/15',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'R'));
    $objArray[] = array(
        'id'               => 16,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Gender',
        'display_name'     => Categories_makeDisplayName(_CATDISP_GENDER),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Gender',
        'ipath'            => '/1/3/16',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 17,
        'parent_id'        => 16,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Male',
        'display_name'     => Categories_makeDisplayName(_CATDISP_MALE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Gender/Male',
        'ipath'            => '/1/3/16/17',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'M'));
    $objArray[] = array(
        'id'               => 18,
        'parent_id'        => 16,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Female',
        'display_name'     => Categories_makeDisplayName(_CATDISP_FEMALE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Gender/Female',
        'ipath'            => '/1/3/16/18',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'F'));
    $objArray[] = array(
        'id'               => 19,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Title',
        'display_name'     => Categories_makeDisplayName(_CATDISP_TITLE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Title',
        'ipath'            => '/1/3/19',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 20,
        'parent_id'        => 19,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Dr',
        'display_name'     => Categories_makeDisplayName(_CATDISP_DR),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Title/Dr',
        'ipath'            => '/1/3/19/20',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 21,
        'parent_id'        => 19,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Prof',
        'display_name'     => Categories_makeDisplayName(_CATDISP_PROF),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Title/Prof',
        'ipath'            => '/1/3/19/21',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 22,
        'parent_id'        => 19,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Mag',
        'display_name'     => Categories_makeDisplayName(_CATDISP_MAG),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Title/Mag',
        'ipath'            => '/1/3/19/13',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 23,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'ActiveStatus',
        'display_name'     => Categories_makeDisplayName(_CATDISP_ACTIVITYSTATUS),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/ActiveStatus',
        'ipath'            => '/1/3/22',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 24,
        'parent_id'        => 23,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Active',
        'display_name'     => Categories_makeDisplayName(_CATDISP_ACTIVE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/ActiveStatus/Active',
        'ipath'            => '/1/3/22/23',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'A'));
    $objArray[] = array(
        'id'               => 25,
        'parent_id'        => 23,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Inactive',
        'display_name'     => Categories_makeDisplayName(_CATDISP_INACTIVE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/ActiveStatus/Inactive',
        'ipath'            => '/1/3/22/24',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'I'));
    $objArray[] = array(
        'id'               => 26,
        'parent_id'        => 3,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Pending Status Basic',
        'display_name'     => Categories_makeDisplayName(_CATDISP_PENDINGSTATUSBASIC),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Basic',
        'ipath'            => '/1/3/25',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 27,
        'parent_id'        => 26,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Pending',
        'display_name'     => Categories_makeDisplayName(_CATDISP_PENDING),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Basic/Pending',
        'ipath'            => '/1/3/25/26',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'P'));
    $objArray[] = array(
        'id'               => 28,
        'parent_id'        => 26,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Approved',
        'display_name'     => Categories_makeDisplayName(_CATDISP_APPROVED),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/General/Pending Status Basic/Approved',
        'ipath'            => '/1/3/25/27',
        'status'           => 'A',
        '__ATTRIBUTES__'   => array('code' => 'A'));
    $objArray[] = array(
        'id'               => 29,
        'parent_id'        => 1,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Users',
        'display_name'     => Categories_makeDisplayName(_CATDISP_USERS),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Users',
        'ipath'            => '/1/29',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 30,
        'parent_id'        => 2,
        'is_locked'        => 0,
        'is_leaf'          => 0,
        'name'             => 'Global',
        'display_name'     => Categories_makeDisplayName(_CATDISP_GLOBAL),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global',
        'ipath'            => '/1/2/30',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 31,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Arts & Entertainment',
        'display_name'     => Categories_makeDisplayName(_CATDISP_ARTSENTERTAINMENT),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Arts & Entertainment',
        'ipath'            => '/1/2/30/31',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 32,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Default',
        'display_name'     => Categories_makeDisplayName(_CATDISP_DEFAULT),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Default',
        'ipath'            => '/1/2/30/32',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 33,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Information Technology',
        'display_name'     => Categories_makeDisplayName(_CATDISP_INFORMATIONTECHNOLOGY),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Information Technology',
        'ipath'            => '/1/2/30/33',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 34,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Politics',
        'display_name'     => Categories_makeDisplayName(_CATDISP_POLITICS),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Politics',
        'ipath'            => '/1/2/30/34',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 35,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Science',
        'display_name'     => Categories_makeDisplayName(_CATDISP_SCIENCE),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Science',
        'ipath'            => '/1/2/30/35',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 36,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Society',
        'display_name'     => Categories_makeDisplayName(_CATDISP_SOCIETY),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Society',
        'ipath'            => '/1/2/30/36',
        'status'           => 'A');
    $objArray[] = array(
        'id'               => 37,
        'parent_id'        => 30,
        'is_locked'        => 0,
        'is_leaf'          => 1,
        'name'             => 'Sports',
        'display_name'     => Categories_makeDisplayName(_CATDISP_SPORTS),
        'display_desc'     => Categories_makeDisplayDesc(),
        'path'             => '/__SYSTEM__/Modules/Global/Sports',
        'ipath'            => '/1/2/30/37',
        'status'           => 'A');

    DBUtil::insertObjectArray($objArray, 'categories_category', 'id', true);
}


/**
 * update the value addons tables
 */
function Categories_updateValueAddons_104 ()
{
    // Array of the modules to update
    $mods = array('News' => array('stories' => 'Main'),
                  'Pages' => array('pages' => 'Main'),
                  'FAQ' => array('faqanswer' => 'Main'),
                  'Feeds' => array('feeds' => 'Main'),
                  'Reviews' => array('reviews' => 'Main'),
                  'Content' => array('page' => 'primary'));

    $pntables = pnDBGetTables();
    $regcol   = $pntables['categories_registry_column'];
    $mapcol   = $pntables['categories_mapobj_column'];

    // Update all the items mapped if there's a Register of the module
    foreach ($mods as $module => $data) {
        foreach ($data as $table => $property) {
            $where  = "$regcol[modname]='$module' AND $regcol[table]='$table' AND $regcol[property]='$property'";
            $reg_id = DBUtil::selectObject('categories_registry', $where, array('id'));
            if ($reg_id !== false) {
                $obj = array('reg_id' => $reg_id['id']);
                $where = "$mapcol[modname]='$module' AND $mapcol[table]='$table'";
                DBUtil::updateObject($obj, 'categories_mapobj', $where, 'sid');
            }
        }
    }

    return true;
}


function Categories_makeDisplayName($name)
{

  return serialize(array(pnUserGetLang() => $name));
}


function Categories_makeDisplayDesc()
{
  return serialize(array('eng' => ''));
}


/**
 * upgrade module
*/
function Categories_upgrade($oldversion)
{
    switch ($oldversion)
    {
        case '1.00':
            Categories_createTables_101 ();
            Categories_upgrade(1.03);
            break;
        case '1.03':
            // added the reference to the registry property
            DBUtil::changeTable('categories_category');
            DBUtil::changeTable('categories_mapobj');
            // update the ValueAddons tables
            Categories_updateValueAddons_104();
            break;
    }
    return true;
}

/**
 * delete module
*/
function Categories_delete()
{
    DBUtil::dropTable('categories_category');
    DBUtil::dropTable('categories_mapobj');
    DBUtil::dropTable('categories_mapmeta');
    DBUtil::dropTable('categories_registry');

    pnModDelVar ('Categories');

    // delete other modules use of categories flag
    $pntable = pnDBGetTables();
    $cols    = $pntable['module_vars_column'];
    $name    = DataUtil::formatForStore('enablecategorization');
    $where   = "$cols[name]='$name'";
    $res     = (bool)DBUtil::deleteWhere ('module_vars', $where);

    // Deletion successful
    return true;
}
