<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnuser.php 24813 2008-11-05 14:15:38Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @package Zikula_Core
 */


Loader::loadClass ('HtmlUtil');
Loader::loadClass ('LanguageUtil');
Loader::loadClassFromModule ('Categories', 'category');


/**
 * main user function
 */
function Categories_user_main()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $referer = pnServerGetVar ('HTTP_REFERER');
    if (strpos ($referer, 'module=Categories') === false) {
        SessionUtil::setVar('categories_referer', $referer);
    }

    $pnRender = pnRender::getInstance('Categories', false);
    $pnRender->assign ('allowusercatedit', pnModGetVar('Categories', 'allowusercatedit', 0));
    return $pnRender->fetch('categories_user_editcategories.htm');
}

/**
 * edit category for a simple, non-recursive set of categories
 */
function Categories_user_edit ()
{
    $docroot = FormUtil::getPassedValue ('dr', 0);
    $cid     = FormUtil::getPassedValue ('cid', 0);
    $url     = pnModURL ('Categories', 'user', 'edit', array('dr' => $docroot));

    if (!SecurityUtil::checkPermission('Categories::category', "ID::$docroot", ACCESS_EDIT)) {
        return LogUtil::registerPermissionError($url);
    }

    $referer = pnServerGetVar ('HTTP_REFERER');
    if (strpos ($referer, 'module=Categories') === false) {
        SessionUtil::setVar('categories_referer', $referer);
    }

    $rootCat = array();
    $allCats = array();
    $editCat = array();

    if (!$docroot) {
        return LogUtil::registerError(_CATEGORIES_DOCROOT_INVALID, null, $url);
    }
    if ($docroot == 1) {
        return LogUtil::registerError(_CATEGORIES_ROOT_CANT_EDIT, null, $url);
    }

    Loader::loadClass ('CategoryUtil');

    if (is_int((int)$docroot) && $docroot > 0) {
        $rootCat = CategoryUtil::getCategoryByID ($docroot);
    } else {
        $rootCat = CategoryUtil::getCategoryByPath ($docroot);
        if (!$rootCat) {
            $rootCat = CategoryUtil::getCategoryByPath ($docroot, 'ipath');
        }
    }

    // now check if someone is trying edit another user's categories
    $userRoot = pnModGetVar ('Categories', 'userrootcat', 0);
    if ($userRoot) {
        $userRootCat = CategoryUtil::getCategoryByPath ($userRoot);
        if ($userRootCat) {
            $userRootCatIPath = $userRootCat['ipath'];
            $rootCatIPath     = $rootCat['ipath'];
            if (strpos($rootCatIPath, $userRootCatIPath) !== false) {
                if (!SecurityUtil::checkPermission('Categories::category', "ID::$docroot", ACCESS_ADMIN)) {
                    $thisUserRootCategoryName = pnModAPIFunc ('Categories', 'user', 'getusercategoryname');
                    $thisUserRootCatPath      = $userRootCat['path'] . '/' . $thisUserRootCategoryName;
                    $userRootCatPath          = $userRootCat['path'];
                    $rootCatPath              = $rootCat['path'];
                    if (strpos($rootCatPath, $userRootCatPath) === false) {
                        return LogUtil::registerError(_CATEGORIES_EDIT_USER_NOT_OWN. " ($docroot)", null, $url);
                    }
                }
            }
        }
    }

    if ($cid) {
       $editCat = CategoryUtil::getCategoryByID ($cid);
       if ($editCat['is_locked']) { 
           return LogUtil::registerError(_CATEGORIES_CATEGORY_IS_LOCKED. " ($cid, $editCat[name])", null, $url);
       }
    }

    if (!$rootCat) {
        return LogUtil::registerError(_CATEGORIES_DOCROOT_CANT_RETRIEVE . " ($docroot)", null, $url);
    }
    if ($editCat && !$editCat['is_leaf']) {
        return LogUtil::registerError(_CATEGORIES_EDITCAT_NOT_LEAF . " ($cid)", null, $url);
    }
    if ($editCat && !CategoryUtil::isDirectSubCategory ($rootCat, $editCat)) {
        return LogUtil::registerError(_CATEGORIES_EDITCAT_NOT_SUBCAT . " ($docroot, $cid)", null, $url);
    }

    $allCats = CategoryUtil::getSubCategoriesForCategory ($rootCat, false, false, false, true, true);

    $pnRender = pnRender::getInstance('Categories', false);
    $pnRender->assign('rootCat', $rootCat);
    $pnRender->assign('category', $editCat);
    $pnRender->assign('allCats', $allCats);
    $pnRender->assign('languages', LanguageUtil::getLanguages());
    $pnRender->assign('referer', SessionUtil::getVar('categories_referer'));

    return $pnRender->fetch('categories_user_edit.htm');
}


/**
 * edit categories for the currently logged in user
 */
function Categories_user_edituser ()
{
    if (!SecurityUtil::checkPermission('Categories::category', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    if (!pnUserLoggedIn()) {
        return LogUtil::registerError(_CATEGORIES_USERCAT_LOGGED_IN);
    }

    $allowUserEdit = pnModGetVar ('Categories', 'allowusercatedit', 0);
    if (!$allowUserEdit) {
        return LogUtil::registerError(_CATEGORIES_EDIT_USER_DISABLED);
    }

    $userRoot = pnModGetVar ('Categories', 'userrootcat', 0);
    if (!$userRoot) {
        return LogUtil::registerError(_CATEGORIES_USERDOCROOT_EMPTY);
    }

    Loader::loadClass ('CategoryUtil');
    $userRootCat = CategoryUtil::getCategoryByPath ($userRoot);
    if (!$userRoot) {
        return LogUtil::registerError(_CATEGORIES_USERDOCROOT_INVALID . "[$userRoot]");
    }

    if ($userRootCat == 1) {
        return LogUtil::registerError(_CATEGORIES_ROOT_CANT_EDIT);
    }

    $userCatName = Categories_user_getusercategoryname ();
    if (!$userCatName) {
        return LogUtil::registerError(_CATEGORIES_NO_USERCATNAME);
    }

    $thisUserRootCatPath = $userRoot . '/' . $userCatName;
    $thisUserRootCat = CategoryUtil::getCategoryByPath ($thisUserRootCatPath);

    $dr = null;
    if (!$thisUserRootCat) {
        $autoCreate = pnModGetVar ('Categories', 'autocreateusercat', 0);
        if (!$autoCreate) {
            return LogUtil::registerError(_CATEGORIES_USERCAT_NOAUTOCREATE);
        }

        require_once ('system/Categories/pninit.php'); // need this for Categories_makeDisplayName() && Categories_makeDisplayDesc()
        $cat = array('id'               => '',
                     'parent_id'        => $userRootCat['id'],
                     'name'             => $userCatName,
                     'display_name'     => Categories_makeDisplayName($userCatName),
                     'display_desc'     => Categories_makeDisplayDesc(),
                     'security_domain'  => 'Categories::',
                     'path'             => $thisUserRootCatPath,
                     'status'           => 'A');
        if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
            return pn_exit ("Unable to load class [category] ...");
        }

        $obj = new $class ();
        $obj->setData ($cat);
        $obj->insert ();
        // since the original insert can't construct the ipath (since
        // the insert id is not known yet) we update the object here
        $obj->update ();
        $dr = $obj->getID ();

        $autoCreateDefaultUserCat = pnModGetVar ('Categories', 'autocreateuserdefaultcat', 0);
        if ($autoCreateDefaultUserCat) {
            $userdefaultcatname = pnModGetVar ('Categories', 'userdefaultcatname', _CATDISP_DEFAULT);
            $cat = array('id'               => '',
                         'parent_id'        => $dr,
                         'name'             => $userdefaultcatname,
                         'display_name'     => Categories_makeDisplayName($userdefaultcatname),
                         'display_desc'     => Categories_makeDisplayDesc(),
                         'security_domain'  => 'Categories::',
                         'path'             => $thisUserRootCatPath . '/' . $userdefaultcatname,
                         'status'           => 'A');
            $obj->setData ($cat);
            $obj->insert ();
            // since the original insert can't construct the ipath (since
            // the insert id is not known yet) we update the object here
            $obj->update ();
        }
    } else {
        $dr = $thisUserRootCat['id'];
    }

    $url = pnModURL ('Categories', 'user', 'edit', array('dr' => $dr));
    return pnRedirect($url);
}

/**
 * refer the user back to the calling page
 */
function Categories_user_referBack()
{
    $referer = SessionUtil::getVar ('categories_referer');
    SessionUtil::DelVar ('categories_referer');
    return pnRedirect ($referer);
}

/**
 * return the categories for the currently logged in user, really only used for testing purposes
 */
function Categories_user_getusercategories ()
{
    return pnModAPIFunc ('Categories', 'user', 'getusercategories');
}

/**
 * return the category name for a user, really only used for testing purposes
 */
function Categories_user_getusercategoryname ()
{
    return pnModAPIFunc ('Categories', 'user', 'getusercategoryname');
}

