<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnuserform.php 25356 2009-01-26 15:52:19Z rgasch $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch rgasch@gmail.com
 * @package Zikula_Core
 */


/**
 * delete category
 */
function Categories_userform_delete ()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_DELETE)) {
        return LogUtil::registerPermissionError();
    }

    $cid = (int)FormUtil::getPassedValue ('cid', 0, 'GETPOST');
    $dr  = (int)FormUtil::getPassedValue ('dr', 0, 'GETPOST');
    $url = pnServerGetVar('HTTP_REFERER');

    if (!$dr) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DOCROOT, null, $url);
    }

    if (!$cid) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_CID, null, $url);
    }

    if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
        $msg = sprintf (_UNABLETOLOADMODULECLASS, 'category', 'Categories');
        return LogUtil::registerError ($msg, null, $url);
    }

    $obj = new $class ();
    $data = $obj->get ($cid);

    if (!$data) {
        $msg = sprintf (_CATEGORIES_UNABLE_TO_RETRIEVE, $cid);
        return LogUtil::registerError ($msg, null, $url);
    }

    if ($data['is_locked']) {
        return LogUtil::registerError (_CATEGORIES_CATEGORY_IS_LOCKED, null, $url);
    }

    if (!Loader::loadClass ('CategoryUtil')) {
        $msg = sprintf (_UNABLETOLOADCLASS, 'CategoryUtil');
        return LogUtil::registerError ($msg, null, $url);
    }

    CategoryUtil::deleteCategoryByID ($cid);
    return pnRedirect($url);
}

/**
 * update category
 */
function Categories_userform_edit ()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $dr   = (int)FormUtil::getPassedValue ('dr', 0, 'POST');
    $ref  = pnServerGetVar('HTTP_REFERER');

    $returnfunc = strpos ($ref, "useredit") !== false ? 'useredit' :  'edit';
    $url = pnModURL ('Categories', 'user', $returnfunc, array ('dr' => $dr));

    if (!$dr) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DOCROOT, null, $url);
    }

    if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
        $msg = sprintf (_UNABLETOLOADMODULECLASS, 'category', 'Categories');
        return LogUtil::registerError ($msg, null, $url);
    }

    $obj     = new $class ();
    $data    = $obj->getDataFromInput ();
    $oldData = $obj->get ($data['id']);
    $obj->setData ($data);

    if (!$oldData) {
        $msg = sprintf (_CATEGORIES_UNABLE_TO_RETRIEVE, $category['id']);
        return LogUtil::registerError ($msg, null, $url);
    }

    if ($oldData['is_locked']) {
        return LogUtil::registerError (_CATEGORIES_CATEGORY_IS_LOCKED, null, $url);
    }

    if (!$obj->validate()) {
        $_POST['cid'] = (int)$_POST['category']['id'];
        return pnRedirect(pnModURL('Categories', 'user', 'edit', $_POST) . '#top');
    }

    // update new category data
    $obj->update ();

    // since a name change will change the object path, we must rebuild it here
    if ($oldData['name'] != $data['name']) {
        if (!Loader::loadClass ('CategoryUtil')) {
            $msg = sprintf (_UNABLETOLOADCLASS, 'CategoryUtil');
            return LogUtil::registerError ($msg, null, $url);
        }
        CategoryUtil::rebuildPaths ('path', 'name', $data['id']);
    }

    $msg = sprintf (_CATEGORIES_UPDATE_SUCCESS, $oldData['name']);
    LogUtil::registerStatus ($msg);
    return pnRedirect($url);
}

/**
 * move field
 */
function Categories_userform_moveField ()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $cid = (int)FormUtil::getPassedValue ('cid', 0, 'GET');
    $dir = FormUtil::getPassedValue ('direction', null, 'GET');
    $dr  = (int)FormUtil::getPassedValue ('dr', 0, 'GET');
    $url = pnServerGetVar('HTTP_REFERER');

    if (!$dr) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DOCROOT, null, $url);
    }

    if (!$cid) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_CID, null, $url);
    }

    if (!$dir) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DIRECTION, null, $url);
    }

    if (!($class = Loader::loadClass ('CategoryUtil'))) {
        $msg = sprintf (_UNABLETOLOADCLASS, 'CategoryUtil');
        return LogUtil::registerError ($msg, null, $url);
    }

    if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
        $msg = sprintf (_UNABLETOLOADMODULECLASS, 'category', 'Categories');
        return LogUtil::registerError ($msg, null, $url);
    }

    $cats = CategoryUtil::getSubCategories ($dr, false, false, false, false);
    $cats = CategoryUtil::resequence ($cats, 10);
    $ak = array_keys($cats);
    foreach ($ak as $k) {
        $obj = new PNCategory ($cats[$k]);
        $obj->update ();
    }

    $data   = array('id' => $cid);
    $val    = ObjectUtil::moveField ($data, 'categories_category', $dir, 'sort_value');

    $url = pnServerGetVar('HTTP_REFERER');
    return pnRedirect($url);
}

/**
 * create category
 */
function Categories_userform_new ()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_ADD)) {
        return LogUtil::registerPermissionError();
    }

    $dr  = (int)FormUtil::getPassedValue ('dr', 0, 'POST');
    $url = pnServerGetVar('HTTP_REFERER');

    if (!$dr) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DOCROOT, null, $url);
    }

    if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
        $msg = sprintf (_UNABLETOLOADMODULECLASS, 'category', 'Categories');
        return LogUtil::registerError ($msg, null, $url);
    }

    $cat = new PNCategory ();
    $data = $cat->getDataFromInput ();

    if (!$cat->validate()) {
        return pnRedirect(pnModURL('Categories', 'user', 'edit', $_POST) . '#top');
    }

    $cat->insert ();
    // since the original insert can't construct the ipath (since
    // the insert id is not known yet) we update the object here.
    $cat->update ();

    $msg = sprintf (_CATEGORIES_INSERT_SUCCESS, $data['name']);
    LogUtil::registerStatus ($msg);
    return pnRedirect($url);
}

/**
 * resequence categories
 */
function Categories_userform_resequence ()
{
    if (!SecurityUtil::checkPermission('Categories::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $dr  = (int)FormUtil::getPassedValue ('dr', 0, 'GET');
    $url = pnServerGetVar('HTTP_REFERER');

    if (!$dr) { 
        return LogUtil::registerError (_CATEGORIES_INVALID_DOCROOT, null, $url);
    }

    if (!($class = Loader::loadClass ('CategoryUtil'))) {
        $msg = sprintf (_UNABLETOLOADCLASS, 'CategoryUtil');
        return LogUtil::registerError ($msg, null, $url);
    }

    $cats = CategoryUtil::getSubCategories ($dr, false, false, false, false);
    $cats = CategoryUtil::resequence ($cats, 10);

    if (!($class = Loader::loadClassFromModule ('Categories', 'category'))) {
        $msg = sprintf (_UNABLETOLOADMODULECLASS, 'category', 'Categories');
        return LogUtil::registerError ($msg, null, $url);
    }

    $ak = array_keys($cats);
    foreach ($ak as $k) {
        $obj = new PNCategory ($cats[$k]);
        $obj->update ();
    }

    return pnRedirect(pnServerGetVar('HTTP_REFERER'));
}

