<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnajax.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Permissions
*/

//header("Content-Type:text/plain; charset=iso-8859-15");

/**
 * Updates a group in the database
 *
 * @author Frank Schummertz - Frank Chestnut
 * @param gid the group id
 * @param gtype the group type
 * @param state the group state
 * @param nbumax the maximum of users
 * @param name the group name
 * @param description the group description
 * @return mixed updated group as array or Ajax error
 */
function Groups_ajax_updategroup($args)
{
    if (!SecurityUtil::checkPermission('Groups::', '::', ACCESS_ADMIN)) {
        AjaxUtil::error(DataUtil::formatForDisplayHTML(_MODULENOAUTH));
    }

    if (!SecurityUtil::confirmAuthKey()) {
        AjaxUtil::error(FormUtil::getPassedValue('authid') . ' : ' . _BADAUTHKEY);
    }

    pnModLangLoad('Groups', 'admin');

    $gid          = FormUtil::getPassedValue('gid', null,    'post');
    $gtype        = FormUtil::getPassedValue('gtype', 9999,  'post');
    $state        = FormUtil::getPassedValue('state', null,  'post');
    $nbumax       = FormUtil::getPassedValue('nbumax', 9999, 'post');
    $name         = DataUtil::convertFromUTF8(FormUtil::getPassedValue('name',        null, 'post'));
    $description  = DataUtil::convertFromUTF8(FormUtil::getPassedValue('description', null, 'post'));

    if (empty($name)) {
        return array('error'   => true,
                     'gid'     => $gid,
                     'message' => DataUtil::formatForDisplay(_GROUPS_MISSINGNAME));
    }

    if (ereg("[\n\r\t\x0B]",$name)) {
        $name = trim(ereg_replace("[\n\r\t\x0B]","",$name));
    }
    if (ereg("[\n\r\t\x0B]",$description)) {
        $description = trim(ereg_replace("[\n\r\t\x0B]","",$description));
    }

    // Pass to API
    $res = pnModAPIFunc('Groups',
                        'admin',
                        'update',
                        array('gid'         => $gid,
                              'name'        => $name,
                              'gtype'       => $gtype,
                              'state'       => $state,
                              'nbumax'      => $nbumax,
                              'description' => $description));
    if ($res==false) {
        // check for sessionvar
        $msgs = LogUtil::getStatusMessagesText ();
        if (!empty($msgs)) {
            // return with msg, but not via AjaxUtil::error
            return array('error'   => true,
                         'gid'     => $gid,
			 'message' => $msgs);
        }
    }
    // Setting various defines
    $typelabel = array('0'  => _GROUPS_CORE,
                       '1'  => _GROUPS_PUBLICSHORT,
                       '2'  => _GROUPS_PRIVATESHORT);

    $statelabel = array('0' => _GROUPS_CLOSED,
                        '1' => _GROUPS_OPEN);

    // Using uncached query here as it was returning the unupdated group
    $group = DBUtil::selectObjectByID('groups', $gid, 'gid', null, null, null, false);
    
    // get group member count
    $group['nbuser'] = pnModAPIFunc('Groups', 'user', 'countgroupmembers', array('gid' => $gid));

    $group['statelbl'] = $statelabel[$group['state']];
    $group['gtypelbl'] = $typelabel[$group['gtype']];

    return $group;

}

/**
 * Create a blank group and return it
 *
 * @author Frank Schummertz - Frank Chestnut
 * @param none
 * @return mixed array with new permission or Ajax error
 */
function Groups_ajax_creategroup()
{

    if (!SecurityUtil::checkPermission('Groups::', '::', ACCESS_ADMIN)) {
        AjaxUtil::error(DataUtil::formatForDisplayHTML(_MODULENOAUTH));
    }

    if (!SecurityUtil::confirmAuthKey()) {
        AjaxUtil::error(FormUtil::getPassedValue('authid') . ' : ' . _BADAUTHKEY);
    }

    pnModLangLoad('Groups', 'admin');

    $typelabel = array(0  => _GROUPS_CORE,
                       1  => _GROUPS_PUBLICSHORT,
                       2  => _GROUPS_PRIVATESHORT);

    $statelabel = array(0 => _GROUPS_CLOSED,
                        1 => _GROUPS_OPEN);

    // Default values
    $obj = array('name'        => '',
                 'gtype'       => 0,
                 'state'       => 0,
                 'nbumax'      => 0,
                 'description' => '');

    $newgroup = pnModAPIFunc('Groups', 'admin', 'create', $obj);
    
    if ($newgroup == false) {
        AjaxUtil::error(DataUtil::formatForDisplay(_GROUPS_ADDERROR));
    }

    // temporary group name
    $updobj = array('name' => _GROUPS_ADDINGNEW  . '(' . $newgroup . ')',
                    'gid'  => $newgroup);
    DBUtil::updateObject($updobj, 'groups', null, 'gid');

    // finally select the new group    
    $obj = DBUtil::selectObjectByID('groups', $newgroup, 'gid', null, null, null, false);

    $obj['statelbl']   = $statelabel[$obj['state']];
    $obj['gtypelbl']   = $typelabel[$obj['gtype']];
    $obj['membersurl'] = pnModURL('Groups', 'admin', 'groupmembership', array('gid' => $newgroup));

    return $obj;

}

/**
 * Delete a group
 *
 * @author Frank Schummertz - Frank Chestnut
 * @param gid the group id
 * @return mixed the id of the group that has been deleted or Ajax error
 */
function Groups_ajax_deletegroup()
{
    if (!SecurityUtil::checkPermission('Groups::', '::', ACCESS_ADMIN)) {
        AjaxUtil::error(DataUtil::formatForDisplayHTML(_MODULENOAUTH));
    }

    if (!SecurityUtil::confirmAuthKey()) {
        AjaxUtil::error(DataUtil::formatForDisplayHTML(_BADAUTHKEY));
    }

    $gid   = FormUtil::getPassedValue('gid', null, 'get');
    $group = DBUtil::selectObjectByID('groups', $gid, 'gid');

    // Check if it is the default group...
    $defaultgroup = pnModGetVar('Groups', 'defaultgroup');

    if ($group['name'] == $defaultgroup) {
        pnModLangLoad('Groups', 'admin');
        AjaxUtil::error(DataUtil::formatForDisplayHTML(_GROUPS_DEFAULTGROUPCANNOTBEDELETED));
    }

    if (pnModAPIFunc('Groups', 'admin', 'delete', array('gid' => $gid)) == true) {
        return array('gid' => $gid);
    }

    AjaxUtil::error(_GROUPS_DELETEERROR . $gid);

}
