<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24982 2008-12-06 17:00:48Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Groups
*/

/**
 * initialise the groups module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if initialisation succesful, false otherwise
 */
function Groups_init()
{
    // create the groups table
    if (!DBUtil::createTable('groups')) {
        return false;
    }

    // create the group membership table
    if (!DBUtil::createTable('group_membership')) {
        return false;
    }

    // create the groups applications table
    if (!DBUtil::createTable('group_applications')) {
        return false;
    }

    // set all our module vars
    pnModSetVar('Groups', 'itemsperpage', 25);
    pnModSetVar('Groups', 'defaultgroup', _GROUPS_1_a);
    pnModSetVar('Groups', 'mailwarning', 0);
    pnModSetVar('Groups', 'hideclosed', 0);

    // create the default data for the modules module
    groups_defaultdata();

    // Initialisation successful
    return true;
}

/**
 * upgrade the groups module from an old version
 * This function can be called multiple times
 * @author Mark West
 * @param int $oldversion version to upgrade from
 * @return bool true if upgrade succesful, false otherwise */
function Groups_upgrade($oldversion)
{
    // Upgrade dependent on old version number
    switch($oldversion) {
        // version 0.1 shipped with PN .72x
        case 0.1:
            pnModSetVar('Groups', 'defaultgroup', pnConfigGetVar('defaultgroup'));
            pnConfigDelVar('defaultgroup');
            pnModSetVar('Groups', 'itemsperpage', 25);
            return Groups_upgrade(1.0);
        // version 1.0 shipped with PN .76x
        case 1.0:
            pnModSetVar('Groups', 'mailwarning', false);
            pnModSetVar('Groups', 'hideclosed', 0);
            // add the group applications table
            if (!DBUtil::createTable('group_applications')) {
                return false;
            }
            if (!DBUtil::changeTable('groups')) {
                return false;
            }
            return Groups_upgrade(2.0);
        case 2.0:
            pnModSetVar('Groups', 'itemsperpage', 25);
            break;
    }
    // Update successful
    return true;
}

/**
 * delete the groups module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true if delete succesful, false otherwise */
function Groups_delete()
{
    // Deletion not allowed
    return false;
}

/**
 * create the default data for the groups module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Mark West
 * @return       bool       false
 */
function groups_defaultdata()
{
    $records = array(array('name'        => _GROUPS_1_a,
                           'description' => _GROUPS_1_b,
                           'prefix'      => _GROUPS_1_c),
                     array('name'        => _GROUPS_2_a,
                           'description' => _GROUPS_2_b,
                           'prefix'      => _GROUPS_2_c));

    DBUtil::insertObjectArray($records, 'groups', 'gid');

    // Insert Anonymous and Admin users
    $records = array(array('gid' => _GROUP_MEMBERSHIP_1_a,
                           'uid' => _GROUP_MEMBERSHIP_1_b),
                     array('gid' => _GROUP_MEMBERSHIP_2_a,
                           'uid' => _GROUP_MEMBERSHIP_2_b));

    DBUtil::insertObjectArray($records, 'group_membership', 'gid', true);
}
