<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnadmin.php 24854 2008-11-08 22:34:57Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Modules
 */

/**
 * Modules Module main admin function
 * @author Jim McDonald
 * @return string HTML output string
 */
function modules_admin_main()
{
    // security check
    if (!SecurityUtil::checkPermission('Modules::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
    $pnRender = pnRender::getInstance('Modules', false);

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_main.htm');
}

/**
 * Modules_admin_modify - modify a module
 * @author Jim McDonald
 * @return string HTML output string
 */
function modules_admin_modify()
{
    // get our input
    $id = (int)FormUtil::getPassedValue('id', null, 'GET');

    // check the input
    if (!is_numeric($id)) {
        return LogUtil::registerError (_MODARGSERROR, null, pnModURL('Modules', 'admin', 'view'));
    }

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
    $pnRender = pnRender::getInstance('Modules', false);

    $result = pnModAPIFunc('Modules', 'admin', 'modify',
                            array('id' => $id));
    if (!$result) {
        return LogUtil::registerError (_MODULES_NOSUCHMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // Security check - important to do this as early as possible to avoid
    // potential security holes or just too much wasted processing.  For the
    // main function we want to check that the user has at least edit privilege
    // for some item within this component, or else they won't be able to do
    // anything and so we refuse access altogether.  The lowest level of access
    // for administration depends on the particular module, but it is generally
    // either 'edit' or 'delete'
    if (!SecurityUtil::checkPermission('Modules::', "$result[name]::$id", ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // add the module id
    $pnRender->assign('id', $id);

    // assign the module item
    $pnRender->assign($result);

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_modify.htm');
}

/**
 * Modules_admin_update - update a module
 * @author Jim McDonald
 * @return string HTML output string
 * @param int 'id' module id
 * @param string 'newdisplayname' new display name of the module
 * @param string 'newdescription' new description of the module
 */
function modules_admin_update()
{
    // Get parameters
    $id = (int)FormUtil::getPassedValue('id', null, 'POST');
    $newdisplayname = FormUtil::getPassedValue('newdisplayname', null, 'POST');
    $newdescription = FormUtil::getPassedValue('newdescription', null, 'POST');

    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    // Pass to API
    if (pnModAPIFunc('Modules', 'admin', 'update',
                     array('id' => $id,
                           'displayname' => $newdisplayname,
                           'description' => $newdescription))) {
        // Success
        LogUtil::registerStatus (_MODULES_UPDATEDINFORMATION);
    }

    return pnRedirect(pnModURL('Modules', 'admin', 'view'));
}

/**
 * Modules_admin_update hooks
 * update hooks for a module
 * @author Mark West
 * @param int 'id' module id
 * @return bool true if successful, false otherwise
 */
function modules_admin_updatehooks()
{
    // Get parameters
    $id = (int)FormUtil::getPassedValue('id', null, 'REQUEST');

    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    // Pass to API
    if (pnModAPIFunc('Modules', 'admin', 'updatehooks',
                     array('id' => $id))) {
        // Success
        LogUtil::registerStatus (_MODULES_UPDATEDINFORMATION);
    }

    return pnRedirect(pnModURL('Modules', 'admin', 'view'));
}

/**
 * Modules_admin_update hooks, extended version
 * update hooks for a module
 * @author Mark West
 * @author Frank Schummertz
 * @param int 'id' module id
 * @return bool true if successful, false otherwise
 */
function modules_admin_extendedupdatehooks()
{
    // Get parameters
    $id = (int)FormUtil::getPassedValue('id', null, 'REQUEST');

    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    // Pass to API
    if (pnModAPIFunc('Modules', 'admin', 'extendedupdatehooks',
                     array('id' => $id))) {
        // Success
        LogUtil::registerStatus (_MODULES_UPDATEDINFORMATION);
    }

    return pnRedirect(pnModURL('Modules', 'admin', 'view'));
}

/**
 * Modules_admin_view - list modules and current settings
 * @author Jim McDonald
 * @return string HTML output string
 */
function modules_admin_view()
{
    // Security check
    if (!SecurityUtil::checkPermission('Modules::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Get parameters from whatever input we need.
    $startnum = (int)FormUtil::getPassedValue('startnum', null, 'REQUEST');
    $letter = FormUtil::getPassedValue('letter', null, 'REQUEST');
    $state = FormUtil::getPassedValue('state', (!strstr(pnServerGetVar('HTTP_REFERER'), 'module=Modules')) ? null : SessionUtil::getVar('state', null), 'REQUEST');
    $sort = FormUtil::getPassedValue('sort', (!strstr(pnServerGetVar('HTTP_REFERER'), 'module=Modules')) ? null : SessionUtil::getVar('sort', null), 'REQUEST');

    // do some clean up
    SessionUtil::delVar('interactive_init');
    SessionUtil::delVar('interactive_remove');
    SessionUtil::delVar('interactive_upgrade');

    // Create output object
    $pnRender = pnRender::getInstance('Modules', false);

    // assign the state filter
    $pnRender->assign('state', $state);

    // Get list of modules
    $mods = pnModAPIFunc('Modules', 'admin', 'list',
                          array('startnum' => $startnum,
                                    'letter' => $letter,
                                    'state' => $state,
                                    'numitems' => pnModGetVar('Modules', 'itemsperpage'),
                                    'sort' => $sort));

    // generate an auth key to use in urls
    $authid = SecurityUtil::generateAuthKey();

    $moduleinfo = array();
    if (!empty($mods)) {
        foreach($mods as $mod) {
            // Add applicable actions
            $actions = array();

            if (SecurityUtil::checkPermission('Modules::', "$mod[name]::$mod[id]", ACCESS_ADMIN)) {
                switch ($mod['state']) {
                    case PNMODULE_STATE_ACTIVE:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'deactivate', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => 'folder_grey.gif',
                                           'title' => _DEACTIVATE);
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'hooks', array('id' => $mod['id'])),
                                           'image' => 'attach.gif',
                                           'title' => _HOOKS);
                        break;
                    case PNMODULE_STATE_INACTIVE:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'activate', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => 'folder_green.gif',
                                           'title' => _ACTIVATE);
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'remove', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => '14_layer_deletelayer.gif',
                                           'title' => _MODULES_REMOVE);
                        break;
                    case PNMODULE_STATE_MISSING:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'remove', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => '14_layer_deletelayer.gif',
                                           'title' => _MODULES_REMOVE);
                        break;
                    case PNMODULE_STATE_UPGRADED:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'upgrade', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => 'agt_update-product.gif',
                                           'title' => _MODULES_UPGRADE);
                        break;
                    case PNMODULE_STATE_INVALID:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'remove', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => '14_layer_deletelayer.gif',
                                           'title' => _MODULES_REMOVE);
                        break;
                    case PNMODULE_STATE_UNINITIALISED:
                    default:
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'initialise', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => 'agt_update_misc.gif',
                                           'title' => _MODULES_INITIALISE);
                        $actions[] = array('url' => pnModURL('Modules', 'admin', 'remove', array('id' => $mod['id'], 'startnum' => $startnum, 'authid' => $authid, 'letter' => $letter, 'state' => $state)),
                                           'image' => '14_layer_deletelayer.gif',
                                           'title' => _MODULES_REMOVE);
                        break;
                }

                // RNG: can't edit an invalid module
                if ($mod['state'] != PNMODULE_STATE_INVALID)
                    $actions[] = array('url' => pnModURL('Modules', 'admin', 'modify', array('id' => $mod['id'])),
                                       'image' => 'xedit.gif',
                                       'title' => _EDIT);
            }

            // Translate state
            switch ($mod['state']) {
                case PNMODULE_STATE_INACTIVE:
                    $status = _INACTIVE;
                    $statusimage = 'yellowled.gif';
                    break;
                case PNMODULE_STATE_ACTIVE:
                    $status = _ACTIVE;
                    $statusimage = 'greenled.gif';
                    break;
                case PNMODULE_STATE_MISSING:
                    $status = _MODULES_FILESMISSING;
                    $statusimage = 'redled.gif';
                    break;
                case PNMODULE_STATE_UPGRADED:
                    $status = _MODULES_UPGRADED;
                    $statusimage = 'redled.gif';
                    break;
                case PNMODULE_STATE_INVALID:
                    $status = _MODULES_INVALID;
                    $statusimage = 'redled.gif';
                    break;
                case PNMODULE_STATE_UNINITIALISED:
                default:
                    $status = _MODULES_UNINIT;
                    $statusimage = 'redled.gif';
                    break;
            }

            $moduleinfo[] = array('modinfo' => $mod,
                                  'status' => $status,
                                  'statusimage' => $statusimage,
                                  'options' => $actions);

        }
    }

    $pnRender->assign('modules', $moduleinfo);

    // Assign the values for the smarty plugin to produce a pager.
    $pnRender->assign('pager', array('numitems' => pnModAPIFunc('Modules', 'admin', 'countitems',
                                                    array('letter' => $letter, 'state' => $state)),
                                     'itemsperpage' => pnModGetVar('Modules', 'itemsperpage')));

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_view.htm');
}

/**
 * Modules_admin_view - list modules and current settings
 *
 * This function is an alias to modules_admin_view as pnphpbb calls this
 * function name directly on the install
 *
 * @author Jim McDonald
 * @see modules_admin_view
 * @return string HTML output string
 */
function modules_admin_list()
{
    return modules_admin_view();
}

/**
 * Initialise a module
 * @author Jim McDonald
 * @param int 'id' module id
 * @return bool true
 */
function modules_admin_initialise()
{
    // Security and sanity checks
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    // Get parameters from whatever input we need
    $id  = (int)FormUtil::getPassedValue ('id', 0);
    $objectid = (int)FormUtil::getPassedValue ('objectid', 0);
    $confirmation = (bool)FormUtil::getPassedValue ('confirmation', false);
    $startnum = (int)FormUtil::getPassedValue ('startnum');
    $letter = FormUtil::getPassedValue ('letter');
    $state = FormUtil::getPassedValue ('state');
    if ($objectid) {
        $id = $objectid;
    }

    // assign any dependencies - filtering out non-active module dependents
    // when getting here without a valid id we are in interactive init mode and then
    // the dependencies checks have been done before already
    if ($id <> 0) {
        $dependencies = pnModAPIFunc('Modules', 'admin', 'getdependencies', array('modid' => $id));
        $modulenotfound = false;
        if (empty($confirmation) && $dependencies) {
            foreach ($dependencies as $key => $dependency) {
                $dependencies[$key]['insystem'] = true;
                $modinfo = pnModGetInfo(pnModGetIDFromName($dependency['modname']));
                if (pnModAvailable($dependency['modname'])) {
                    unset($dependencies[$key]);
                } elseif (!empty($modinfo)) {
                    $dependencies[$key] = array_merge($dependencies[$key], $modinfo);
                } else {
                    $dependencies[$key]['insystem'] = false;
                    $modulenotfound = true;
                }
            }

            // we have some dependencies so let's warn the user about these
            if (!empty($dependencies)) {
                $pnRender = pnRender::getInstance('Modules', false);
                $pnRender->assign('id', $id);
                $pnRender->assign('dependencies', $dependencies);
                $pnRender->assign('modulenotfound', $modulenotfound);
                return $pnRender->fetch('modules_admin_initialise.htm');
            }
        } else {
            $dependencies = (array)FormUtil::getPassedValue('dependencies', array(), 'POST');
        }
    }

    $interactive_init = SessionUtil::getVar('interactive_init');
    $interactive_init = (empty($interactive_init)) ? false : true;
    if ($interactive_init == false) {
        SessionUtil::setVar('modules_id', $id);
        SessionUtil::setVar('modules_startnum', $startnum);
        SessionUtil::setVar('modules_letter', $letter);
        SessionUtil::setVar('modules_state', $state);
        $activate = false;
    } else {
        $id       = SessionUtil::getVar('modules_id');
        $startnum = SessionUtil::getVar('modules_startnum');
        $letter   = SessionUtil::getVar('modules_letter');
        $state    = SessionUtil::getVar('modules_state');
        $activate = (bool)FormUtil::getPassedValue('activate');
    }

    if (empty($id) || !is_numeric($id)) {
        return LogUtil::registerError (_MODULES_NOMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // initialise and activate any dependencies
    if(isset($dependencies) && is_array($dependencies)) {
        foreach ($dependencies as $dependency) {
            if (!pnModAPIFunc('Modules', 'admin', 'initialise', array('id' => $dependency))) {
                return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
            }
            if (!pnModAPIFunc('Modules', 'admin', 'setstate', array('id' => $dependency, 'state' => PNMODULE_STATE_ACTIVE))) {
                return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
            }
        }
    }

    // Now we've initialised the dependencies initialise the main module
    $res = pnModAPIFunc('Modules', 'admin', 'initialise',
                        array('id' => $id,
                              'interactive_init' => $interactive_init));
    if (is_bool($res) && $res == true) {
        // Success
        SessionUtil::delVar('modules_id');
        SessionUtil::delVar('modules_startnum');
        SessionUtil::delVar('modules_letter');
        SessionUtil::delVar('modules_state');
        SessionUtil::delVar('interactive_init');
        LogUtil::registerStatus (_MODULES_INITIALISED);

        if ($activate==true) {
            if (pnModAPIFunc('Modules', 'admin', 'setstate',
                             array('id' => $id,
                                   'state' => PNMODULE_STATE_ACTIVE))) {
                // Success
                LogUtil::registerStatus (_MODULES_ACTIVATED);
            }
        }
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } elseif (is_bool($res)) {
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } else {
        return $res;
    }
}

/**
 * Activate a module
 * @author Jim McDonald
 * @param int 'id' module id
 * @return bool true
 */
function modules_admin_activate()
{
    // Security and sanity checks
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    $id = (int)FormUtil::getPassedValue('id', null, 'GET');
    $startnum = (int)FormUtil::getPassedValue('startnum', null, 'GET');
    $letter = FormUtil::getPassedValue('letter', null, 'GET');
    $state = FormUtil::getPassedValue('state', null, 'GET');
    if (empty($id) || !is_numeric($id)) {
        return LogUtil::registerError (_MODULES_NOMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // Update state
    if (pnModAPIFunc('Modules', 'admin', 'setstate',
                     array('id' => $id,
                           'state' => PNMODULE_STATE_ACTIVE))) {
        // Success
        LogUtil::registerStatus (_ACTIVATED);
    }

    return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
}

/**
 * Upgrade a module
 * @author Jim McDonald
 * @param int 'id' module id
 * @return bool true
 */
function modules_admin_upgrade()
{
    // Security and sanity checks
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    $interactive_upgrade = SessionUtil::getVar('interactive_upgrade');
    $interactive_upgrade = (empty($interactive_upgrade)) ? false : true;
    if ($interactive_upgrade==false) {
        $id = (int)FormUtil::getPassedValue('id', null, 'GET');
        $startnum = (int)FormUtil::getPassedValue('startnum', null, 'GET');
        $letter = FormUtil::getPassedValue('letter', null, 'GET');
        $state = FormUtil::getPassedValue('state', null, 'GET');
        SessionUtil::setVar('modules_id', $id);
        SessionUtil::setVar('modules_startnum', $startnum);
        SessionUtil::setVar('modules_letter', $letter);
        SessionUtil::setVar('modules_state', $state);
        $activate = false;
    } else {
        $id       = SessionUtil::getVar('modules_id');
        $startnum = SessionUtil::getVar('modules_startnum');
        $letter   = SessionUtil::getVar('modules_letter');
        $state    = SessionUtil::getVar('modules_state');
        $activate = (bool)FormUtil::getPassedValue('activate', null, 'POST');
    }
    if (empty($id) || !is_numeric($id)) {
        return LogUtil::registerError (_MODULES_NOMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // Upgrade module
    $res = pnModAPIFunc('Modules', 'admin', 'upgrade',
                        array('id' => $id,
                              'interactive_upgrade' => $interactive_upgrade));
    if (is_bool($res) && $res==true) {
        // Success
        SessionUtil::delVar('modules_id');
        SessionUtil::delVar('modules_startnum');
        SessionUtil::delVar('modules_letter');
        SessionUtil::delVar('modules_state');
        SessionUtil::setVar('interactive_upgrade', false);
        LogUtil::registerStatus (_MODULES_UPGRADED);
        if ($activate==true) {
            if (pnModAPIFunc('Modules', 'admin', 'setstate',
                             array('id' => $id,
                                   'state' => PNMODULE_STATE_ACTIVE))) {
                // Success
                LogUtil::registerStatus (_ACTIVATED);
            }
        }
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } elseif (is_bool($res)) {
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } else {
        return $res;
    }
}

/**
 * Deactivate a module
 * @author Jim McDonald
 * @param int 'id' module id
 * @return bool true
 */
function modules_admin_deactivate()
{
    // Security and sanity checks
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    $id = (int)FormUtil::getPassedValue('id', null, 'GET');
    $startnum = (int)FormUtil::getPassedValue('startnum', null, 'GET');
    $letter = FormUtil::getPassedValue('letter', null, 'GET');
    $state = FormUtil::getPassedValue('state', null, 'GET');
    if (empty($id) || !is_numeric($id)) {
        return LogUtil::registerError (_MODULES_NOMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // check if the modules is the systems start module
    $modinfo = pnModGetInfo($id);
    if ($modinfo==false) {
        return LogUtil::registerError (_MODULES_NOSUCHMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }
    $startmod = pnConfigGetVar('startpage');
    if ($startmod == $modinfo['name']) {
        return LogUtil::registerError (_MODULES_MODULEISSTARTMODULE, null, pnModURL('Modules', 'admin', 'view'));
    }

    $coremodules = array('Modules', 'Permissions', 'Groups', 'Blocks', 'ObjectData', 'Users', 'Theme', 'Admin', 'Settings', 'Categories');
    if (in_array($modinfo['name'], $coremodules)) {
        return LogUtil::registerError (_MODULES_MANDATORYCOREMODULE, null, pnModURL('Modules', 'admin', 'view'));
    }

    // Update state
    if (pnModAPIFunc('Modules', 'admin', 'setstate',
                     array('id' => $id,
                           'state' => PNMODULE_STATE_INACTIVE))) {
        // Success
        LogUtil::registerStatus (_MODULES_DEACTIVATED);
    }

    return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
}

/**
 * Remove a module
 * @author Jim McDonald
 * @param int 'id' module id
 * @return bool true if successful
 */
function modules_admin_remove()
{
    // Get parameters from whatever input we need
    $id  = (int)FormUtil::getPassedValue ('id', 0);
    $objectid = (int)FormUtil::getPassedValue ('objectid', 0);
    $confirmation = (bool)FormUtil::getPassedValue ('confirmation', false);
    $dependents = (array)FormUtil::getPassedValue ('dependents');
    $startnum = (int)FormUtil::getPassedValue ('startnum');
    $letter = FormUtil::getPassedValue ('letter');
    $state = FormUtil::getPassedValue ('state');
    if ($objectid) {
        $id = $objectid;
    }

    $interactive_remove = SessionUtil::getVar('interactive_remove');
    $interactive_remove = (empty($interactive_remove)) ? false : true;

    if ($interactive_remove == false) {
        SessionUtil::setVar('modules_id', $id);
        SessionUtil::setVar('modules_startnum', $startnum);
        SessionUtil::setVar('modules_letter', $letter);
        SessionUtil::setVar('modules_state', $state);
    } else {
        $id       = SessionUtil::getVar('modules_id');
        $startnum = SessionUtil::getVar('modules_startnum');
        $letter   = SessionUtil::getVar('modules_letter');
        $state    = SessionUtil::getVar('modules_state');
        $confirmation = 1;
    }

    if (empty($id) || !is_numeric($id) || !pnModGetInfo($id)) {
        return LogUtil::registerError (_MODULES_NOMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    // Check for confirmation.
    if (empty($confirmation)) {
        // No confirmation yet
        // Create output object
        $pnRender = pnRender::getInstance('Modules', false);

        // Add a hidden field for the item ID to the output
        $pnRender->assign('id', $id);

        // assign any dependencies - filtering out non-active module dependents
        $dependents = pnModAPIFunc('Modules', 'admin', 'getdependents', array('modid' => $id));
        foreach ($dependents as $key => $dependent) {
            $modinfo = pnModGetInfo($dependent['modid']);
            if (!pnModAvailable($modinfo['name'])) {
                unset($dependents[$key]);
            } else {
                $dependents[$key] = array_merge($dependents[$key], $modinfo);
            }
        }
	    // check the blocks module for existing blocks
	    $blocks = pnModAPIFunc('Blocks', 'user', 'getall', array('modid' => $id));
        $pnRender->assign('hasBlocks', !empty($blocks));

        $pnRender->assign('dependents', $dependents);
        $pnRender->assign('startnum', $startnum);
        $pnRender->assign('letter', $letter);
        $pnRender->assign('state', $state);

        // Return the output that has been generated by this function
        return $pnRender->fetch('modules_admin_remove.htm');
    }

    // If we get here it means that the user has confirmed the action

    // Security and sanity checks
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }


    // check the blocks module for existing blocks
    $blocks = pnModAPIFunc('Blocks', 'user', 'getall', array('modid' => $id));
    if (!empty($blocks)) {
        return LogUtil::registerError (_MODULES_BLOCKSEXIST, null, pnModURL('Modules', 'admin', 'view'));
    }

    // remove dependent modules

    foreach ($dependents as $dependent) {
        if (!pnModAPIFunc('Modules', 'admin', 'remove', array('id' => $dependent))) {
            return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
        }
    }

    // Now we've removed dependents remove the main module
    $res = pnModAPIFunc('Modules', 'admin', 'remove',
                        array('id' => $id,
                              'interactive_remove' => $interactive_remove));
    if (is_bool($res) && $res == true) {
        // Success
        SessionUtil::delVar('modules_id');
        SessionUtil::delVar('modules_startnum');
        SessionUtil::delVar('modules_letter');
        SessionUtil::delVar('modules_state');
        SessionUtil::delVar('interactive_remove');
        LogUtil::registerStatus (_MODULES_REMOVED);
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } elseif (is_bool($res)) {
        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    } else {
        return $res;
    }
}

/**
 * Re-generate modules list
 * @author Jim McDonald
 * @author Jrg Napp
 */
function modules_admin_regenerate()
{
    // Security check
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    $filemodules = pnModAPIFunc('Modules', 'admin', 'getfilemodules');
    $inconsistencies = pnModAPIFunc('Modules', 'admin', 'checkconsistency', array('filemodules' => $filemodules));

    if (empty($inconsistencies['errors_modulenames']) && empty($inconsistencies['errors_displaynames'])) {
        // No inconsistencies, so we can regenerate modules
        if (pnModAPIFunc('Modules', 'admin', 'regenerate', array('filemodules' => $filemodules))) {
            // Success
            LogUtil::registerStatus (_MODULES_REGENERATED);
        }


        // get variables to pass back to pager
        $startnum = (int)FormUtil::getPassedValue('startnum', null, 'GET');
        $letter = FormUtil::getPassedValue('letter', null, 'GET');
        $state = FormUtil::getPassedValue('state', null, 'GET');

        return pnRedirect(pnModURL('Modules', 'admin', 'view', array('startnum' => $startnum, 'letter' => $letter, 'state' => $state)));
    }

    // Create output object
    $pnRender = pnRender::getInstance('Modules', false);

    $pnRender->assign('errors_modulenames', $inconsistencies['errors_modulenames']);
    $pnRender->assign('errors_displaynames', $inconsistencies['errors_displaynames']);
    $pnRender->assign('returnurl', $returnurl);

    return $pnRender->fetch('modules_admin_regenerate_errors.htm');

}

/**
 * display available hooks
 * @author Mark West
 * @param int 'id' module id
 * @return string HTML output string
  */
function modules_admin_hooks()
{
    // get our input
    $id = (int)FormUtil::getPassedValue('id', null, 'GET');

    // check the input
    if (!is_numeric($id)) {
        return LogUtil::registerError (_MODARGSERRROR, null, pnModURL('Modules', 'admin', 'view'));
    }

    // get the modules information
    $modinfo = pnModGetInfo($id);
    if ($modinfo == false) {
        return LogUtil::registerError (_MODULES_NOSUCHMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    if (!SecurityUtil::checkPermission('Modules::', "$modinfo[name]::$id", ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
    $pnRender = pnRender::getInstance('Modules', false);

    // assign the modinfo to the template
    $pnRender->assign('modinfo', $modinfo);

    // add module id to form
    $pnRender->assign('id', $id);

    $hooks = pnModAPIFunc('Modules', 'admin', 'getmoduleshooks',
                          array('modid' => $id));

    $pnRender->assign('hooks', $hooks);

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_hooks.htm');

}

/**
 * display available hooks, extended version
 * @author Mark West
 * @author Frank Schummertz
 * @param int 'id' module id
 * @return string HTML output string
  */
function modules_admin_extendedhooks()
{
    // get our input
    $id = (int)FormUtil::getPassedValue('id', null, 'GET');

    // check the input
    if (!is_numeric($id)) {
        return LogUtil::registerError (_MODARGSERROR, null, pnModURL('Modules', 'admin', 'view'));
    }

    // get the modules information
    $modinfo = pnModGetInfo($id);
    if ($modinfo == false) {
        return LogUtil::registerError (_MODULES_NOSUCHMODID, 404, pnModURL('Modules', 'admin', 'view'));
    }

    if (!SecurityUtil::checkPermission('Modules::', "$modinfo[name]::$id", ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
    $pnRender = pnRender::getInstance('Modules', false);

    // assign the modinfo to the template
    $pnRender->assign('modinfo', $modinfo);

    // add module id to form
    $pnRender->assign('id', $id);

    $grouped_hooks = pnModAPIFunc('Modules', 'admin', 'getextendedmoduleshooks',
                                  array('modid' => $id));
    $pnRender->assign('grouped_hooks', $grouped_hooks);

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_extendedhooks.htm');

}

/**
 * This is a standard function to modify the configuration parameters of the
 * module
 * @author Mark West
 * @since 1.14
 * @return string HTML string
 */
function modules_admin_modifyconfig()
{
    // Security check
    if (!SecurityUtil::checkPermission('Modules::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Modules', false);

    // assign all the module vars
    $pnRender->assign(pnModGetVar('Modules'));

    // load support for legacy modules
    $pnRender->assign('loadlegacy', pnConfigGetVar('loadlegacy'));

    // Return the output that has been generated by this function
    return $pnRender->fetch('modules_admin_modifyconfig.htm');
}

/**
 * This is a standard function to update the configuration parameters of the
 * module given the information passed back by the modification form
 * @author Mark West
 * @since 1.14
 * @return bool true
 */
function modules_admin_updateconfig()
{
    // Security check
    if (!SecurityUtil::checkPermission('Modules::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Confirm authorisation code.
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('Modules','admin','view'));
    }

    // Update module variables.
    $itemsperpage = (int)FormUtil::getPassedValue('itemsperpage', 25, 'POST');
    if (!is_integer($itemsperpage) || $itemsperpage < 1) {
        LogUtil::registerError (_MODULES_ITEMSPERPAGENUMERIC);
        $itemsperpage = (int)$itemsperpage;
        if ($itemsperpage < 1) {
            $itemsperpage = 25;
        }
    }
    pnModSetVar('Modules', 'itemsperpage', $itemsperpage);

    $loadlegacy = (bool)FormUtil::getPassedValue('loadlegacy', false, 'POST');
    pnConfigSetVar('loadlegacy', $loadlegacy);

    // the module configuration has been updated successfuly
    LogUtil::registerStatus (_CONFIGUPDATED);

    // Let any other modules know that the modules configuration has been updated
    pnModCallHooks('module','updateconfig', 'Modules', array('module' => 'Modules'));

    // This function generated no output, and so now it is complete we redirect
    // the user to an appropriate page for them to carry on their work
    return pnRedirect(pnModURL('Modules', 'admin', 'view'));
}
