<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24934 2008-12-01 18:21:50Z Landseer $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Modules
 */

/**
 * initialise the Modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance.
 * This function MUST exist in the pninit file for a module
 *
 * @author       Mark West
 * @return       bool       true on success, false otherwise
 */
function modules_init()
{
    // modules
    if (!DBUtil::createTable('modules')) {
        return false;
    }

    // module_vars
    if (!DBUtil::createTable('module_vars')) {
        return false;
    }

    // hooks
    if (!DBUtil::createTable('hooks')) {
        return false;
    }

    // module_deps
    if (!DBUtil::createTable('module_deps')) {
        return false;
    }

    // populate default data
    modules_defaultdata();
    pnModSetVar('Modules', 'itemsperpage', 25);
    pnConfigSetVar('loadlegacy', 0);
    // Initialisation successful
    return true;
}


/**
 * upgrade the modules module from an old version
 *
 * This function can be called multiple times
 * This function MUST exist in the pninit file for a module
 *
 * @author       Mark West
 * @return       bool       true on success, false otherwise
 */
function modules_upgrade($oldversion)
{
    // Upgrade dependent on old version number
    switch($oldversion) {
        // version 2.0 was supplied with PN .72x
        case 2.0:
            pnModSetVar('Modules', 'itemsperpage', 25);
            // unlike a new install we'll assume that we need legacy support on
            pnConfigSetVar('loadlegacy', 1);
            return modules_upgrade(2.5);
        case 2.5:
            // v2.5 shipped with .76x
            // nothing to do here....
            return modules_upgrade(2.6);
        case 2.6:
            // module_deps
            if (!DBUtil::changeTable('module_deps')) {
               return false;
            }
            return modules_upgrade(3.0);
            break;
        case 3.0:
            return modules_upgrade(3.1);
            break;
        case 3.1:
            // in mysql 5 strict mode we need to set any null values before changing the table
            $table = DBUtil::getLimitedTableName('hooks');
            DBUtil::executeSQL("UPDATE {$table} SET pn_smodule = '' WHERE pn_smodule IS NULL");
            DBUtil::executeSQL("UPDATE {$table} SET pn_stype = '' WHERE pn_stype IS NULL");
            if (!DBUtil::changeTable('hooks')) {
               return false;
            }
            return modules_upgrade(3.2);
            break;
        case 3.2:
            // new field profile_capable
            if (!DBUtil::changeTable('modules')) {
               return false;
            }
            // set this new flag for existing profile modules
            $table = DBUtil::getLimitedTableName('modules');
            DBUtil::executeSQL("UPDATE {$table} SET pn_profile_capable = '1' WHERE pn_name = 'Profile' OR pn_name = 'MyProfile'");
            return modules_upgrade(3.3);
            break;
    }

    // Update successful
    return true;
}


/**
 * delete the modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * This function MUST exist in the pninit file for a module
 *
 * Since the modules module should never be deleted we'all always return false here
 * @author       Mark West
 * @return       bool       false
 */
function modules_delete()
{
    // Deletion not allowed
    return false;
}

/**
 * create the default data for the Modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Mark West
 * @return       bool       false
 */
function modules_defaultdata()
{
    // modules module
    $record = array();
    $record['name']          = 'Modules';
    $record['type']          = 2;
    $record['displayname']   = ''._MODULES_1_c.'';
    $record['description']   = ''._MODULES_1_d.'';
    $record['regid']         = 1;
    $record['directory']     = 'Modules';
    $record['version']       = 3.3;
    $record['admin_capable'] = 1;
    $record['user_capable']  = 0;
    $record['state']         = 3;

    DBUtil::insertObject($record, 'modules');
}
/**
 * update the default data for the Modules module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Frank Schummertz
 * @return       none
 */
function modules_updatedefaultdata($lang)
{
    // set the default data for the Modules module
   
    $pntables = pnDBGetTables();
    $modcolumn = $pntables['modules_column'];
    
    $where = 'WHERE '.$modcolumn['name'].'=\'Modules\'';
    $record = DBUtil::selectObject('modules', $where);
    $record['version']         = '3.3';
    $record['admin_capable']   = 1;
    $record['user_capable']    = 0;
    $record['profile_capable'] = 0;
    $record['message_capable'] = 0;
    $record['state']           = 3;
    DBUtil::updateObject($record, 'modules');
}

function modules_init_changestructure()
{
    // Apply the table transform
    // modules
    $sql = "pn_id I PRIMARY AUTO,
            pn_name C(64) NOTNULL DEFAULT '',
            pn_type I1 NOTNULL,
            pn_displayname C(64) NOTNULL DEFAULT '',
            pn_description C(255) NOTNULL DEFAULT '',
            pn_regid I NOTNULL DEFAULT '0',
            pn_directory C(64) NOTNULL DEFAULT '',
            pn_version C(10) NOTNULL DEFAULT '0',
            pn_official L NOTNULL DEFAULT 0,
            pn_author C(255) NOTNULL DEFAULT '',
            pn_contact C(255) NOTNULL DEFAULT '',
            pn_admin_capable L NOTNULL DEFAULT 0,
            pn_user_capable L NOTNULL DEFAULT 0,
            pn_profile_capable L NOTNULL DEFAULT 0,
            pn_message_capable L NOTNULL DEFAULT 0,
            pn_state I1 NOTNULL DEFAULT '0',
            pn_credits C(255) NOTNULL DEFAULT '',
            pn_help C(255) NOTNULL DEFAULT '',
            pn_changelog C(255) NOTNULL DEFAULT '',
            pn_license C(255) NOTNULL DEFAULT '',
            pn_securityschema X DEFAULT ''
    ";

    DBUtil::changeTable('modules', $sql);
    DBUtil::changeTable('hooks');
    DBUtil::changeTable('module_deps');
}
