<?php
/**
 * Zikula Application Framework
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Permissions
 * @license http://www.gnu.org/copyleft/gpl.html
 */

/**
 * initialise the permissions module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance.
 * This function MUST exist in the pninit file for a module
 *
 * @author       Mark West
 * @return       bool       true on success, false otherwise
 */
function permissions_init()
{
    if (!DBUtil::createTable('group_perms')) {
        return false;
    }

    // Create any default for this module
    permissions_defaultdata();
    // Initialisation successful
    return true;
}


/**
 * upgrade the permissions module from an old version
 *
 * @author       Mark West
 * @author       Drak
 * @return       bool       true on success, false otherwise
 */
function permissions_upgrade($oldversion)
{
    if ($oldversion == '0.4'){
        pnModSetVar('Permissions', 'filter', 1);
        pnModSetVar('Permissions', 'warnbar', 1);
        pnModSetVar('Permissions', 'rowview', 20);
        pnModSetVar('Permissions', 'rowedit', 20);
        pnModSetVar('Permissions', 'lockadmin', 1);
        pnModSetVar('Permissions', 'adminid', 1);
        // we no longer have user_perms in pntables so use group_perms instead
        $prefix = pnDBGetTablePrefix('group_perms');
        $sql = "DROP TABLE {$prefix}_user_perms";
        return DBUtil::executeSQL($sql);
    }

    // Update successful
    return true;
}


/**
 * delete the permissions module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * Since the permissions module should never be deleted we'all always return false here
 * @author       Mark West
 * @return       bool       false
 */
function permissions_delete()
{
    // Deletion not allowed
    return false;
}

/**
 * create the default data for the permissions module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 *
 * @author       Mark West
 * @return       bool       false
 */
function permissions_defaultdata()
{
    $record = array();
    $record['gid']       = ''._GROUP_PERMS_1_a.'';
    $record['sequence']  = ''._GROUP_PERMS_1_b.'';
    $record['realm']     = ''._GROUP_PERMS_1_c.'';
    $record['component'] = ''._GROUP_PERMS_1_d.'';
    $record['instance']  = ''._GROUP_PERMS_1_e.'';
    $record['level']     = ''._GROUP_PERMS_1_f.'';
    $record['bond']      = ''._GROUP_PERMS_1_g.'';
    DBUtil::insertObject($record, 'group_perms', 'pid');

    $record = array();
    $record['gid']       = ''._GROUP_PERMS_2_a.'';
    $record['sequence']  = ''._GROUP_PERMS_2_b.'';
    $record['realm']     = ''._GROUP_PERMS_2_c.'';
    $record['component'] = ''._GROUP_PERMS_2_d.'';
    $record['instance']  = ''._GROUP_PERMS_2_e.'';
    $record['level']     = ''._GROUP_PERMS_2_f.'';
    $record['bond']      = ''._GROUP_PERMS_2_g.'';
    DBUtil::insertObject($record, 'group_perms', 'pid');

    $record = array();
    $record['gid']       = ''._GROUP_PERMS_3_a.'';
    $record['sequence']  = ''._GROUP_PERMS_3_b.'';
    $record['realm']     = ''._GROUP_PERMS_3_c.'';
    $record['component'] = ''._GROUP_PERMS_3_d.'';
    $record['instance']  = ''._GROUP_PERMS_3_e.'';
    $record['level']     = ''._GROUP_PERMS_3_f.'';
    $record['bond']      = ''._GROUP_PERMS_3_g.'';
    DBUtil::insertObject($record, 'group_perms', 'pid');

    $record = array();
    $record['gid']       = ''._GROUP_PERMS_4_a.'';
    $record['sequence']  = ''._GROUP_PERMS_4_b.'';
    $record['realm']     = ''._GROUP_PERMS_4_c.'';
    $record['component'] = ''._GROUP_PERMS_4_d.'';
    $record['instance']  = ''._GROUP_PERMS_4_e.'';
    $record['level']     = ''._GROUP_PERMS_4_f.'';
    $record['bond']      = ''._GROUP_PERMS_4_g.'';
    DBUtil::insertObject($record, 'group_perms', 'pid');

    $record = array();
    $record['gid']       = ''._GROUP_PERMS_5_a.'';
    $record['sequence']  = ''._GROUP_PERMS_5_b.'';
    $record['realm']     = ''._GROUP_PERMS_5_c.'';
    $record['component'] = ''._GROUP_PERMS_5_d.'';
    $record['instance']  = ''._GROUP_PERMS_5_e.'';
    $record['level']     = ''._GROUP_PERMS_5_f.'';
    $record['bond']      = ''._GROUP_PERMS_5_g.'';
    DBUtil::insertObject($record, 'group_perms', 'pid');

    pnModSetVar('Permissions', 'filter', 1);
    pnModSetVar('Permissions', 'warnbar', 1);
    pnModSetVar('Permissions', 'rowview', 20);
    pnModSetVar('Permissions', 'rowedit', 20);
    pnModSetVar('Permissions', 'lockadmin', 1);
    pnModSetVar('Permissions', 'adminid', 1);
}
