<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnadmin.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage SecurityCenter
*/

/**
 * the main administration function
 * This function is the default function, and is called whenever the
 * module is initiated without defining arguments.  As such it can
 * be used for a number of things, but most commonly it either just
 * shows the module menu and returns or calls whatever the module
 * designer feels should be the default function (often this is the
 * view() function)
 * @author Mark West
 * @return string HTML string
 */
function securitycenter_admin_main()
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('SecurityCenter');

    // Return the output that has been generated by this function
    return $pnRender->fetch('securitycenter_admin_main.htm');
}

/**
 * delete item
 * This is a standard function that is called whenever an administrator
 * wishes to delete a current module item.  Note that this function is
 * the equivalent of both of the modify() and update() functions above as
 * it both creates a form and processes its output.  This is fine for
 * simpler functions, but for more complex operations such as creation and
 * modification it is generally easier to separate them into separate
 * functions.  There is no requirement in the Zikula MDG to do one or the
 * other, so either or both can be used as seen appropriate by the module
 * developer
 * @author Mark West
 * @param int $args['hid'] the id of the item to be deleted
 * @param bool $args['confirmation'] confirmation that this item can be deleted
 * @return mixed HTML string if no confirmation, true if successful, false otherwise
 */
function securitycenter_admin_delete($args)
{
    $hid = FormUtil::getPassedValue('hid', isset($args['hid']) ? $args['hid'] : null, 'REQUEST');
    $objectid = FormUtil::getPassedValue('objectid', isset($args['objectid']) ? $args['objectid'] : null, 'REQUEST');
    $confirmation = FormUtil::getPassedValue('confirmation', null, 'POST');
    if (!empty($objectid)) {
        $hid = $objectid;
    }

    // Get the current item
    $item = pnModAPIFunc('SecurityCenter', 'user', 'get', array('hid' => $hid));

    if ($item == false) {
        return LogUtil::registerError(_NOSUCHITEM, 404);
    }

    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', "$item[hid]::$item[hacktime]", ACCESS_DELETE)) {
        return LogUtil::registerPermissionError();
    }

    // Check for confirmation.
    if (empty($confirmation)) {
        // No confirmation yet - display a suitable form to obtain confirmation
        // of this action from the user
        $pnRender = pnRender::getInstance('SecurityCenter', false);
        $pnRender->assign('hid', $hid);
        return $pnRender->fetch('securitycenter_admin_delete.htm');
    }

    // If we get here it means that the user has confirmed the action

    // Confirm authorisation code.
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('SecurityCenter','admin','view'));
    }

    // Call the API to delete the item
    if (pnModAPIFunc('SecurityCenter', 'admin', 'delete', array('hid' => $hid))) {
        // Success
        LogUtil::registerStatus (_DELETESUCCEDED);
    }

    return pnRedirect(pnModURL('SecurityCenter', 'admin', 'view'));
}

/**
 * view items in db
 * @author Mark West
 * @param int $startnum number of item to start view from
 * @return string HTML string
 */
function securitycenter_admin_view()
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $startnum = FormUtil::getPassedValue('startnum', isset($args['startnum']) ? $args['startnum'] : null, 'GET');

    // Create output object
    $pnRender = pnRender::getInstance('SecurityCenter', false);

    // Get all items
    $items = pnModAPIFunc('SecurityCenter', 'user', 'getall',
                          array('startnum' => $startnum,
                                'numitems' => pnModGetVar('SecurityCenter', 'itemsperpage')));

    $hackattempts = array();
    if ($items) {
        foreach ($items as $item) {

            // Get the full item
            $fullitem = pnModAPIFunc('SecurityCenter', 'user', 'get', array('hid' => $item['hid']));

            $fullitem['hacktime'] = ml_ftime(_DATETIMEBRIEF, $fullitem['hacktime']);
            if ($fullitem['userid'] == 0) {
                $fullitem['userid'] = pnConfigGetVar('anonymous');
            } else {
                $fullitem['userid'] = pnUserGetVar('uname', $fullitem['userid']);
            }

            // Add users options for the item.
            $options = array();
            if (SecurityUtil::checkPermission('SecurityCenter::', "$item[hid]::$item[hacktime]", ACCESS_EDIT)) {
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'browserinfo')),
                                   'title' => _SECURITYCENTER_BROWSERINFO);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'requestarray')),
                                   'title' => _SECURITYCENTER_REQUESTARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'getarray')),
                                   'title' => _SECURITYCENTER_GETARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'postarray')),
                                   'title' => _SECURITYCENTER_POSTARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'serverarray')),
                                   'title' => _SECURITYCENTER_SERVERARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'envarray')),
                                   'title' => _SECURITYCENTER_ENVARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'cookiearray')),
                                   'title' => _SECURITYCENTER_COOKIEARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'filesarray')),
                                   'title' => _SECURITYCENTER_FILESARRAY);
                $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'display', array('hid' => $item['hid'], 'arraytype' => 'sessionarray')),
                                   'title' => _SECURITYCENTER_SESSIONARRAY);
                if (SecurityUtil::checkPermission('SecurityCenter::', "$item[hid]::$item[hacktime]", ACCESS_DELETE)) {
                    $options[] = array('url' => pnModURL('SecurityCenter', 'admin', 'delete', array('hid' => $item['hid'])),
                                       'title' => _DELETE);
                }
                $fullitem['options'] = $options;
            }
            $hackattempts[] = $fullitem;
        }
    }
    $pnRender->assign('hackattempts', $hackattempts);

    // Assign the values for the smarty plugin to produce a pager.
    $pnRender->assign('pager', array('numitems' => pnModAPIFunc('SecurityCenter', 'user', 'countitems'),
                                     'itemsperpage' => pnModGetVar('SecurityCenter', 'itemsperpage')));

    return $pnRender->fetch('securitycenter_admin_view.htm');
}

/**
 * This is a standard function to modify the configuration parameters of the
 * module
 * @author Mark West
 * @return string HTML string
 */
function securitycenter_admin_modifyconfig()
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('SecurityCenter', false);

    $pnRender->assign('itemsperpage', pnModGetVar('SecurityCenter', 'itemsperpage'));

    // assign all of our vars
    $vars = pnModGetVar(PN_CONFIG_MODULE);
    $pnRender->assign($vars);

    // Return the output that has been generated by this function
    return $pnRender->fetch('securitycenter_admin_modifyconfig.htm');
}

/**
 * This is a standard function to update the configuration parameters of the
 * module given the information passed back by the modification form
 * @author Mark West
 * @see securitycenter_admin_modifyconfig()
 * @param int enableanticracker
 * @param int emailhackattempt
 * @param int loghackattempttodb
 * @param int onlysendsummarybyemail
 * @param string summarycontent
 * @param string fullcontent
 * @param int itemsperpage
 * @param int usehtaccessbans
 * @param int filtergetvars
 * @param int filtercookievars
 * @param int filterpostvars
 * @param int filterarrays
 * @param int extrapostprotection
 * @param int extragetprotection
 * @param int checkmultipost
 * @param int maxmultipost
 * @param int cpuloadmonitor
 * @param int cpumaxload
 * @param string ccisessionpath
 * @param string htaccessfilelocation
 * @param int nocookiebanthreshold
 * @param int nocookiewarningthreshold
 * @param int fastaccessbanthreshold
 * @param int fastaccesswarnthreshold
 * @param int javababble
 * @param int javaencrypt
 * @param int preservehead
 * @param int outputfilter
 * @return bool true if successful, false otherwise
 */
function securitycenter_admin_updateconfig()
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // Confirm authorisation code.
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError (pnModURL('SecurityCenter','admin','view'));
    }

    // Update module variables.
    $enableanticracker = FormUtil::getPassedValue('enableanticracker', 0, 'POST');
    pnConfigSetVar('enableanticracker', $enableanticracker);

    $emailhackattempt = FormUtil::getPassedValue('emailhackattempt', 0, 'POST');
    pnConfigSetVar('emailhackattempt', $emailhackattempt);

    $loghackattempttodb = FormUtil::getPassedValue('loghackattempttodb', 0, 'POST');
    pnConfigSetVar('loghackattempttodb', $loghackattempttodb);

    $onlysendsummarybyemail = FormUtil::getPassedValue('onlysendsummarybyemail', 0, 'POST');
    pnConfigSetVar('onlysendsummarybyemail', $onlysendsummarybyemail);

    $itemsperpage = FormUtil::getPassedValue('itemsperpage', 10, 'POST');
    pnModSetVar('SecurityCenter', 'itemsperpage', $itemsperpage);

    $usehtaccessbans = FormUtil::getPassedValue('usehtaccessbans', 1, 'POST');
    pnConfigSetVar('usehtaccessbans', $usehtaccessbans);

    $filtergetvars = FormUtil::getPassedValue('filtergetvars', 1, 'POST');
    pnConfigSetVar('filtergetvars', $filtergetvars);

    $filtercookievars = FormUtil::getPassedValue('filtercookievars', 1, 'POST');
    pnConfigSetVar('filtercookievars', $filtercookievars);

    $filterpostvars = FormUtil::getPassedValue('filterpostvars', 0, 'POST');
    pnConfigSetVar('filterpostvars', $filterpostvars);

    $filterarrays = FormUtil::getPassedValue('filterarrays', 0, 'POST');
    pnConfigSetVar('filterarrays', $filterarrays);

    $extrapostprotection = FormUtil::getPassedValue('extrapostprotection', 0, 'POST');
    pnConfigSetVar('extrapostprotection', $extrapostprotection);

    $extragetprotection = FormUtil::getPassedValue('extragetprotection', 0, 'POST');
    pnConfigSetVar('extragetprotection', $extragetprotection);

    $checkmultipost = FormUtil::getPassedValue('checkmultipost', 1, 'POST');
    pnConfigSetVar('checkmultipost', $checkmultipost);

    $maxmultipost = FormUtil::getPassedValue('maxmultipost', 4, 'POST');
    pnConfigSetVar('maxmultipost', $maxmultipost);

    $zipcompress = FormUtil::getPassedValue('zipcompress', 0, 'POST');
    pnConfigSetVar('zipcompress', $zipcompress);

    $compresslevel = FormUtil::getPassedValue('compresslevel', 9, 'POST');
    pnConfigSetVar('compresslevel', $compresslevel);

    $cpuloadmonitor = FormUtil::getPassedValue('cpuloadmonitor', 0, 'POST');
    pnConfigSetVar('cpuloadmonitor', $cpuloadmonitor);

    $cpumaxload = FormUtil::getPassedValue('cpumaxload', 10.0, 'POST');
    pnConfigSetVar('cpumaxload', $cpumaxload);

    $ccisessionpath = FormUtil::getPassedValue('ccisessionpath', '', 'POST');
    pnConfigSetVar('ccisessionpath', $ccisessionpath);

    $htaccessfilelocation = FormUtil::getPassedValue('htaccessfilelocation', '.htaccess', 'POST');
    pnConfigSetVar('htaccessfilelocation', $htaccessfilelocation);

    $nocookiebanthreshold = FormUtil::getPassedValue('nocookiebanthreshold', 10, 'POST');
    pnConfigSetVar('nocookiebanthreshold', $nocookiebanthreshold);

    $nocookiewarnthreshold = FormUtil::getPassedValue('nocookiewarnthreshold', 2, 'POST');
    pnConfigSetVar('nocookiewarnthreshold', $nocookiewarnthreshold);

    $fastaccessbanthreshold = FormUtil::getPassedValue('fastaccessbanthreshold', 40, 'POST');
    pnConfigSetVar('fastaccessbanthreshold', $fastaccessbanthreshold);

    $fastaccesswarnthreshold = FormUtil::getPassedValue('fastaccesswarnthreshold', 10, 'POST');
    pnConfigSetVar('fastaccesswarnthreshold', $fastaccesswarnthreshold);

    $javababble = FormUtil::getPassedValue('javababble', 0, 'POST');
    pnConfigSetVar('javababble', $javababble);

    $javaencrypt = FormUtil::getPassedValue('javaencrypt', 0, 'POST');
    pnConfigSetVar('javaencrypt', $javaencrypt);

    $preservehead = FormUtil::getPassedValue('preservehead', 0, 'POST');
    pnConfigSetVar('preservehead', $preservehead);

    $outputfilter = FormUtil::getPassedValue('outputfilter', 0, 'POST');
    pnConfigSetVar('outputfilter', $outputfilter);

    // to do set some defaults here possbily read default content from file
    // so it's not repeated in code - markwest
    $summarycontent = FormUtil::getPassedValue('summarycontent', '', 'POST');
    pnConfigSetVar('summarycontent', $summarycontent);
    $fullcontent = FormUtil::getPassedValue('fullcontent', '', 'POST');
    pnConfigSetVar('fullcontent', $fullcontent);

    // Let any other modules know that the modules configuration has been updated
    pnModCallHooks('module','updateconfig', 'SecurityCenter', array('module' => 'SecurityCenter'));

    // the module configuration has been updated successfuly
    LogUtil::registerStatus (_CONFIGUPDATED);

    // This function generated no output, and so now it is complete we redirect
    // the user to an appropriate page for them to carry on their work
    return pnRedirect(pnModURL('SecurityCenter','admin', 'main'));
}

/*
 * output contents of http array
 * @author Mark West
 * @param int $args['hid'] hack id
 * @param string $args['arraytype'] type of array to output
 * @return string HTML output string
*/
function securitycenter_admin_display($args)
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $hid = FormUtil::getPassedValue('hid', isset($args['hid']) ? $args['hid'] : null, 'GET');
    $arraytype = FormUtil::getPassedValue('arraytype', isset($args['arraytype']) ? $args['arraytype'] : null, 'GET');

    if (empty($hid) ||
        empty($arraytype)) {
        return LogUtil::registerError (_MODARGSERROR);
    }

    // Create output object
    $pnRender = pnRender::getInstance('SecurityCenter', false);

    // assign the page title
    $pnRender->assign('title', strtoupper($arraytype));

    // Get the item from our API
    $item = pnModAPIFunc('SecurityCenter', 'user', 'get', array('hid' => $hid));
    // extract the data we serialised in the db
    $variablearray = unserialize($item[$arraytype]);

    $arrayvariables = array();
    if (is_array($variablearray)) {
        // output the variables into the table
        while (list ($key, $value) = each ($variablearray)) {
            $arrayvariables[] = array('key' => $key, 'value' => $value);
        }
    }
    $pnRender->assign('arrayvariables', $arrayvariables);

    return $pnRender->fetch('securitycenter_admin_display.htm');
}


/*
 * Generic view function - used for log events
 * @author Robert Gasch
 * @return string HTML output string
*/
function securitycenter_admin_viewobj ()
{
    // Security check
    if (!SecurityUtil::checkPermission('SecurityCenter::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    $filterdefault = array('uid' => null, 'component' => null, 'module' => null, 'type' => null);

    $ot       = FormUtil::getPassedValue('ot', 'log_event', 'GETPOST');
    $sort     = FormUtil::getPassedValue('sort', 'lge_date DESC', 'GETPOST');
    $filter   = FormUtil::getPassedValue('filter', $filterdefault, 'GETPOST');
    $startnum = (int)FormUtil::getPassedValue('startnum', 0, 'GET');
    $pagesize = (int)pnModGetVar ('SecurityCenter', 'pagesize', 25);

    // load array class
    if (!($class = Loader::loadClassFromModule ('SecurityCenter', $ot, true)))
        return pn_exit ("Unable to load class [" . DataUtil::formatForDisplay($ot) . "] ...");

    // instantiate object, generate where clause and select
    $objArray = new $class ();
    $where = $objArray->genFilter ();
    $data  = $objArray->get ($where, '', $startnum, $pagesize);

    // Create output object
    $pnRender = pnRender::getInstance('SecurityCenter', false);
    $pnRender->assign('ot', $ot);
    $pnRender->assign('filter', $filter);
    $pnRender->assign('objectArray', $data);

    // Assign the values for the smarty plugin to produce a pager.
    $pager = array();
    $pager['numitems']     = $objArray->getCount ($where);
    $pager['itemsperpage'] = $pagesize;
    $pnRender->assign('startnum', $startnum);
    $pnRender->assign('pager', $pager);

    // fetch output from template
    return $pnRender->fetch('securitycenter_admin_view_' . DataUtil::formatForOS($ot) . '.htm');
}
