<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage AntiCracker
*/

/**
 * initialise the anticracker module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true on success, false otherwise
 */
function securitycenter_init()
{
    // create table
    if (!DBUtil::createTable('sc_anticracker')) {
        return false;
    }

    // create table
    if (!DBUtil::createTable('sc_logevent')) {
        return false;
    }

    // Set up an initial value for a module variable.
    pnModSetVar('SecurityCenter', 'itemsperpage', 10);

    // We use config vars for the rest of the configuration as config vars
    // are available earlier in the PN initialisation process
    pnConfigSetVar('enableanticracker', 1);
    pnConfigSetVar('emailhackattempt', 1);
    pnConfigSetVar('loghackattempttodb', 1);
    pnConfigSetVar('onlysendsummarybyemail', 1);
    pnConfigSetVar('usehtaccessbans', 0);
    pnConfigSetVar('filtergetvars', 1);
    pnConfigSetVar('filtercookievars', 1);
    pnConfigSetVar('filterpostvars', 1);
    pnConfigSetVar('filterarrays', 1);
    pnConfigSetVar('extrapostprotection', 0);
    pnConfigSetVar('extragetprotection', 0);
    pnConfigSetVar('checkmultipost', 0);
    pnConfigSetVar('maxmultipost', 4);
    pnConfigSetVar('zipcompress', 0);
    pnConfigSetVar('compresslevel', 9);
    pnConfigSetVar('cpuloadmonitor', 0);
    pnConfigSetVar('cpumaxload', 10.0);
    pnConfigSetVar('ccisessionpath', '');
    pnConfigSetVar('htaccessfilelocation', '.htaccess');
    pnConfigSetVar('nocookiebanthreshold', 10);
    pnConfigSetVar('nocookiewarningthreshold', 2);
    pnConfigSetVar('fastaccessbanthreshold', 40);
    pnConfigSetVar('fastaccesswarnthreshold', 10);
    pnConfigSetVar('javababble', 0);
    pnConfigSetVar('javaencrypt', 0);
    pnConfigSetVar('preservehead', 0);
    pnConfigSetVar('outputfilter', 1);

    // now lets set the default mail message contents
    // file is read from pnincludes directory
    $summarycontent = implode('', file(getcwd() . '/system/SecurityCenter/pnincludes/summary.txt'));
    pnConfigSetVar('summarycontent', $summarycontent);
    $fullcontent = implode('', file(getcwd() . '/system/SecurityCenter/pnincludes/full.txt'));
    pnConfigSetVar('fullcontent', $fullcontent);

    // Initialisation successful
    return true;
}

/**
 * upgrade the anticracker module from an old version
 * This function can be called multiple times
 * @author Mark West
 * @param int $oldversion version number to upgrade from
 * @return bool true on success, false otherwise
 */
function securitycenter_upgrade($oldversion)
{
    if (!DBUtil::changeTable('sc_anticracker')) {
        return false;
    }

    if (!DBUtil::changeTable('sc_logevent')) {
        return false;
    }

    // Update successful
    return true;
}

/**
 * delete the anticracker module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Mark West
 * @return bool true on success, false otherwise
 */
function securitycenter_delete()
{
    if (!DBUtil::dropTable('sc_anticracker')) {
        return false;
    }

    if (!DBUtil::dropTable('sc_logevent')) {
        return false;
    }

    // Delete any module variables
    pnConfigDelVar('enableanticracker');
    pnConfigDelVar('emailhackattempt');
    pnConfigDelVar('loghackattempttodb');
    pnConfigDelVar('onlysendsummarybyemail');
    pnConfigDelVar('itemsperpage');
    pnConfigDelVar('summarycontent');
    pnConfigDelVar('fullcontent');
    pnConfigDelVar('usehtaccessbans');
    pnConfigDelVar('filtergetvars');
    pnConfigDelVar('filtercookievars');
    pnConfigDelVar('filterpostvars');
    pnConfigDelVar('filterarrays');
    pnConfigDelVar('extrapostprotection');
    pnConfigDelVar('extragetprotection');
    pnConfigDelVar('checkmultipost');
    pnConfigDelVar('maxmultipost');
    pnConfigDelVar('zipcompress');
    pnConfigDelVar('compresslevel');
    pnConfigDelVar('cpuloadmonitor');
    pnConfigDelVar('cpumaxload');
    pnConfigDelVar('ccisessionpath');
    pnConfigDelVar('htaccessfilelocation');
    pnConfigDelVar('nocookiebanthreshold');
    pnConfigDelVar('nocookiewarningthreshold');
    pnConfigDelVar('fastaccessbanthreshold');
    pnConfigDelVar('fastaccesswarnthreshold');
    pnConfigDelVar('javababble');
    pnConfigDelVar('javaencrypt');
    pnConfigDelVar('preservehead');
    pnConfigDelVar('outputfilter');

    // Deletion successful
    return true;
}
