<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnadmin.php 24833 2008-11-05 22:00:22Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Simon Wunderlin
 * @package Zikula_System_Modules
 * @subpackage Settings
 */

/**
 * entry point for the module
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_main($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }
    // create a new output object
    $pnRender = pnRender::getInstance('Settings', false);

    return $pnRender->fetch('settings_admin_main.htm');
}

/**
 * display the main site settings form
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_modifyconfig($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // create a new output object
    $pnRender = pnRender::getInstance('Settings', false);

    // get all config vars and assign them to the template
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    // since config vars are serialised and module vars aren't we
    // need to unserialise each config var in turn before assigning
    // them to the template
    foreach ($configvars as $key => $configvar) {
        $configvars[$key] = $configvar;
    }
    $pnRender->assign('settings', $configvars);

    return $pnRender->fetch('settings_admin_modifyconfig.htm');
}

/**
 * update main site settings
 *
 * @author Zikula development team
 * @return mixed true if successful, false if unsuccessful, error string otherwise
 */
function settings_admin_updateconfig($args) {

    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // get settings from form - do before authid check
    $settings = FormUtil::getPassedValue('settings', null, 'POST');

    // if this form wasnt posted to redirect back
    if ($settings === NULL) {
        return pnRedirect(pnModURL('Settings', 'admin', 'modifyconfig'));
    }

    // confirm the forms auth key
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError ();
    }

    // validate the entry point
    $falseEntryPoints = array('admin.php', 'ajax.php', 'backend.php', 'error.php', 'footer.php', 
                              'header.php', 'install.php', 'modules.php', 'print.php', 'upgrade76.php',
                              'upgrade.php', 'user.php');
    $entryPointExt = pathinfo($settings['entrypoint'], PATHINFO_EXTENSION);
    if (in_array($settings['entrypoint'], $falseEntryPoints) || !file_exists($settings['entrypoint'])
        || strtolower($entryPointExt) != 'php') {
        LogUtil::registerError (_SETTINGS_ENTRYPOINTERROR);
        $settings['entrypoint'] = pnConfigGetVar('entrypoint');
    }

    // Write the vars
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    foreach($settings as $key => $value) {
        $oldvalue = pnConfigGetVar($key);
        if ($value != $oldvalue) {
            pnConfigSetVar($key, $value);
        }
    }

    // clear all cache and compile directories
    pnModAPIFunc('Settings', 'admin', 'clearallcompiledcaches');

    LogUtil::registerStatus (_CONFIGUPDATED);

    // Let any other modules know that the modules configuration has been updated
    pnModCallHooks('module','updateconfig','Settings', array('module' => 'Settings'));

    return pnRedirect(pnModURL('Settings', 'admin', 'modifyconfig'));
}

/**
 * display the alloed html form
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_allowedhtml($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // create a new output object
    $pnRender = pnRender::getInstance('Settings', false);

    $pnRender->assign('htmltags', _settings_admin_gethtmltags());
    $pnRender->assign('currenthtmltags', pnConfigGetVar('AllowableHTML'));
    $pnRender->assign('htmlentities', pnConfigGetVar('htmlentities'));

    // check for safehtml outputfilter
    $safehtml = (bool) (pnModAvailable('SecurityCenter') && pnConfigGetVar('outputfilter') == 1);
    $pnRender->assign('safehtml', $safehtml);
    $pnRender->assign('safehtmlurl', pnModURL('SecurityCenter', 'admin', 'modifyconfig'));

    return $pnRender->fetch('settings_admin_allowedhtml.htm');
}

/**
 * update allowed html settings
 *
 * @author Zikula development team
 * @return mixed true if successful, false if unsuccessful, error string otherwise
 */
function settings_admin_updateallowedhtml($args) {

    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // confirm the forms auth key
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError ();
    }

    // update the allowed html settings
    $allowedhtml = array();
    $htmltags = _settings_admin_gethtmltags();
    foreach ($htmltags as $htmltag) {
        $tagval = FormUtil::getPassedValue('htmlallow'.$htmltag.'tag', 0, 'POST');
        if (($tagval != 1) && ($tagval != 2)) {
            $tagval = 0;
        }
        $allowedhtml[$htmltag] = $tagval;
    }
    pnConfigSetVar('AllowableHTML', $allowedhtml);

    // one additonal config var is set on this page
    $htmlentities = FormUtil::getPassedValue('xhtmlentities', 0, 'POST');
    pnConfigSetVar('htmlentities', $htmlentities);

    // clear all cache and compile directories
    pnModAPIFunc('Settings', 'admin', 'clearallcompiledcaches');

    // all done successfully
    LogUtil::registerStatus (_CONFIGUPDATED);

    return pnRedirect(pnModURL('Settings', 'admin', 'allowedhtml'));
}

/**
 * display the ML settings form
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_multilingual($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // create a new output object
    $pnRender = pnRender::getInstance('Settings', false);

    // get all config vars and assign them to the template
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    // since config vars are serialised and module vars aren't we
    // need to unserialise each config var in turn before assigning
    // them to the template
    foreach ($configvars as $key => $configvar) {
        $configvars[$key] = $configvar;
    }
    $pnRender->assign($configvars);

    return $pnRender->fetch('settings_admin_multilingual.htm');
}

/**
 * update ML settings
 *
 * @author Zikula development team
 * @return mixed true if successful, false if unsuccessful, error string otherwise
 */
function settings_admin_updatemultilingual($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // confirm the forms auth key
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError ();
    }

    $settings = array('mlsettings_language'        => 'language',
                      'mlsettings_timezone_offset' => 'timezone_offset',
                      'mlsettings_timezone_server' => 'timezone_server',
                      'mlsettings_multilingual'    => 'multilingual',
                      'mlsettings_language_detect' => 'language_detect');

    // Write the vars
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    foreach($settings as $formname => $varname) {
        $newvalue = FormUtil::getPassedValue($formname, null, 'POST');
        $oldvalue = pnConfigGetVar($varname);
        if ($newvalue != $oldvalue) {
            pnConfigSetVar($varname, $newvalue);
        }
    }

    // clear the language session var for the current user
    // if the variable hasn't changed this won't matter anyway.....
    SessionUtil::delVar('lang');

    // clear all cache and compile directories
    pnModAPIFunc('Settings', 'admin', 'clearallcompiledcaches');

    // all done successfully
    LogUtil::registerStatus (_CONFIGUPDATED);

    return pnRedirect(pnModURL('Settings', 'admin', 'multilingual'));
}

/**
 * display the error handling settings form
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_errorhandling($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // create a new output object
    $pnRender = pnRender::getInstance('Settings');

    // get all config vars and assign them to the template
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    // since config vars are serialised and module vars aren't we
    // need to unserialise each config var in turn before assigning
    // them to the template
    foreach ($configvars as $key => $configvar) {
        $configvars[$key] = $configvar;
    }
    // add the development flag
    $configvars['development'] = pnConfigGetVar('development');
    $pnRender->assign($configvars);

    return $pnRender->fetch('settings_admin_errorhandling.htm');
}

/**
 * update error handling settings
 *
 * @author Zikula development team
 * @return mixed true if successful, false if unsuccessful, error string otherwise
 */
function settings_admin_updateerrorhandling($args) {

    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // confirm the forms auth key
    if (!SecurityUtil::confirmAuthKey()) {
        return LogUtil::registerAuthidError ();
    }

    $settings = array('errorsettings_errordisplay' => 'errordisplay',
                      'errorsettings_errorlog'     => 'errorlog',
                      'errorsettings_errorlogtype' => 'errorlogtype');
    // Write the vars
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    foreach($settings as $formname => $varname) {
        $newvalue = FormUtil::getPassedValue($formname, null, 'POST');
        $oldvalue = pnConfigGetVar($varname);
        if ($newvalue != $oldvalue) {
            pnConfigSetVar($varname, $newvalue);
        }
    }

    // clear all cache and compile directories
    pnModAPIFunc('Settings', 'admin', 'clearallcompiledcaches');

    // all done successfully
    LogUtil::registerStatus (_CONFIGUPDATED);

    return pnRedirect(pnModURL('Settings', 'admin', 'errorhandling'));
}

/**
 * display the main site settings form
 *
 * @author Zikula development team
 * @return string html output
 */
function settings_admin_security($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // create a new output object
    $pnRender = pnRender::getInstance('Settings', false);

    // get all config vars and assign them to the template
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    // since config vars are serialised and module vars aren't we
    // need to unserialise each config var in turn before assigning
    // them to the template
    foreach ($configvars as $key => $configvar) {
        $configvars[$key] = $configvar;
    }
    $pnRender->assign('settings', $configvars);

    return $pnRender->fetch('settings_admin_security.htm');
}

/**
 * update main site settings
 *
 * @author Zikula development team
 * @return mixed true if successful, false if unsuccessful, error string otherwise
 */
function settings_admin_updatesecurity($args) {

    // security check
    if (!SecurityUtil::checkPermission('Settings::', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // get settings from form - do before authid check
    $settings = FormUtil::getPassedValue('settings', null, 'POST');

    // if this form wasnt posted to redirect back
    if ($settings === NULL) {
        return pnRedirect(pnModURL('Settings', 'admin', 'security'));
    }

    // confirm the forms auth key
    if (!SecurityUtil::confirmAuthKey()) {
        //return LogUtil::registerAuthidError ();
    }

    // check session path config is writable (if method is being changed to session file storeage)
    $cause_logout = false;
    if ($settings['sessionstoretofile']) {
        // fix path on windows systems
        $settings['sessionsavepath'] = str_replace('\\', '/', $settings['sessionsavepath']);
        // sanitize the path
        $settings['sessionsavepath'] = trim(stripslashes($settings['sessionsavepath']));

        // check if sessionsavepath is a dir and if it is writable
        // if yes, we need to logout
        $cause_logout = (is_dir($settings['sessionsavepath'])) ? is_writable($settings['sessionsavepath']) : false;

        // logout if going from db to file storage
        if ($cause_logout==false) {
            // an error occured - we do not change the way of storing session data
            LogUtil::registerStatus (_SETTINGS_SESSIONPATHNOTWRITABLE);
            unset($settings['sessionstoretofile']);
            unset($settings['sessionsavepath']);
        }

    }

    // basic validation and default values
    if ((int)$settings['secmeddays'] < '1' || (int)$settings['secmeddays'] > '365' || !is_numeric($settings['secmeddays'])) {
        $settings['secmeddays'] = '7';
    }
    if ((int)$settings['secinactivemins'] < '1' || (int)$settings['secinactivemins'] > '1440' || !is_numeric($settings['secinactivemins'])) {
        $settings['secinactivemins'] = '20';
    }
    if ((int)$settings['gc_probability'] < '1' || (int)$settings['gc_probability'] > '10000' || !is_numeric($settings['gc_probability'])) {
        $settings['gc_probability'] = '100';
    }
    if ((int)$settings['sessionregeneratefreq'] < '1' || (int)$settings['sessionregeneratefreq'] > '100' || !is_numeric($settings['sessionregeneratefreq'])) {
        $settings['sessionregeneratefreq'] = '10';
    }
    if ((int)$settings['keyexpiry'] < '0' || (int)$settings['keyexpiry'] > '3600' || !is_numeric($settings['keyexpiry'])) {
        $settings['keyexpiry'] = '0';
    }
    if (strlen($settings['sessionname']) < 3) {
        $settings['sessionname'] = 'ZKSID';
    }

    // cause logout if we changed session name
    if ($settings['sessionname'] <> pnConfigGetVar('sessionname')) {
        $cause_logout = true;
    }

    // logout if going from file to db session storage
    if ((bool)$settings['sessionstoretofile'] != (bool)pnConfigGetVar('sessionstoretofile')) {
        $cause_logout = true;
    }

    // Write the vars
    $configvars = pnModGetVar(PN_CONFIG_MODULE);
    foreach($settings as $key => $value) {
        $oldvalue = pnConfigGetVar($key);
        if ($value != $oldvalue) {
            pnConfigSetVar($key, $value);
        }
    }

    // clear all cache and compile directories
    pnModAPIFunc('Settings', 'admin', 'clearallcompiledcaches');

    LogUtil::registerStatus (_CONFIGUPDATED);

    // we need to auto logout the user if they changed from DB to FILE
    if ($cause_logout==true) {
        pnUserLogOut();
        return pnRedirect(pnModURL('Users', 'user', 'loginscreen'));
    }

    return pnRedirect(pnModURL('Settings', 'admin', 'security'));
}

/**
 * utility function to return the list of available tags
 *
 * @access private
 * @author Zikula development team
 * @return string html output
 */
function _settings_admin_gethtmltags()
{
    // Possible allowed HTML tags
    return array('!--',
                  'a',
                  'abbr',
                  'acronym',
                  'address',
                  'applet',
                  'area',
                  'b',
                  'base',
                  'basefont',
                  'bdo',
                  'big',
                  'blockquote',
                  'br',
                  'button',
                  'caption',
                  'center',
                  'cite',
                  'code',
                  'col',
                  'colgroup',
                  'del',
                  'dfn',
                  'dir',
                  'div',
                  'dl',
                  'dd',
                  'dt',
                  'em',
                  'embed',
                  'fieldset',
                  'font',
                  'form',
                  'h1',
                  'h2',
                  'h3',
                  'h4',
                  'h5',
                  'h6',
                  'hr',
                  'i',
                  'iframe',
                  'img',
                  'input',
                  'ins',
                  'kbd',
                  'label',
                  'legend',
                  'li',
                  'map',
                  'marquee',
                  'menu',
                  'nobr',
                  'object',
                  'ol',
                  'optgroup',
                  'option',
                  'p',
                  'param',
                  'pre',
                  'q',
                  's',
                  'samp',
                  'script',
                  'select',
                  'small',
                  'span',
                  'strike',
                  'strong',
                  'sub',
                  'sup',
                  'table',
                  'tbody',
                  'td',
                  'textarea',
                  'tfoot',
                  'th',
                  'thead',
                  'tr',
                  'tt',
                  'u',
                  'ul',
                  'var');
}
