<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24945 2008-12-02 20:20:51Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_System_Modules
 * @subpackage Settings
 */

/**
 * initialise the settings module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Patrick Kellum
 * @author Simon Wunderlin
 * @return bool true if successful, false otherwise
 */
function settings_init()
{
    // Set up an initial value for a module variable.  Note that all module
    // variables should be initialised with some value in this way rather
    // than just left blank, this helps the user-side code and means that
    // there doesn't need to be a check to see if the variable is set in
    // the rest of the code as it always will be
    pnConfigSetVar('debug', _MODULE_VARS_000);
    pnConfigSetVar('sitename', _MODULE_VARS_001);
    pnConfigSetVar('site_logo', _MODULE_VARS_002);
    pnConfigSetVar('slogan', _MODULE_VARS_003);
    pnConfigSetVar('metakeywords', _MODULE_VARS_004);
    pnConfigSetVar('dyn_keywords', _MODULE_VARS_005);
    pnConfigSetVar('startdate', _MODULE_VARS_006);
    pnConfigSetVar('adminmail', _MODULE_VARS_007);
    pnConfigSetVar('Default_Theme', _MODULE_VARS_008);
    pnConfigSetVar('anonymous', _MODULE_VARS_011);
    pnConfigSetVar('timezone_offset', _MODULE_VARS_013);
    pnConfigSetVar('timezone_server', _MODULE_VARS_014);
    pnConfigSetVar('funtext', _MODULE_VARS_015);
    pnConfigSetVar('reportlevel', _MODULE_VARS_016);
    pnConfigSetVar('startpage', _MODULE_VARS_017);
    pnConfigSetVar('backend_title', _MODULE_VARS_020);
    pnConfigSetVar('backend_language', _MODULE_VARS_021);
    pnConfigSetVar('seclevel', _MODULE_VARS_022);
    pnConfigSetVar('secmeddays', _MODULE_VARS_023);
    pnConfigSetVar('secinactivemins', _MODULE_VARS_024);
    pnConfigSetVar('Version_Num', PN_VERSION_NUM);
    pnConfigSetVar('Version_ID', PN_VERSION_ID);
    pnConfigSetVar('Version_Sub', PN_VERSION_SUB);
    pnConfigSetVar('debug_sql', _MODULE_VARS_028);
    pnConfigSetVar('language', _MODULE_VARS_035);
    pnConfigSetVar('locale', _MODULE_VARS_036);
    pnConfigSetVar('multilingual', _MODULE_VARS_037);
    pnConfigSetVar('useflags', _MODULE_VARS_038);
    pnModSetVar('/PNConfig', 'AllowableHTML', _MODULE_VARS_050);
    pnConfigSetVar('theme_change', _MODULE_VARS_054);
    pnConfigSetVar('htmlentities', _MODULE_VARS_057);
    pnConfigSetVar('UseCompression', _MODULE_VARS_058);
    pnConfigSetVar('refereronprint', _MODULE_VARS_059);
    pnConfigSetVar('storyorder', _MODULE_VARS_061);
    pnConfigSetVar('timezone_info', _settings_gettimezones());
    pnConfigSetVar('errordisplay', 1);
    pnConfigSetVar('errorlog', 0);
    pnConfigSetVar('errorlogtype', 0);
    pnConfigSetVar('errormailto', _MODULE_VARS_007);
    pnConfigSetVar('siteoff', 0);
    pnConfigSetVar('siteoffreason', '');
    pnConfigSetVar('starttype', '');
    pnConfigSetVar('startfunc', '');
    pnConfigSetVar('startargs', '');
    pnConfigSetVar('entrypoint', 'index.php');
    pnConfigSetVar('secure_domain', '');
    pnConfigsetVar('language_detect', 1);
    pnConfigSetVar('shorturls', false);
    pnConfigSetVar('shorturlstype', '0');
    pnConfigSetVar('shorturlsext', 'html');
    pnConfigSetVar('shorturlsseparator', '-');
    pnConfigSetVar('shorturlsstripentrypoint', false);
    pnConfigSetVar('shorturlsdefaultmodule', '');
    pnConfigSetVar('signcookies', 1);
    pnConfigSetVar('signingkey', sha1(rand(0, time())));
    pnConfigSetVar('sessionipcheck', 0);
    pnConfigSetVar('keyexpiry', 0);
    pnConfigSetVar('gc_probability', 100);
    pnConfigSetVar('anonymoussessions', 1);
    pnConfigSetVar('sessionstoretofile', 0);
    pnConfigSetVar('sessionsavepath', '');
    pnConfigSetVar('sessionauthkeyua', false);
    pnConfigSetVar('sessionname', 'ZKSID');
    pnConfigSetVar('sessionregenerate', true);
    pnConfigSetVar('sessionrandregenerate', true);
    pnConfigSetVar('sessionregeneratefreq', 10);
    pnConfigSetVar('jsquicktags', true);
    $groupModules = _settings_getDefaultGroupModules();
    pnConfigSetVar('profilemodule', $groupModules['profile']);
    pnConfigSetVar('messagemodule', $groupModules['message']);

    // Initialisation successful
    return true;
}

/**
 * upgrade the settings module from an old version
 * This function can be called multiple times
 * @author Patrick Kellum
 * @author Simon Wunderlin
 * @param int $oldversion version to upgrade from
 * @return bool true if successful, false otherwise
 */
function settings_upgrade($oldversion)
{
    // always ensure that the version info is upgraded
    pnConfigSetVar('Version_Num', PN_VERSION_NUM);
    pnConfigSetVar('Version_ID', PN_VERSION_ID);
    pnConfigSetVar('Version_Sub', PN_VERSION_SUB);

    // Upgrade dependent on old version number
    switch($oldversion) {
        case 1.2:
            pnConfigSetVar('anonymoussessions', true);
            pnConfigDelVar('locale');
            return settings_upgrade(1.3);
        case 1.3:
            return settings_upgrade(1.4);
        case 1.4:
            // Code to upgrade from version 1.0 goes here
            if (pnModAvailable('Xanthia')) {
                pnConfigSetVar('shorturls', pnModGetVar('Xanthia', 'shorturls'));
                pnConfigSetVar('shorturlstype', '1');
                pnConfigSetVar('shorturlsext', pnModGetVar('Xanthia', 'shorturlsextension'));
                pnConfigSetVar('shorturlsstripentrypoint', false);
            } else {
                pnConfigSetVar('shorturls', false);
                pnConfigSetVar('shorturlstype', '0');
                pnConfigSetVar('shorturlsext', 'html');
                pnConfigSetVar('shorturlsstripentrypoint', false);
            }
            return settings_upgrade(1.5);
            break;
        case 1.5:
            pnConfigSetVar('signcookies', 1);
            pnConfigSetVar('signingkey', sha1(rand(0, time())));
            pnConfigSetVar('sessionipcheck', 0);
            pnConfigSetVar('keyexpiry', 0);
            pnConfigSetVar('gc_probability', 100);
            pnConfigSetVar('sessionstoretofile', 0);
            pnConfigSetVar('sessionsavepath', '');
            return settings_upgrade(1.6);
        case 1.6:
            pnConfigSetVar('timezone_info', _settings_gettimezones());
            pnConfigSetVar('timezone_offset', pnConfigGetVar('timezone_offset') - 12);
            pnConfigSetVar('timezone_server', 0);
            return settings_upgrade(1.7);
        case 1.7:
        	// Write recovery file.
        	Loader::loadClass('FileUtil');
        	FileUtil::writeFile(CacheUtil::getLocalDir().'/footermsg.html', pnConfigGetVar('foot1'));
        	pnConfigDelVar('foot1');
        	return settings_upgrade(1.8);
        case 1.8:
            pnConfigSetVar('sessionname', 'ZIKULASID');
            pnConfigSetVar('sessionregenerate', true);
            pnConfigSetVar('sessionregeneratefreq', 10);
            return settings_upgrade(1.9);
        case 1.9:
            pnConfigSetVar('sessionregenerate', true);
            pnConfigSetVar('sessionregeneratefreq', 10);
            pnConfigSetVar('sessionrandregenerate', true);
            pnConfigSetVar('sessionauthkeyua', false);
            return settings_upgrade(2.0);
       case 2.0:
            pnConfigSetVar('shorturlsseparator', '-');
            return settings_upgrade(2.1);
       case 2.1:
            pnConfigSetVar('jsquicktags', true);
            return settings_upgrade(2.2);
       case 2.2:
            pnConfigSetVar('errordisplay', 1);
            pnConfigSetVar('errorlog', 0);
            pnConfigSetVar('errorlogtype', 0);
            pnConfigSetVar('errormailto', _MODULE_VARS_007);
            pnConfigSetVar('entrypoint', 'index.php');
            pnConfigSetVar('secure_domain', '');
            return settings_upgrade(2.3);
       case 2.3:
            pnConfigSetVar('shorturlsdefaultmodule', '');
            return settings_upgrade(2.4);
       case 2.4:
            $groupModules = _settings_getDefaultGroupModules();
            pnConfigSetVar('profilemodule', $groupModules['profile']);
            pnConfigSetVar('messagemodule', $groupModules['message']);
            return settings_upgrade(2.5);
    }

    // Update successful
    return true;
}

/**
 * delete the settings module
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * @author Patrick Kellum
 * @author Simon Wunderlin
 * @return bool true if successful, false otherwise
 */
function settings_delete()
{
    // Deletion fail - we dont want users disabling this module!
    return false;
}


/*
 * return a list of timezones
 *
 * @link http://en.wikipedia.org/wiki/List_of_time_zones
 */
function _settings_gettimezones()
{
    return array('-12' => '(GMT -12:00 hours) Baker Island',
                 '-11' => '(GMT -11:00 hours) Midway Island, Samoa',
                 '-10' => '(GMT -10:00 hours) Hawaii',
                 '-9.5' => '(GMT -9:30 hours) French Polynesia',
                 '-9' => '(GMT -9:00 hours) Alaska',
                 '-8' => '(GMT -8:00 hours) Pacific Time (US & Canada)',
                 '-7' => '(GMT -7:00 hours) Mountain Time (US & Canada)',
                 '-6' => '(GMT -6:00 hours) Central Time (US & Canada), Mexico City',
                 '-5' => '(GMT -5:00 hours) Eastern Time (US & Canada), Bogota, Lima, Quito',
                 '-4' => '(GMT -4:00 hours) Atlantic Time (Canada), Caracas, La Paz',
                 '-3.5' => '(GMT -3:30 hours) Newfoundland',
                 '-3' => '(GMT -3:00 hours) Brazil, Buenos Aires, Georgetown',
                 '-2' => '(GMT -2:00 hours) Mid-Atlantic',
                 '-1' => '(GMT -1:00 hours) Azores, Cape Verde Islands',
                 '0' => '(GMT) Western Europe Time, London, Lisbon, Casablanca, Monrovia',
                 '1' => '(GMT +1:00 hours) CET(Central Europe Time), Brussels, Copenhagen, Madrid, Paris',
                 '2' => '(GMT +2:00 hours) EET(Eastern Europe Time), Kaliningrad, South Africa',
                 '3' => '(GMT +3:00 hours) Baghdad, Kuwait, Riyadh, Moscow, St. Petersburg',
                 '3.5' => '(GMT +3:30 hours) Tehran',
                 '4' => '(GMT +4:00 hours) Abu Dhabi, Muscat, Baku, Tbilisi',
                 '4.5' => '(GMT +4:30 hours) Kabul',
                 '5' => '(GMT +5:00 hours) Ekaterinburg, Islamabad, Karachi, Tashkent',
                 '5.5' => '(GMT +5:30 hours) Bombay, Calcutta, Madras, New Delhi',
                 '5.75' => '(GMT +5:45 hours) Kathmandu',
                 '6' => '(GMT +6:00 hours) Almaty, Dhaka, Colombo',
                 '6.5' => '(GMT +6:30 hours) Cocos Islands, Myanmar',
                 '7' => '(GMT +7:00 hours) Bangkok, Hanoi, Jakarta',
                 '8' => '(GMT +8:00 hours) Beijing, Perth, Singapore, Hong Kong, Chongqing, Urumqi, Taipei',
                 '9' => '(GMT +9:00 hours) Tokyo, Seoul, Osaka, Sapporo, Yakutsk',
                 '9.5' => '(GMT +9:30 hours) Adelaide, Darwin',
                 '10' => '(GMT +10:00 hours) EAST(East Australian Standard)',
                 '10.5' => '(GMT +10:30 hours) Lord Howe Island (NSW, Australia)',
                 '11' => '(GMT +11:00 hours) Magadan, Solomon Islands, New Caledonia',
                 '11.5' => '(GMT +11:30 hours) Norfolk Island',
                 '12' => '(GMT +12:00 hours) Auckland, Wellington, Fiji, Kamchatka, Marshall Island',
                 '12.75' => '(GMT +12:45 hours) Chatham Islands',
                 '13' => '(GMT +13:00 hours Tonga, Kiribati (Phoenix Islands)',
                 '14' => '(GMT +14:00 hours) Kiribati (Line Islands)');
}

/**
 * utility function to retrieve default values for module group vars
 * 
 * @author Axel Guckelsberger
 * @return array with determined module names
 */
function _settings_getDefaultGroupModules()
{
    $groupModules = array();

    $groupModules['profile'] = '';
    if (pnModAvailable('Profile')) {
        $groupModules['profile'] = 'Profile';
    }
    elseif (pnModAvailable('MyProfile')) {
        $groupModules['profile'] = 'MyProfile';
    }

    $groupModules['message'] = '';
    if (pnModAvailable('InterCom')) {
        $groupModules['message'] = 'InterCom';
    }

    return $groupModules;
}
