<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.userlinks.php 24737 2008-10-18 18:19:13Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display some user links
 *
 * Example
 * <!--[userlinks start="[" end="]" seperator="|"]-->
 *
 * Two additional defines need adding to a xanthia theme for this plugin
 * _CREATEACCOUNT and _YOURACCOUNT
 *
 * @author       Mark West
 * @since        21/10/03
 * @see          function.userlinks.php::smarty_function_userlinks()
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @param        string      $start       start delimiter
 * @param        string      $end         end delimiter
 * @param        string      $seperator   seperator
 * @return       string      user links
 */
function smarty_function_userlinks($params, &$smarty)
{
    extract($params);
    unset($params);

    if (!isset($start)) {
        $start = '[';
    }
    if (!isset($end)) {
        $end = ']';
    }
    if (!isset($seperator)) {
        $seperator = '|';
    }

    if (pnUserLoggedIn()) {
        $links = "$start ";
        $profileModule = pnConfigGetVar('profilemodule', '');
        if (!empty($profileModule) && pnModAvailable($profileModule)) {
            $links .= "<a href=\"" . DataUtil::formatForDisplay(pnModURL($profileModule)) . '">' . _YOURACCOUNT . "</a> $seperator ";
        }
        $links .= "<a href=\"" . DataUtil::formatForDisplay(pnModURL('Users', 'user', 'logout')) . '">'  . _LOGOUT . "</a> $end";
    } else {
        $links = "$start <a href=\"" . DataUtil::formatForDisplay(pnModURL('Users', 'user', 'register')) . '">' . _CREATEACCOUNT . "</a> $seperator "
               . "<a href=\"" . DataUtil::formatForDisplay(pnModURL('Users', 'user', 'view')) . '">' . _LOGIN . "</a> $end";
    }

    return DataUtil::formatForDisplayHTML($links);
}
