<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id$
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Filters
 */

/**
 * Smarty outputfilter to create links from urls. This has beend derived from the 
 * pn_bbclick module and uses a slightly changed regular expression found at
 * http://smarty.incutio.com/?page=ActivateLinksPlugin
 *
 * @author    Frank Schummertz
 * @param     string
 * @param     Smarty
 */
function smarty_outputfilter_makelinks($text, &$smarty)
{
    $text = ' ' . $text;

    // remove areas that should not be made clickable, eg. for the pndebug plugin
    // those areas are marked with <!--raw-->some hml<!/raw-->
    $rawcount = preg_match_all("/<!--raw-->(.*)<!--\/raw-->/Usi", $text, $raws);
    for($i=0; $i < $rawcount; $i++) {
        $text = substr_replace($text, " MLRAWREPLACEMENT{$i} ", strpos($text, $raws[0][$i]), strlen($raws[0][$i]));
    }

    // move all links out of the text and replace them with placeholders
    $linkscount = preg_match_all('/<a (.*)>(.*)<\/a>/siU', $text, $links);
    for ($i = 0; $i < $linkscount; $i++) {
        $text = substr_replace($text, " MLLINKREPLACEMENT{$i} ", strpos($text, $links[0][$i]), strlen($links[0][$i]));
    }
    
    // move all absolute urls inside of "" and '' out of the text and replace them with placeholders
    // these are most likely valid parameters in href= or src=
    $urlcount = preg_match_all("#(?:\"|\')(\w+)://([\w\+\-\,\@\=\?\.\%\/\:\&\;~\|\#]+)(\.)?(?:\"|\')#", $text, $urls);
    for($i=0; $i < $urlcount; $i++) {
        $text = substr_replace($text, " MLURLREPLACEMENT{$i} ", strpos($text, $urls[0][$i]), strlen($urls[0][$i]));
    }
    
    // remove input
    $inputcount = preg_match_all("/<input(.*)>/Usi", $text, $inputs);
    for($i=0; $i < $inputcount; $i++) {
        $text = substr_replace($text, " MLINPUTREPLACEMENT{$i} ", strpos($text, $inputs[0][$i]), strlen($inputs[0][$i]));
    }

    // remove textareas
    $textareacount = preg_match_all("/<textarea(.*)>(.*)<\/textarea>/Usi", $text, $textareas);
    for($i=0; $i < $textareacount; $i++) {
        $text = substr_replace($text, " MLTEXTAREAREPLACEMENT{$i} ", strpos($text, $textareas[0][$i]), strlen($textareas[0][$i]));
    }

    // remove options in selects
    $optioncount = preg_match_all("/<option(.*)>(.*)<\/option>/Usi", $text, $options);
    for($i=0; $i < $optioncount; $i++) {
        $text = substr_replace($text, " MLOPTIONREPLACEMENT{$i} ", strpos($text, $options[0][$i]), strlen($options[0][$i]));
    }

    // remove javascript
    $scriptcount = preg_match_all("/<script(.*)>(.*)<\/script>/Usi", $text, $scripts);
    for($i=0; $i < $scriptcount; $i++) {
        $text = substr_replace($text, " MLSCRIPTREPLACEMENT{$i} ", strpos($text, $scripts[0][$i]), strlen($scripts[0][$i]));
    }

    $text = preg_replace_callback(
             "#(\w+)://([\w\+\-\@\=\?\.\%\/\:\&\;~\|\#]+)(\.)?#",
             '_smarty_outputfilter_makelinks_callback',
            $text);

    // replace the tags that we removed before
    for ($i = 0; $i < $scriptcount; $i++) {
        $text = str_replace(" MLSCRIPTREPLACEMENT{$i} ", $scripts[0][$i], $text);
    }

    for ($i = 0; $i < $optioncount; $i++) {
        $text = str_replace(" MLOPTIONREPLACEMENT{$i} ", $options[0][$i], $text);
    }

    for ($i = 0; $i < $textareacount; $i++) {
        $text = str_replace(" MLTEXTAREAREPLACEMENT{$i} ", $textareas[0][$i], $text);
    }

    for ($i = 0; $i < $inputcount; $i++) {
        $text = str_replace(" MLINPUTREPLACEMENT{$i} ", $inputs[0][$i], $text);
    }

    for ($i = 0; $i < $urlcount; $i++) {
        $text = str_replace(" MLURLREPLACEMENT{$i} ", $urls[0][$i], $text);
    }

    for ($i = 0; $i < $linkscount; $i++) {
        $text = str_replace(" MLLINKREPLACEMENT{$i} ", $links[0][$i], $text);
    }

    for ($i = 0; $i < $rawcount; $i++) {
        $text = str_replace(" MLRAWREPLACEMENT{$i} ", $raws[0][$i], $text);
    }

    $text = substr($text, 1);
    // return the modified source
    return $text;
}


/**
 * linktest_callback
 *
 */
function _smarty_outputfilter_makelinks_callback($matches)
{
    // get the maximum size of the urls to show
    $maxsize = pnModGetVar('Theme', 'maxsizeforlinks', 30);
    $before = round($maxsize / 2);
    $after  = $maxsize - 3 - $before; // minus 3 for '...'

    // replace &amp; with & to determine the correct length
    $tempurl = str_replace('&amp;', '&', $matches[0]);
    if ($maxsize <> -1 && strlen($tempurl) > $maxsize) {
        $displayurl = DataUtil::formatForDisplay(StringUtil::left($tempurl, $before) . '...' . StringUtil::right($tempurl, $after));
    } else {
        $displayurl = $matches[0];
    }
    return '<a href="' . $matches[0] . '">' . $displayurl . '</a>';
}
