<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pninit.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Mark West
 * @package Zikula_System_Modules
 * @subpackage Theme
*/

/**
 * initialise the theme module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance.
 * This function MUST exist in the pninit file for a module
 *
 * @author       Mark West
 * @return       bool       true on success, false otherwise
 */
function theme_init()
{
    // create the table
    if (!DBUtil::createTable('themes')) {
        return false;
    }

    // detect all themes on install
    pnModAPILoad('Theme', 'admin', true);
    pnModAPIFunc('Theme', 'admin', 'regenerate');

    // define defaults for module vars
    pnModSetVar('Theme', 'modulesnocache', '');
    pnModSetVar('Theme', 'enablecache', false);
    pnModSetVar('Theme', 'compile_check', true);
    pnModSetVar('Theme', 'cache_lifetime', 3600);
    pnModSetVar('Theme', 'force_compile', false);
    pnModSetVar('Theme', 'trimwhitespace', false);
    pnModSetVar('Theme', 'makelinks', false);
    pnModSetVar('Theme', 'maxsizeforlinks', 30);
    pnModSetVar('Theme', 'itemsperpage', 25);

    // Initialisation successful
    return true;
}

/**
 * upgrade the theme module from an old version
 *
 * This function can be called multiple times
 * This function MUST exist in the pninit file for a module
 *
 * @author       Mark West
 * @return       bool       true on success, false otherwise
 */
function theme_upgrade($oldversion)
{
    // update the table
    if (!DBUtil::changeTable('themes')) {
        return false;
    }

    switch ($oldversion) {
        // versions shipped with .7x	
        case 2.0:
        case 2.1:
            // detect all themes on upgrade
            pnModAPILoad('Theme', 'admin', true);
            pnModAPIFunc('Theme', 'admin', 'regenerate');
            // define defaults for module vars
            pnModSetVar('Theme', 'modulesnocache', null);
            pnModSetVar('Theme', 'enablecache', false);
            pnModSetVar('Theme', 'compile_check', true);
            pnModSetVar('Theme', 'cache_lifetime', 3600);
            pnModSetVar('Theme', 'force_compile', false);
            pnModSetVar('Theme', 'trimwhitespace', false);
            pnModSetVar('Theme', 'makelinks', false);
            pnModSetVar('Theme', 'maxsizeforlinks', 30);
            pnModSetVar('Theme', 'itemsperpage', 25);
            $oldtables = array('theme_config', 'theme_layout', 'theme_skins', 'theme_palette', 'theme_zones', 
                               'theme_cache', 'theme_blcontrol', 'theme_addons', 'theme_tplfile', 'theme_tplsource');
            foreach ($oldtables as $oldtable) {
                if (!DBUtil::dropTable($oldtable)) {
                    return false;
                }
            }
            pnModDelVar('Xanthia');
            break;
    }

    // Update successful
    return true;
}

/**
 * delete the theme module
 *
 * This function is only ever called once during the lifetime of a particular
 * module instance
 * This function MUST exist in the pninit file for a module
 *
 * Since the theme module should never be deleted we'all always return false here
 * @author       Mark West
 * @return       bool       false
 */
function theme_delete()
{
    // drop the table
    if (!DBUtil::dropTable('themes')) {
        return false;
    }

    // delete all module variables
    pnModDelVar('Theme');

    // Deletion not allowed
    return false;
}
