<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: pnadmin.php 24972 2008-12-06 13:07:57Z Landseer $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Xiaoyu Huang
 * @package Zikula_System_Modules
 * @subpackage Users
 */

/**
 * users_admin_main()
 * Redirects users to the "view" page
 *
 * @return bool true if successful false otherwise
 */
function users_admin_main()
{
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_ADMIN)){
        return LogUtil::registerPermissionError();
    }

    return pnRedirect(pnModURL('Users', 'admin', 'view'));
}

/**
 * form to add new item
 *
 * This is a standard function that is called whenever an administrator
 * wishes to create a new module item
 *
 * @author       The Zikula Development Team
 * @return       output       The main module admin page.
 */
function users_admin_new()
{
    // Security check - important to do this as early as possible to avoid
    // potential security holes or just too much wasted processing
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_ADD)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object - this object will store all of our output so that
    // we can return it easily when required
    $pnRender = pnRender::getInstance('Users', false);

    // load user language file
    pnModLangLoad('Users', 'user');
    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule)) {
        pnModLangLoad($profileModule, 'user');
    }

    // Assign Users setting
    $modvars = pnModGetVar('Users');
    $pnRender->assign($modvars);

    if ($modvars['reg_optitems'] && $profileModule == 'Profile') {
        $pnRender->assign('optitems', pnModAPIFunc($profileModule, 'user', 'getallactive', array()));
    }

    // Return the output that has been generated by this function
    return $pnRender->fetch('users_admin_new.htm');
}

/**
 * Create user
 *
 * @param  $args
 * @return mixed true if successful, string otherwise
 */
function users_admin_create()
{
    // check permisisons
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_ADD)) {
        return LogUtil::registerPermissionError();
    }

    // load user language file
    pnModLangLoad('Users', 'user');

    // get arugments
    $uname = FormUtil::getPassedValue('add_uname', null, 'POST');
    $email = FormUtil::getPassedValue('add_email', null, 'POST');
    $pass  = FormUtil::getPassedValue('add_pass', null, 'POST');
    $vpass = FormUtil::getPassedValue('add_vpass', null, 'POST');

    $optionals = false;
    $dynadata = array();

    $profileModule = pnConfigGetVar('profilemodule', '');
    $useProfileDUD = (!empty($profileModule) && pnModAvailable($profileModule));

    if (pnModGetVar('Users', 'reg_optitems') == 1) {
        $dynadata = FormUtil::getPassedValue('dynadata');
        $optionals = true;
        if ($useProfileDUD) {
            pnModLangLoad($profileModule, 'user');
        }
    }

    // call the API
    $checkuser = pnModAPIFunc('Users', 'user', 'checkuser',
                              array('uname'        => $uname,
                                    'email'        => $email,
                                    'agreetoterms' => 1));

    // if errorcode != 1 then return error msgs
    if ($checkuser != 1) {
        switch($checkuser) {
        case -1:
            LogUtil::registerError(_MODULENOAUTH);
            break;
        case 2:
            LogUtil::registerError(_USERS_INVALIDEMAIL);
            break;
        case 3:
            LogUtil::registerError(_USERS_ERRORMUSTAGREE);
            break;
        case 4:
            LogUtil::registerError(_USERS_USERNAMEINVALID);
            break;
        case 5:
            LogUtil::registerError(_USERS_USERNAMETOOLONG);
            break;
        case 6:
            LogUtil::registerError(_USERS_USERNAMERESERVED);
            break;
        case 7:
            LogUtil::registerError(_USERS_USERNAMENOSPACES);
            break;
        case 8:
            LogUtil::registerError(_USERS_USERNAMETAKEN);
            break;
        case 9:
            LogUtil::registerError(_USERS_EMAILREGISTERED);
            break;
        default:
            LogUtil::registerError(_MODULENOAUTH);
        } // switch
        return pnRedirect(pnModURL('Users', 'admin', 'main'));
    }

    if ($optionals && $useProfileDUD && $profileModule == 'Profile') {
        // Check for required fields - The API function is called.
        $checkrequired = pnModAPIFunc($profileModule, 'user', 'checkrequired',
                                      array('dynadata' => $dynadata));

        if ($checkrequired['result'] == true) {
            LogUtil::registerError(_PROFILE_REQUIREDMISSING . ' (' . $checkrequired['translatedFieldsStr'] . ')');
            return pnRedirect(pnModURL('Users','admin','new'));
        }
    }

    $minpass = pnModGetVar('Users', 'minpass');

    if (isset($message)) {
        LogUtil::registerError ($message);
        return pnRedirect(pnModURL('Users','admin','new'));
    }

    if (empty($pass)) {
        LogUtil::registerError (_USERS_NOPASS);
        return pnRedirect(pnModURL('Users', 'admin', 'main'));
    } elseif ((isset($pass)) && ("$pass" != "$vpass")) {
        LogUtil::registerError (_USERS_PASSDIFFERENT);
        return pnRedirect(pnModURL('Users', 'admin', 'main'));
    } elseif (($pass != '') && (strlen($pass) < $minpass)) {
        LogUtil::registerError (pnML('_USERS_YOURPASSMUSTBETHISLONG', array('x' => $minpass)));
        return pnRedirect(pnModURL('Users', 'admin', 'main'));
    }

    $registered = pnModAPIFunc('Users', 'user', 'finishnewuser',
                               array('isadmin'       => 1,
                                     'name'          => 'blank',
                                     'uname'         => $uname,
                                     'pass'          => $pass,
                                     'email'         => $email,
                                     'user_avatar'   => 'blank.gif',
                                     'user_regdate'  => date("Y-m-d H:i:s", time()),
                                     'moderated'     => false,
                                     'dynadata'      => $dynadata,
                                     'optionals'     => $optionals));


    if ($registered) {
       LogUtil::registerStatus (pnML('_CREATEITEMSUCCEDED', array('i' => _USERS_USER)));
    } else {
        LogUtil::registerError (_CREATEFAILED);
    }

    return pnRedirect(pnModURL('Users', 'admin', 'main'));
}

/**
 * view items
 *
 * This function shows all items and lists the administration
 * options.
 *
 * @author       The Zikula Development Team
 * @param        startnum     The number of the first item to show
 * @return       output       The main module admin page
 */
function users_admin_view()
{
    // Get parameters from whatever input we need.
    $startnum = FormUtil::getPassedValue('startnum', isset($args['startnum']) ? $args['startnum'] : null, 'GET');
    $letter = FormUtil::getPassedValue('letter', isset($args['letter']) ? $args['letter'] : null, 'GET');

    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Users', false);

    // we need this value multiple times, so we keep it
    $itemsperpage = pnModGetVar('Users', 'itemsperpage');

    // Get all users
    $items = pnModAPIFunc('Users', 'user', 'getall',
                          array('startnum' => $startnum,
                                'numitems' => $itemsperpage,
                                'letter' => $letter));

    // Loop through each returned item adding in the options that the user has over
    // each item based on the permissions the user has.
    foreach ($items as $key => $item) {
        $options = array();
        if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[uid]", ACCESS_READ) && $item['uid'] != 1) {
            // Options for the item.
            if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[uid]", ACCESS_EDIT)) {
                $options[] = array('url'   => pnModURL('Users', 'admin', 'modify', array('userid' => $item['uid'])),
                                   'title' => _EDIT,
                                   'image' => 'xedit.gif');
                if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[uid]", ACCESS_DELETE)) {
                    $options[] = array('url'   => pnModURL('Users', 'admin', 'deleteusers', array('userid' => $item['uid'])),
                                       'title' => _DELETE,
                                       'image' => '14_layer_deletelayer.gif');
                }
            }
        }
        // Add the calculated menu options to the item array
        $items[$key]['options'] = $options;
    }

    // Assign the items to the template
    $pnRender->assign('usersitems', $items);

    // assign the values for the smarty plugin to produce a pager in case of there
    // being many items to display.
    $pnRender->assign('pager', array('numitems'     => pnModAPIFunc('Users', 'user', 'countitems', array('letter' => $letter)),
                                     'itemsperpage' => $itemsperpage));

    // Return the output that has been generated by this function
    return $pnRender->fetch('users_admin_view.htm');
}

/**
 * view items
 *
 * This function shows all items and lists the administration
 * options.
 *
 * @author       The Zikula Development Team
 * @param        startnum     The number of the first item to show
 * @return       output       The main module admin page
 */
function users_admin_viewapplications()
{
    // security check
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    // Get parameters from whatever input we need.
    $startnum = FormUtil::getPassedValue ('startnum');

    // Create output object
    $pnRender = pnRender::getInstance('Users', false);

    // we need this value multiple times, so we keep it
    $itemsperpage = pnModGetVar('Users', 'itemsperpage');

    // The user API function is called.
    $items = pnModAPIFunc('Users', 'admin', 'getallpendings',
                          array('startnum' => $startnum,
                                'numitems' => $itemsperpage));

    // Loop through each returned item adding in the options that the user has over
    // each item based on the permissions the user has.
    foreach ($items as $key => $item) {
        if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[tid]", ACCESS_READ)) {
            // Options for the item.
            $options = array();
            if (pnModGetVar('Users', 'reg_optitems')) {
                $options[] = array('url'   => pnModURL('Users', 'admin', 'viewtempuserinfo', array('userid' => $item['tid'])),
                                   'imgfile' => 'list.gif',
                                   'title' => _DETAILS);
            }
            if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[tid]", ACCESS_ADD)) {
                $options[] = array('url'   => pnModURL('Users', 'admin', 'processusers', array('userid' => $item['tid'], 'op' => 'approve')),
                                   'imgfile' => 'add_user.gif',
                                   'title' => _ADD);
                if (SecurityUtil::checkPermission('Users::', "$item[uname]::$item[tid]", ACCESS_DELETE)) {
                    $options[] = array('url'   => pnModURL('Users', 'admin', 'processusers', array('userid' => $item['tid'], 'op' => 'deny')),
                                       'imgfile' => 'delete_user.gif',
                                       'title' => _DELETE);
                }
            }

            // Add the calculated menu options to the item array
            $items[$key]['options'] = $options;
        }
    }

    // Assign the items to the template
    $pnRender->assign('usersitems', $items);

    // assign the values for the smarty plugin to produce a pager in case of there
    // being many items to display.
    $pnRender->assign('pager', array('numitems'     => pnModAPIFunc('Users', 'admin', 'countpending'),
                                     'itemsperpage' => $itemsperpage));

    // Return the output that has been generated by this function
    return $pnRender->fetch('users_admin_viewapplications.htm');
}

/**
 * users_admin_viewtempuserinfo()
 * Shows the information for the temporary user
 *
 * @param  $args
 * @return string HTML string
 */
function users_admin_viewtempuserinfo()
{
    // security check
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_ADD)) {
        return LogUtil::registerPermissionError();
    }

    // Get parameters from whatever input we need.
    $userid = FormUtil::getPassedValue('userid', null, 'GET');

    if (empty($userid) || !is_numeric($userid)) {
        return LogUtil::registerError(_MODARGSERROR);
    }

    // Create output object
    $pnRender = pnRender::getInstance('Users', false);

    $tempuser = pnModAPIFunc('Users', 'admin', 'getapplication', array('userid' => $userid));

    $userinfo = array();

    $dynamics = unserialize($tempuser['dynamics']);

    $userinfo = array_merge($tempuser, $dynamics);
    //unset($tempuserinfo['dynamics']);

    $items = array();

    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule) && $profileModule == 'Profile') {
        $items = pnModAPIFunc($profileModule, 'user', 'getall');
        // Removing password from the list
        unset($items['_PASSWORD']);

        foreach($items as $field) {
            $alias = pnModAPIFunc($profileModule, 'user', 'aliasing', array('label' => $field['prop_label']));

            if (isset($userinfo[$field['prop_label']]) && $alias != $field['prop_label']) {
                $userinfo[$alias] = $userinfo[$field['prop_label']];
            } elseif (isset($userinfo[$alias]) && $alias != $field['prop_label']) {
                $userinfo[$field['prop_label']] = $userinfo[$alias];
            }

            if ($field['prop_displaytype'] > 6) {
                $pnRender->assign($alias, unserialize($userinfo[$alias]));
            } else if (isset($userinfo[$alias])) {
                //$pnRender->assign($alias, @unserialize($userinfo[$alias]));
                $pnRender->assign($alias, $userinfo[$alias]);
            }
        }

        pnModLangLoad($profileModule, 'user');
    }

    $pnRender->assign('uname',    $userinfo['uname']);
    $pnRender->assign('userid',   $userid);
    $pnRender->assign('fields',   $items);
    $pnRender->assign('userinfo', $userinfo);

    return $pnRender->fetch('users_admin_viewtempuserdetails.htm');
}

/**
 * user_admin_view()
 * Shows the search box for Edit/Delete
 * Shows Add User Dialog
 *
 * @param  $args
 * @return string HTML string
 */
function users_admin_search($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT)){
        return LogUtil::registerPermissionError();
    }

    // create output object
    $pnRender = pnRender::getInstance('Users', false);

    // get group items
    // TODO: move to a call to the groups module
    $groups = pnModAPIFunc('Users', 'admin', 'getusergroups', array());
    $pnRender->assign('groups', $groups);

    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule)) {
        // load the profile language file
        pnModLangLoad($profileModule, 'user');

        if ($profileModule == 'Profile') {
            $duditems = pnModAPIFunc($profileModule, 'user', 'getallactive');
            $pnRender->assign('duditems', $duditems);
        }
    }

    return $pnRender->fetch('users_admin_search.htm');
}

/**
 * users_admin_listusers()
 * list users
 *
 * @param  $args
 * @return string HTML string
 */
function users_admin_listusers($args)
{
    $uname         = FormUtil::getPassedValue('uname', null, 'POST');
    $rname         = FormUtil::getPassedValue('rname', null, 'POST');
    $ugroup        = FormUtil::getPassedValue('ugroup', null, 'POST');
    $email         = FormUtil::getPassedValue('email', null, 'POST');
    $homepage      = FormUtil::getPassedValue('homepage', null, 'POST');
    $icq           = FormUtil::getPassedValue('icq', null, 'POST');
    $msn           = FormUtil::getPassedValue('msn', null, 'POST');
    $aim           = FormUtil::getPassedValue('aim', null, 'POST');
    $yim           = FormUtil::getPassedValue('yim', null, 'POST');
    $signature     = FormUtil::getPassedValue('signature', null, 'POST');
    $regdateafter  = FormUtil::getPassedValue('regdateafter', null, 'POST');
    $regdatebefore = FormUtil::getPassedValue('regdatebefore', null, 'POST');
    $dynadata      = FormUtil::getPassedValue('dynadata', null, 'POST');

    // call the api
    $items = pnModAPIFunc('Users', 'admin', 'findusers',
                          array('uname'         => $uname,
                                'email'         => $email,
                                'ugroup'        => $ugroup,
                                'regdateafter'  => $regdateafter,
                                'regdatebefore' => $regdatebefore,
                                'dynadata'      => $dynadata)
                         );
    if (!$items) {
        LogUtil::registerError(pnML('_NOFOUND', array('i' => _USERS_USERS)), 404, pnModURL('Users', 'admin', 'search'));
    }

    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule)) {
        // load the profile module language file
        pnModLangLoad($profileModule, 'user');
    }

    // create output object
    $pnRender = pnRender::getInstance('Users', false);

    // assign the matching results
    $pnRender->assign('items', $items);

    return $pnRender->fetch('users_admin_listusers.htm');
}

/**
 * users_admin_processusers()
 * Edit, Delete and Mail Users
 *
 * @param  $args
 * @return mixed true successful, false or string otherwise
 */
function users_admin_processusers($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT)){
        return LogUtil::registerPermissionError();
    }

    // get the arguments from our input
    $op     = FormUtil::getPassedValue('op', null, 'GETPOST');
    $do     = FormUtil::getPassedValue('do', null, 'POST');
    $userid = FormUtil::getPassedValue('userid', null, 'POST');

    // create the output object
    $pnRender = pnRender::getInstance('Users', false);

    if ($op == 'edit' && !empty($userid)){
        if ($do != 'yes'){
            return pnRedirect(pnModURL('Users', 'admin', 'modify', array('userid' => $userid)));
        } else {
            $uname              = FormUtil::getPassedValue('uname', null, 'POST');
            $email              = FormUtil::getPassedValue('email', null, 'POST');
            $activated          = FormUtil::getPassedValue('activated', null, 'POST');
            $pass               = FormUtil::getPassedValue('pass', null, 'POST');
            $vpass              = FormUtil::getPassedValue('vpass', null, 'POST');
            $theme              = FormUtil::getPassedValue('theme', null, 'POST');
            $access_permissions = FormUtil::getPassedValue('access_permissions', null, 'POST');
            $dynadata           = FormUtil::getPassedValue('dynadata', null, 'POST');

            $return = pnModAPIFunc('Users', 'admin', 'saveuser', array('uid'                => $userid,
                                                                       'uname'              => $uname,
                                                                       'email'              => $email,
                                                                       'activated'          => $activated,
                                                                       'pass'               => $pass,
                                                                       'vpass'              => $vpass,
                                                                       'theme'              => $theme,
                                                                       'dynadata'           => $dynadata,
                                                                       'access_permissions' => $access_permissions));

            if ($return == true) {
                LogUtil::registerStatus (pnML('_UPDATEITEMSUCCEDED', array('i' => _USERS_USER)));
                return pnRedirect(pnModUrl('Users', 'admin', 'main'));
            } else {
                return false;
            }
        }
    } elseif ($op == 'delete' && !empty($userid)) {
        $userid = FormUtil::getPassedValue('userid', null, 'POST');
        if ($do != 'yes'){
            return pnRedirect(pnModURL('Users', 'admin', 'deleteusers', array('userid' => $userid)));
        } else {
            $return = pnModAPIFunc('Users', 'admin', 'deleteuser', array('uid' => $userid));

            if ($return == true) {
                LogUtil::registerStatus (pnML('_DELETEITEMSUCCEDED', array('i' => _USERS_USER)));
                return pnRedirect(pnModUrl('Users', 'admin', 'main'));
            } else {
                return false;
            }
        }
    } elseif ($op == 'mail' && !empty($userid)) {
        // TODO: unify with users_admin_mailuserssend
        $userid = FormUtil::getPassedValue('userid', null, 'POST');
        if ($do != 'yes'){
            return pnRedirect(pnModURL('Users', 'admin', 'mailusers', array('userid' => $userid)));
        } else {
            $userid  = FormUtil::getPassedValue('userid', null, 'POST');
            $from    = FormUtil::getPassedValue('from', null, 'POST');
            $rpemail = FormUtil::getPassedValue('rpemail', null, 'POST');
            $subject = FormUtil::getPassedValue('subject', null, 'POST');
            $message = FormUtil::getPassedValue('message', null, 'POST');

            $return = pnModAPIFunc('Users', 'admin', 'mailuser',
                                   array('uid'     => $userid,
                                         'from'    => $from,
                                         'rpemail' => $rpemail,
                                         'subject' => $subject,
                                         'message' => $message
                                        ));

            if ($return == true) {
                LogUtil::registerStatus (_USERS_MAILSENT);
               return  pnRedirect(pnModUrl('Users', 'admin', 'main'));
            } else {
                LogUtil::registerError (_USERS_MAILSENTFAILED);
                return pnRedirect(pnModUrl('Users', 'admin', 'main'));
            }
        }
    } elseif ($op == 'approve' || $op == 'deny') {
        $tag = FormUtil::getPassedValue ('tag');

        if (empty($tag)) {
            $userid = FormUtil::getPassedValue('userid', null, 'GET');

            $pnRender->caching = false;

            $item = pnModAPIFunc('Users', 'admin', 'getapplication', array('userid' => $userid));

            if (!$item) {
                LogUtil::registerError (_USERS_NOUSERINFOFOUND);
                return pnRedirect(pnModUrl('Users', 'admin', 'main'));
            }

            $pnRender->assign('action', $op);
            $pnRender->assign('userid', $userid);
            $pnRender->assign('item',   $item);

            return $pnRender->fetch('users_admin_pendingaction.htm');

        } else {

            $userid = FormUtil::getPassedValue ('userid');
            $action = FormUtil::getPassedValue ('action');

            $return = pnModAPIFunc('Users', 'admin', $action, array('userid' => $userid));

            if ($return == true) {
                if ($op == 'approve') {
                    LogUtil::registerStatus (pnML('_CREATEITEMSUCCEDED', array('i' => _USERS_USER)));
                } else {
                    LogUtil::registerStatus (pnML('_DELETEITEMSUCCEDED', array('i' => _USERS_USER)));
                }
            } else {
                LogUtil::registerError (_CREATEFAILED);
            }
            return pnRedirect(pnModUrl('Users', 'admin', 'main'));
        }

    } else {
        return LogUtil::registerError(_USERS_NOUSERSELECTED);
    }
}

/**
 * users_admin_modify()
 *
 * @param  $args
 * @return string HTML string
 */
function users_admin_modify($args)
{
    // security check
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_EDIT)){
        return LogUtil::registerPermissionError();
    }

    // get our input
    $op     = FormUtil::getPassedValue('op', (isset($args['op']) ? $args['op'] : null), 'GET');
    $userid = FormUtil::getPassedValue('userid', (isset($args['userid']) ? $args['userid'] : null), 'GET');
    $uname  = FormUtil::getPassedValue('uname', (isset($args['uname']) ? $args['uname'] : null), 'GET');
    if (is_null($userid) && !is_null($uname) && !empty($uname)) {
        $userid = pnUserGetIDFromName($uname);
    }

    // get the user vars
    $uservars = pnUserGetVars($userid);
    if ($uservars == false) {
        return LogUtil::registerError(_NOSUCHITEM, 404);
    }

    $profileModule = pnConfigGetVar('profilemodule', '');
    $useProfile = (!empty($profileModule) && pnModAvailable($profileModule));

    // load additional language files
    if ($useProfile) {
        pnModLangLoad($profileModule, 'user');
    }
    pnModLangLoad('Theme', 'admin');

    // create the output oject
    $pnRender = pnRender::getInstance('Users', false);

    // urls
    $pnRender->assign('urlprocessusers', pnModUrl('Users', 'admin', 'processusers', array('op' => 'edit', 'do' => 'yes')));
    $pnRender->assign('op', 'edit');


    $pnRender->assign('userid', $userid);
    $pnRender->assign($uservars);

    $items = array();
    if ($useProfile) {
        $items = pnModAPIFunc($profileModule, 'user', 'getallactive');
    }

    // assign the items to the template
    $pnRender->assign('duditems', $items);

    $permissions_array = array();
    $access_types_array = array();
    $usergroups = pnModAPIFunc('Groups', 'user', 'getusergroups', array('uid' => $userid));
    foreach ($usergroups as $usergroup) {
        $permissions_array[] = (int)$usergroup['gid'];
    }
    $allgroups = pnModAPIFunc('Groups', 'user', 'getall');
    foreach ($allgroups as $group) {
        $access_types_array[$group['gid']] = $group['name'];
    }

    $pnRender->assign('permissions_array', $permissions_array);
    $pnRender->assign('access_types_array', $access_types_array);

    return $pnRender->fetch('users_admin_modify.htm');
}

/**
 * users_admin_deleteusers()
 *
 * @param $args
 * @return string HTML string
 **/
function users_admin_deleteusers()
{
    // check permissions
    if (!SecurityUtil::checkPermission('Users::', '::', ACCESS_DELETE)){
        return LogUtil::registerPermissionError();
    }

    // get arguments
    $op     = FormUtil::getPassedValue('op', null, 'GET');
    $userid = FormUtil::getPassedValue('userid', null, 'GET');
    $uname  = FormUtil::getPassedValue('uname', (isset($args['uname']) ? $args['uname'] : null), 'GET');
    if (is_null($userid) && !is_null($uname) && !empty($uname)) {
        $userid = pnUserGetIDFromName($uname);
    }

    if ($userid == 1) {
        return LogUtil::registerError(_MODARGSERROR);
    }

    if (is_null($uname)) {
        $uname = pnUserGetVar('uname', $userid);
    }

    // create the output object
    $pnRender = pnRender::getInstance('Users', false);

    $pnRender->assign('userid', $userid);
    $pnRender->assign('uname', $uname);

    // return output
    return $pnRender->fetch('users_admin_deleteusers.htm');
}

/**
 * users_admin_mailusers()
 *
 * Display a form to send an e-mail to particular users
 * @param  $args
 * @return string   render output
 */
function users_admin_mailusers($args)
{
    // check permissions
    if (!SecurityUtil::checkPermission('Users::MailUsers', '::', ACCESS_EDIT)) {
        return LogUtil::registerPermissionError();
    }

    // get the input arguments
    $op     = FormUtil::getPassedValue('op', null, 'GET');
    $userid = FormUtil::getPassedValue('userid', null, 'GET');

    if (isset($args['op']) && !empty($args['op'])) {
        $op = $args['op'];
    }
    if (isset($args['userid']) && is_numeric($args['userid'])) {
        $userid = $args['userid'];
    }

    // get the groups from the groups API and process into
    // a form useful for the html_options smarty plugin
    $groups = array();
    if (pnModAvailable('Groups')) {
        $groupsapi = pnModAPIFunc('Groups', 'user', 'getall');
        foreach ($groupsapi as $group) {
            $groups[$group['gid']] = $group['name'];
        }
    }

    // create the output oject
    $pnRender = pnRender::getInstance('Users', false);

    // if we have user id from the input then assign the username
    if (isset($userid) && is_numeric($userid)) {
        $pnRender->assign('userid', $userid);
        $pnRender->assign('uname', pnUserGetVar('uname', $userid));
    }

    $pnRender->assign('groups', $groups);

    // return the output object
    return $pnRender->fetch('users_admin_mailusers.htm');
}


/**
 * users_admin_mailuserssend()
 *
 * This sends an email with given settings
 *
 * @author       Xiaoyu Huang
 * @todo         Move groups table sql to groups API
 * @todo         Move users table sql to users API
 * @todo         Unify with users_admin_processusers(op=mail) and users_adminapi_mailuser
 */
function users_admin_mailuserssend()
{
    // check permissions
    if (!SecurityUtil::checkPermission('Users::MailUsers', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    // get the input arguments
    $from          = FormUtil::getPassedValue('from', null, 'REQUEST');
    $rpemail       = FormUtil::getPassedValue('rpemail', null, 'REQUEST');
    $subject       = FormUtil::getPassedValue('subject', null, 'REQUEST');
    $message       = FormUtil::getPassedValue('message', null, 'REQUEST');
    $uname         = FormUtil::getPassedValue('uname', null, 'REQUEST');
    $unamecontains = FormUtil::getPassedValue('unamecontains', null, 'REQUEST');
    $rname         = FormUtil::getPassedValue('rname', null, 'REQUEST');
    $ugroup        = FormUtil::getPassedValue('ugroup', null, 'REQUEST');
    $email         = FormUtil::getPassedValue('email', null, 'REQUEST');
    $homepage      = FormUtil::getPassedValue('homepage', null, 'REQUEST');
    $icq           = FormUtil::getPassedValue('icq', null, 'REQUEST');
    $msn           = FormUtil::getPassedValue('msn', null, 'REQUEST');
    $aim           = FormUtil::getPassedValue('aim', null, 'REQUEST');
    $yim           = FormUtil::getPassedValue('yim', null, 'REQUEST');
    $signature     = FormUtil::getPassedValue('signature', null, 'REQUEST');
    $regdateafter  = FormUtil::getPassedValue('regdateafter', null, 'REQUEST');
    $regdatebefore = FormUtil::getPassedValue('regdatebefore', null, 'REQUEST');
    $testonly      = FormUtil::getPassedValue('testonly', null, 'REQUEST');
    $perpage       = FormUtil::getPassedValue('perpage', null, 'REQUEST');
    $page          = FormUtil::getPassedValue('page', null, 'REQUEST');
    $totalpages    = FormUtil::getPassedValue('totalpages', null, 'REQUEST');
    $startat       = FormUtil::getPassedValue('startat', null, 'REQUEST');
    $output        = FormUtil::getPassedValue('output', null, 'REQUEST');

    $errorMsg = '';
    if (empty($from)) {
        $errorMsg = _USERS_NM_ERROR2;
    } elseif (!pnVarValidate($rpemail,'email')) {
        $errorMsg = _USERS_NM_ERROR3;
    } elseif (empty($subject)) {
        $errorMsg = _USERS_NM_ERROR4;
    } elseif (empty($message)) {
        $errorMsg = _USERS_NM_ERROR5;
    }
    if (!empty($errorMsg)) {
        return LogUtil::registerError($errorMsg, null, pnModURL('Users', 'admin', 'mailusers'));
    }


    if (!isset($perpage) || empty($perpage)) {
        $perpage = 500;
    }
    if (!isset($startat) || empty($startat)) {
        $startat = 0;
    }

    $pntable = pnDBGetTables();
    $column  = $pntable['users_column'];

    if (!isset($condition) || $condition == '') {
        $condition = '1=1';

        if ($uname != '') {
            $condition .= " AND ".$column['uname']." = '".DataUtil::formatForStore($uname)."'";
        }
        if ($unamecontains != '') {
            $condition .= " AND INSTR(".$column['uname'].",'".DataUtil::formatForStore($unamecontains)."')>0";
        }
        if ($rname != '') {
            $condition .= " AND INSTR(".$column['name'].",'".DataUtil::formatForStore($rname)."')>0";
        }
    if ($ugroup != 0) {
            $uids = DBUtil::selectFieldArrayByID ('group_membership', 'uid', $ugroup, 'gid');
            $condition .= " AND (";
        foreach ($uids as $uid) {
                $condition.="".$column['uid']."='".DataUtil::formatForStore($uid)."' OR ";
            }
            $condition .= '0)';
        }
        if ($email != '') {
            $condition .= " AND INSTR(".$column['email'].",'".DataUtil::formatForStore($email)."')>0";
        }
        if ($homepage != '') {
            $condition .= " AND INSTR(".$column['url'].",'".DataUtil::formatForStore($homepage)."')>0";
        }
        if ($icq != '') {
            $condition .= " AND INSTR(".$column['user_icq'].",'".DataUtil::formatForStore($icq)."')>0";
        }
        if ($msn != '') {
            $condition .= " AND INSTR(".$column['user_msnm'].",'".DataUtil::formatForStore($msn)."')>0";
        }
        if ($aim != '') {
            $condition .= " AND INSTR(".$column['user_aim'].",'".DataUtil::formatForStore($aim)."')>0";
        }
        if ($yim != '') {
            $condition .= " AND INSTR(".$column['user_yim'].",'".DataUtil::formatForStore($yim)."')>0";
        }
        if ($signature != '') {
            $condition .= " AND INSTR(".$column['user_sig'].",'".DataUtil::formatForStore($signature)."')>0";
        }
        if ($regdateafter != '') {
            $condition .= " AND ".$column['user_regdate'].">UNIX_TIMESTAMP('".DataUtil::formatForStore($regdateafter)."')";
        }
        if ($regdatebefore != '') {
            $condition .= " AND ".$column['user_regdate']."<UNIX_TIMESTAMP('".DataUtil::formatForStore($regdatebefore)."')";
        }
    }

    $where = $condition . " AND " . $column['uname'] . " != '" . pnModGetVar('Users', 'anonymous') . "'";
    $count = DBUtil::selectObjectCount ('users', $where);
    if ($count == 0) {
        return _USERS_NM_NOMATCHUSER;
    } else {
        $sort = 'uid';
        $users = DBUtil::selectObjectArray ('users', $where, $sort, $startat, $perpage);
        if ($users) {
            $page = $startat/$perpage+1;
            $totalpages = ceil($count/$perpage);
            $output = _USERS_NM_SENDING._USERS_NM_PAGE1.$page._USERS_NM_PAGE2.$totalpages._USERS_NM_PAGE3.$count._USERS_NM_USERS."<br />";

            foreach ($users as $userinfo) {
                if ($testonly == 1) {
                    $output .= _USERS_NM_TESTING.$userinfo['uname']." "._USERS_NM_EMAIL.$userinfo['email']."<br />";
                } else {
                    pnMail($rpemail, $subject, $message, "From: \"".$from."\" <".$rpemail.">\nBcc: ".$userinfo['email']."\nX-Mailer: PHP/" . phpversion());
                    $output .= _USERS_NM_SENDING.$userinfo['uname']." "._USERS_NM_EMAIL.$userinfo['email']."<br />";
                }
            }
            $modname = pnModGetName();

            pnModDelVar($modname, 'mailusers_from');
            pnModDelVar($modname, 'mailrpemail');
            pnModDelVar($modname, 'mailsubject');
            pnModDelVar($modname, 'mailmessage');
            pnModDelVar($modname, 'mailuname');
            pnModDelVar($modname, 'mailrname');
            pnModDelVar($modname, 'mailugroup');
            pnModDelVar($modname, 'mailemail');
            pnModDelVar($modname, 'mailhomepage');
            pnModDelVar($modname, 'mailicq');
            pnModDelVar($modname, 'mailmsn');
            pnModDelVar($modname, 'mailaim');
            pnModDelVar($modname, 'mailyim');
            pnModDelVar($modname, 'mailsignature');
            pnModDelVar($modname, 'mailregdateafter');
            pnModDelVar($modname, 'mailregdatebefore');
            pnModDelVar($modname, 'mailtestonly');
            pnModDelVar($modname, 'mailperpage');
            pnModDelVar($modname, 'mailpage');
            pnModDelVar($modname, 'mailtotalpages');
            pnModDelVar($modname, 'mailstartat');
            pnModDelVar($modname, 'mailoutput');

            pnModSetVar($modname, 'mailfrom', $from);
            pnModSetVar($modname, 'mailrpemail', $rpemail);
            pnModSetVar($modname, 'mailsubject', $subject);
            pnModSetVar($modname, 'mailmessage', $message);
            pnModSetVar($modname, 'mailuname', $uname);
            pnModSetVar($modname, 'mailrname', $rname);
            pnModSetVar($modname, 'mailugroup', $ugroup);
            pnModSetVar($modname, 'mailemail', $email);
            pnModSetVar($modname, 'mailhomepage', $homepage);
            pnModSetVar($modname, 'mailicq', $icq);
            pnModSetVar($modname, 'mailmsn', $msn);
            pnModSetVar($modname, 'mailaim', $aim);
            pnModSetVar($modname, 'mailyim', $yim);
            pnModSetVar($modname, 'mailsignature', $signature);
            pnModSetVar($modname, 'mailregdateafter', $regdateafter);
            pnModSetVar($modname, 'mailregdatebefore', $regdatebefore);
            pnModSetVar($modname, 'mailtestonly', $testonly);
            pnModSetVar($modname, 'mailperpage', $perpage);
            pnModSetVar($modname, 'mailpage', $page);
            pnModSetVar($modname, 'mailtotalpages', $totalpages);
            pnModSetVar($modname, 'mailstartat', $startat);
            pnModSetVar($modname, 'mailoutput', $output);

            pnRedirect(pnModURL('Users', 'admin', 'mailuserssendnext'));

        } else {
            $output .= _USERS_NM_FINISH;
        }
    }
    return $output;
}


/**
 * Display form to continue sending mail
 *
 * This function shows a form to continue send the e-mail
 * when a bulk mail is split to ensure process completes
 *
 * @author       Xiaoyu Huang
 * @todo         template output
 * @return       output       Returned markup
 */
function users_admin_mailuserssendnext()
{
    // check permissions
    if (!SecurityUtil::checkPermission('Users::MailUsers', '::', ACCESS_ADMIN)) {
        return LogUtil::registerPermissionError();
    }

    $modname = pnModGetName();
    $page = pnModGetVar($modname, 'mailpage');
    $totalpages = pnModGetVar($modname, 'mailtotalpages');
    $perpage = pnModGetVar($modname, 'mailperpage');
    $startat = pnModGetVar($modname, 'mailstartat');
    $output = pnModGetVar($modname, 'mailoutput');
    if ($page++ == $totalpages) {
        $output .= _USERS_NM_FINISH . "<br />";
    } else {
        $startat += $perpage;

        $output .= _USERS_NM_PAGE1.($page-1)._USERS_NM_PAGE2.$totalpages.' '._USERS_NM_SEND._USERS_NM_OK._USERS_NM_NOWPROCCESS."<br />";
        $output2 = $output;
        $output .= '
    <form action="'.pnModUrl($modname, 'admin', 'mailuserssend').'" method="post" name="mailform" id="mailform">
    ';
        $output .= users_makehiddenfield("output",$output2);
        $output .= users_makehiddenfield("from",pnModGetVar($modname, 'mailfrom'));
        $output .= users_makehiddenfield("rpemail",pnModGetVar($modname, 'mailrpemail'));
        $output .= users_makehiddenfield("subject",pnModGetVar($modname, 'mailsubject'));
        $output .= users_makehiddenfield("message",pnModGetVar($modname, 'mailmessage'));
        $output .= users_makehiddenfield("uname",pnModGetVar($modname, 'mailuname'));
        $output .= users_makehiddenfield("rname",pnModGetVar($modname, 'mailrname'));
        $output .= users_makehiddenfield("ugroup",pnModGetVar($modname, 'mailugroup'));
        $output .= users_makehiddenfield("email",pnModGetVar($modname, 'mailemail'));
        $output .= users_makehiddenfield("homepage",pnModGetVar($modname, 'mailhomepage'));
        $output .= users_makehiddenfield("icq",pnModGetVar($modname, 'mailicq'));
        $output .= users_makehiddenfield("msn",pnModGetVar($modname, 'mailmsn'));
        $output .= users_makehiddenfield("aim",pnModGetVar($modname, 'mailaim'));
        $output .= users_makehiddenfield("yim",pnModGetVar($modname, 'mailyim'));
        $output .= users_makehiddenfield("signature",pnModGetVar($modname, 'mailsignature'));
        $output .= users_makehiddenfield("regdateafter",pnModGetVar($modname, 'mailregdateafter'));
        $output .= users_makehiddenfield("regdatebefore",pnModGetVar($modname, 'mailregdatebefore'));
        $output .= users_makehiddenfield("testonly",pnModGetVar($modname, 'mailtestonly'));
        $output .= users_makehiddenfield("perpage",$perpage);
        $output .= users_makehiddenfield("startat",$startat);
        $output .= users_makehiddenfield("page",$page);
        $output .= users_makehiddenfield("totalpages",$totalpages);
        $output .= '
        <input type="submit" name="Submit" value="'._USERS_NM_SENDNEXTPAGE.'" />
    </form>
    ';
    }
    return $output;
}


/**
 * Create hidden field
 *
 * Utility function to create a hidden form field
 *
 * @author       Xiaoyu Huang
 * @return       output       A hidden field for given parameters
 */
function users_makehiddenfield($name, $value)
{
    return '<input name="'.$name.'" type="hidden" id="'.$name.'" value="'.$value.'">';
}

/**
 * users_admin_modifyconfig()
 *
 * User configuration settings
 * @author Xiaoyu Huang
 * @see function settings_admin_main()
 * @return string HTML string
 **/
function users_admin_modifyconfig()
{
    // Security check
    if (!(SecurityUtil::checkPermission('Users::', '::', ACCESS_ADMIN))) {
        return LogUtil::registerPermissionError();
    }

    // Create output object
    $pnRender = pnRender::getInstance('Users', false);

    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule)) {
        // load the profile module language file
        pnModLangLoad($profileModule, 'user');
    }

    // assign the module vars
    $config = pnModGetVar('Users');
    $pnRender->assign('config', $config);

    // Return the output that has been generated by this function
    return $pnRender->fetch('users_admin_modifyconfig.htm');
}

/**
 * users_admin_updateconfing()
 *
 * Update user configuration settings
 * @author Xiaoyu Huang
 * @see function settings_admin_main()
 * @return string HTML string
 **/
function users_admin_updateconfig()
{
    // security check
    if (!(SecurityUtil::checkPermission('Users::', '::', ACCESS_ADMIN)))
        return LogUtil::registerPermissionError();

    // get our input
    $config = FormUtil::getPassedValue('config', '', 'POST');

    if (!isset($config['reg_noregreasons'])) {
        $config['reg_noregreasons'] = '';
    }

    pnModSetVar('Users', 'itemsperpage', $config['itemsperpage']);
    pnModSetVar('Users', 'userimg', $config['userimg']);
    pnModSetVar('Users', 'reg_uniemail', $config['reg_uniemail']);
    pnModSetVar('Users', 'reg_optitems', $config['reg_optitems']);
    pnModSetVar('Users', 'reg_allowreg', $config['reg_allowreg']);
    pnModSetVar('Users', 'reg_noregreasons', $config['reg_noregreasons']);
    pnModSetVar('Users', 'moderation', $config['moderation']);
    pnModSetVar('Users', 'reg_verifyemail', $config['reg_verifyemail']);
    pnModSetVar('Users', 'reg_notifyemail', $config['reg_notifyemail']);
    pnModSetVar('Users', 'reg_Illegaldomains', $config['reg_Illegaldomains']);
    pnModSetVar('Users', 'reg_Illegalusername', $config['reg_Illegalusername']);
    pnModSetVar('Users', 'reg_Illegaluseragents', $config['reg_Illegaluseragents']);
    pnModSetVar('Users', 'minage', $config['minage']);
    pnModSetVar('Users', 'minpass', $config['minpass']);
    pnModSetVar('Users', 'anonymous', $config['anonymous']);
    pnModSetVar('Users', 'savelastlogindate', $config['savelastlogindate']);
    pnModSetVar('Users', 'loginviaoption', $config['loginviaoption']);
    pnModSetVar('Users', 'hash_method', $config['hash_method']);
    pnModSetVar('Users', 'login_redirect', $config['login_redirect']);
    pnModSetVar('Users', 'reg_question', $config['reg_question']);
    pnModSetVar('Users', 'reg_answer', $config['reg_answer']);
    pnModSetVar('Users', 'idnnames', $config['idnnames']);

    // Let any other modules know that the modules configuration has been updated
    pnModCallHooks('module','updateconfig','Users', array('module' => 'Users'));

    // the module configuration has been updated successfuly
    LogUtil::registerStatus (_CONFIGUPDATED);

    return pnRedirect(pnModURL('Users', 'admin', 'modifyconfig'));
}
