<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2001, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.legaluserlinks.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package      Zikula_System_Modules
 * @subpackage   legal
 */


/**
 * Smarty function to display admin links for the example module
 * based on the user's permissions
 *
 * Example
 * <!--[legaluserlinks start="[" end="]" seperator="|" class="pn-menuitem-title"]-->
 *
 * @author       Mark West
 * @since        24/11/04
 * @see          function.legaluserlinks.php::smarty_function_legaluserlinks()
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @param        string      $start       start string
 * @param        string      $end         end string
 * @param        string      $seperator   link seperator
 * @param        string      $class       CSS class
 * @return       string      the results of the module function
 */
function smarty_function_legaluserlinks($params, &$smarty)
{
    extract($params);
    unset($params);

    // set some defaults
    if (!isset($start)) {
        $start = '[';
    }
    if (!isset($end)) {
        $end = ']';
    }
    if (!isset($seperator)) {
        $seperator = '|';
    }
    if (!isset($class)) {
        $class = 'pn-menuitem-title';
    }

    $adminlinks = "<span class=\"$class\">$start ";

    if (SecurityUtil::checkPermission('legal::', 'termsofuse::', ACCESS_OVERVIEW) && pnModGetVar('legal', 'termsofuse')) {
        $adminlinks .= "<a href=\"" . DataUtil::formatForDisplay(pnModURL('legal', 'user', 'termsofuse')) . "\">" . _LEGAL_TERMSOFUSE . "</a> ";
    }
    if (SecurityUtil::checkPermission('legal::', 'privacypolicy::', ACCESS_OVERVIEW) && pnModGetVar('legal', 'privacypolicy')) {
        $adminlinks .= "$seperator <a href=\"" . DataUtil::formatForDisplay(pnModURL('legal', 'user', 'privacy')) . "\">" . _LEGAL_PRIVACYPOLICY . "</a> ";
    }
    if (SecurityUtil::checkPermission('legal::', 'accessibilitystatement::', ACCESS_OVERVIEW) && pnModGetVar('legal', 'accessibilitystatement')) {
        $adminlinks .= "$seperator <a href=\"" . DataUtil::formatForDisplay(pnModURL('legal', 'user', 'accessibilitystatement')) . "\">" . _LEGAL_ACCESSIBILITYSTATEMENT . "</a> ";
    }

    $adminlinks .= "$end</span>\n";

    return $adminlinks;
}
