<?php
/**
 * Date input plugin
 *
 * @copyright (c) 2006, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.pnformdateinput.php 24436 2008-07-03 14:13:08Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Jorn Wildt
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/** Make sure to use require_once() instead of Loader::requireOnce() since "function.pnformtextinput.php"
 is loaded by Smarty (the base render class) with the use of require_once(). We do not want to
 get in conflict with that.*/
require_once('system/pnForm/plugins/function.pnformtextinput.php');


/**
 * Date input for pnForms
 *
 * The date input plugin is a text input plugin that only allows dates to be posted. The value
 * returned from {@link pnForm::pnFormGetValues()} is although a string of the format 'YYYY-MM-DD'
 * since this is the standard internal Zikula format for dates.
 *
 * You can also use all of the features from the pnFormTextInput plugin since the date input
 * inherits from it.
 *
 * @package pnForm
 * @subpackage Plugins
 */
class pnFormDateInput extends pnFormTextInput
{
    /**
     * Enable or disable input of time in addition to the date
     * @var bool
     */
    var $includeTime;
    var $initDate;

    /**
     * Default date value
     *
     * This parameter enables the input to be pre-filled with the current date or similar other well defined
     * default values.
     * You can set the default value to be one of the following:
     * - now: current date and time
     * - today: current date
     * - monthstart: first day in current month
     * - monthend: last day in current month
     * - yearstart: first day in the year
     * - yearend: last day in the year
     * - custom: inital Date
     */
    var $defaultValue;


    function getFilename()
    {
        return __FILE__; // FIXME: may be found in smarty's data???
    }


    function create(&$render, &$params)
    {
       $this->includeTime = (array_key_exists('includeTime', $params) ? $params['includeTime'] : 0);
       $this->defaultValue = (array_key_exists('defaultValue', $params) ? $params['defaultValue'] : null);
       $this->initDate = (array_key_exists('initDate', $params) ? $params['initDate'] : 0);
       $this->maxLength = ($this->includeTime ? 18 : 12);
       $params['width'] = ($this->includeTime ? '10em' : '8em');;
       
       parent::create($render, $params);
       
       $this->cssClass .= ' date';
    }


    function render(&$render)
    {
        static $firstTime = true;

        if (!empty($this->defaultValue) && !$render->pnFormIsPostBack())
        {
            $d = strtolower($this->defaultValue);
            $now = getdate();
            $date = null;

            if ($d == 'now')
            {
                $date = time();
            }
            else if ($d == 'today')
            {
                $date = mktime(0,0,0, $now['mon'], $now['mday'], $now['year']);
            }
            else if ($d == 'monthstart')
            {
                $date = mktime(0,0,0, $now['mon'], 1, $now['year']);
            }
            else if ($d == 'monthend')
            {
                $daysInMonth = date('t');
                $date = mktime(0,0,0, $now['mon'], $daysInMonth, $now['year']);
            }
            else if ($d == 'yearstart')
            {
                $date = mktime(0,0,0, 1, 1, $now['year']);
            }
            else if ($d == 'yearend')
            {
                $date = mktime(0,0,0, 12, 31, $now['year']);
            }
            else if ($d == 'custom')
            {
                $date = strtotime($this->initDate);
            }

            if ($date != null)
                $this->text = DateUtil::getDatetime($date, ($this->includeTime ? _DATETIMEINPUT : _DATEINPUT));
            else
                $this->text = 'Unknown default date';
        }

        $result = '<span class="date" style="white-space: nowrap">';

        $result .= parent::render($render);

        $txt = _PNFORM_SELECTDATE;
        $result .= " <img id=\"{$this->id}_img\" src=\"javascript/jscalendar/img.gif\" style=\"vertical-align: middle\" class=\"clickable\" alt=\"$txt\" /></span>";

        if ($firstTime)
        {
            $headers[] = 'javascript/jscalendar/calendar.js';
            $lang = SessionUtil::getVar('lang');
            $lang = substr($lang, 0, 2);
            $headers[] = "javascript/jscalendar/lang/calendar-$lang.js";
            $headers[] = 'javascript/jscalendar/calendar-setup.js';
            PageUtil::addVar('stylesheet', 'javascript/jscalendar/calendar-win2k-cold-2.css');
            PageUtil::addVar('javascript', $headers);
        }
        $firstTime = false;
        if($this->includeTime) {
            $this->initDate = str_replace('-',',', $this->initDate);
            $result .= "<script type=\"text/javascript\">
            Calendar.setup(
                {
                    inputField : \"{$this->id}\",
                    ifFormat : \"%Y-%m-%d %H:%M\",
                    showsTime      :    true,
                    timeFormat     :    \"24\",
                    button : \"{$this->id}_img\",
                    singleClick    :    false,
                    firstDay: " . _DATEFIRSTWEEKDAY . "
                });
                </script>";
        } else {
            $result .= "<script type=\"text/javascript\">
            Calendar.setup(
                {
                    inputField : \"{$this->id}\",
                    ifFormat : \"%Y-%m-%d\",
                    button : \"{$this->id}_img\",
                    firstDay: " . _DATEFIRSTWEEKDAY . "
                }
            );
            </script>";
        }
        return $result;
    }


    function parseValue(&$render, $text)
    {
      if (empty($text))
          return null;
      return $text;
    }


    function validate(&$render)
    {
        parent::validate($render);
        if (!$this->isValid)
            return;

        if (strlen($this->text) > 0)
        {
            if ($this->includeTime)
                $dateValue = DateUtil::parseUIDateTime($this->text);
            else
                $dateValue = DateUtil::parseUIDate($this->text);

            if ($dateValue == null)
                $this->setError(_NOTAVALIDDATE);
        }
    }


    function formatValue(&$render, $value)
    {
        return DateUtil::formatDatetime($value, ($this->includeTime ? _DATETIMEINPUT : _DATEINPUT));
    }
}


function smarty_function_pnformdateinput($params, &$render)
{
    return $render->pnFormRegisterPlugin('pnFormDateInput', $params);
}

?>
