<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.html_select_modulestylesheets.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display a drop down list of module stylesheets
 *
 * Available parameters:
 *   - modname   The module name to show the styles for
 *   - assign:   If set, the results are assigned to the corresponding variable instead of printed out
 *   - id:       ID for the control
 *   - name:     Name for the control
 *   - exclude   Comma seperated list of files to exclude (optional)
 *   - selected: Selected value
 *
 * Example
 *   <!--[modstyleslist name=modulestylesheet selected=navtabs.css]-->
 *
 *
 * @author       Mark West
 * @since        24 July 2005
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      the value of the last status message posted, or void if no status message exists
 */
function smarty_function_html_select_modulestylesheets($params, &$smarty)
{
    extract($params);

    unset($params['name']);
    unset($params['id']);
    unset($params['selected']);
    unset($params['modname']);

    if (!isset($modname)) {
        $smarty->trigger_error('html_select_modulestylesheets: parameter [modname] required');
        return false;
    }

    if (!isset($name)) {
        $name = null;
    }
    if (!isset($selected)) {
        $selected = null;
    }

    if (isset($exclude)) {
        $exclude = explode(',', trim($exclude));
    } else {
        $exclude = array();
    }

    $modstyleslist = pnModAPIFunc('Admin', 'admin', 'getmodstyles', array('modname' => $modname, 'exclude' => $exclude));

    require_once $smarty->_get_plugin_filepath('function','html_options');
    $output = smarty_function_html_options(array('values'  => $modstyleslist,
                                                 'output'  => $modstyleslist,
                                                 'selected' => $selected,
                                                 'name'     => $name,
                                                 'id'       => isset($id) ? $id : null),
                                                 $smarty);

    if (isset($assign)) {
        $smarty->assign($assign, $output);
    } else {
        return $output;
    }
}
