<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: outputfilter.pagevars.php 19321 2006-06-29 13:15:10Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display a drop down list of themes
 *
 * Available parameters:
 *   - name:     Name for the control (optional) if not present then only the option tags are output
 *   - id:       ID for the control
 *   - selected: Selected value
 *   - filter:   Filter themes use (possible values: PNTHEME_FILTER_ALL (default) PNTHEME_FILTER_USER, PNTHEME_FILTER_SYSTEM, PNTHEME_FILTER_ADMIN
 *   - state:    Filter themes by state (possible values: PNTHEME_STATE_ALL (default), PNTHEME_STATE_ACTIVE, PNTHEME_STATE_INACTIVE
 *   - type:     Filter themes by type (possible values: PNTHEME_TYPE_ALL (default), PNTHEME_TYPE_LEGACY, PNTHEME_TYPE_XANTHIA2, PNTHEME_TYPE_XANTHIA3, PNTHEME_TYPE_AUTOTHEME
 *   - assign:   If set, the results are assigned to the corresponding variable instead of printed out
 *
 * Examples
 *
 *     <!--[html_select_themes name=mytheme selected=mythemechoice]-->
 *
 *     <select name="mytheme">
 *         <option value=""><!--[pnml name=_DEFAULT]--></option>
 *         <!--[html_select_themes selected=$mythemechoice]-->
 *     </select>
 *
 * @author       Mark West
 * @since        26 October 2005
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      the value of the last status message posted, or void if no status message exists
 */
function smarty_function_html_select_themes($params, &$smarty)
{
    extract($params);

    unset($params['name']);
    unset($params['id']);
    unset($params['selected']);
    unset($params['filter']);
    unset($params['state']);
    unset($params['type']);

    if (!isset($filter) || !defined($filter)) {
        $filter = PNTHEME_FILTER_ALL;
    } else {
        $filter = constant($filter);
    }
    if (!isset($state) || !defined($state)) {
        $state = PNTHEME_STATE_ALL;
    } else {
        $state = constant($state);
    }
    if (!isset($type) || !defined($type)) {
        $type = PNTHEME_TYPE_ALL;
    } else {
        $type = constant($type);
    }
    if (!isset($name)) {
        $name = null;
    }
    if (!isset($selected)) {
        $selected = null;
    }

    $themelist = array();
    $themes = ThemeUtil::getAllThemes($filter, $state, $type);
    if (!empty($themes)) {
        foreach ($themes as $theme) {
            $themelist[$theme['name']] = $theme['displayname'];
        }
    }
    natcasesort($themelist);

    require_once $smarty->_get_plugin_filepath('function','html_options');
    $output = smarty_function_html_options(array('options'  => $themelist,
                                                 'selected' => $selected,
                                                 'name'     => $name,
                                                 'id'       => isset($id) ? $id : null),
                                                 $smarty);

    if (isset($assign)) {
        $smarty->assign($assign, $output);
    } else {
        return $output;
    }
}
