<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.pnbutton.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display an image form submission button
 *
 * BEWARE: Internt Explorer 6.x does NOT work especially well with <button> tags!
 *
 * available parameters:
 *  - assign      if set, the button will be assigned to this variable
 *  - type        if set, the type of button that will be generated (default: submit)
 *  - mode        if set, the type of HTML element to be used (default: <button>). Values = [button|input]
 *  - name        if set, the name of button that will be generated (default: value of 'type' parameter)
 *  - value       if set, the value of button that will be generated
 *  - id          if set, the ID of button
 *  - class       if set, the class of button
 *  - src         image source name 
 *  - set         image set name
 *  - alt         if set, the constant that will be used for the alt attribute
 *  - title       if set, the constant that will be used for the title attribute
 *  - constants   if false then both alt and title will be assumed to be the final strings not constants
 *  - gt		  if true assume that gettext translation should be looked up for alt and title
 *
 * @author   Mark West
 * @since    17th Nov. 2005
 * @param    array    $params     All attributes passed to this function from the template
 * @param    object   $smarty     Reference to the Smarty object
 * @return   string   the version string
 */
function smarty_function_pnbutton($params, &$smarty)
{
    extract($params);
    unset($params);

    // we're going to make use of pnimg for path searching
    require_once $smarty->_get_plugin_filepath('function', 'pnimg');

    // set some defaults
    if (!isset($constants) || !is_bool($constants)) {
        $constants = true;
    }
    if (isset($gt) && is_bool($gt) && $gt) {
        $gt = true;
        $constants = false;
    } else {
        $gt = false;
        $constants = true;
    }

    if (!isset($alt)) {    	
    	$gt ? $alt = _("Submit") : $alt = _SUBMIT;
    } elseif ($gt) {
    	$alt = _($alt);
    } elseif ($constants) {
        $alt = constant($alt);
    }
    if (!isset($title)) {
    	$gt ? $title = _("Submit") : $title = _SUBMIT;
    } elseif ($gt) {
    	$title = _($title);
    } elseif ($constants) {
    	$title = constant($title);
    }


    if (!isset($type)) {
        $type = 'submit';
    }
    if (!isset($mode)) {
        $mode = 'button';
    }
    if (isset($name)) {
        $name = ' name="'.DataUtil::formatForDisplay($name).'"';
    } else {
        $name = ' name="'.DataUtil::formatForDisplay($type).'"';
    }
    if (isset($value)) {
        $value = ' value="'.DataUtil::formatForDisplay($value).'"';
    } else {
        $value = '';
    }
    if (isset($id)) {
        $id = ' id="'.DataUtil::formatForDisplay($id).'"';
    } else {
        $id = '';
    }
    if (isset($class)) {
        $class = ' class="'.DataUtil::formatForDisplay($class).'"';
    } else {
        $class = '';
    }

    // call the pnimg plugin and work out the src from the assigned template vars
    smarty_function_pnimg(array('assign' => 'buttonsrc', 'src' => $src, 'set' => $set, 'modname' => 'core'), $smarty);
    $imgvars = $smarty->get_template_vars('buttonsrc');
    $imgsrc = $imgvars['src'];

    // form the button html
    if ($mode == 'button') {
        $return = '<button'.$id.$class.' type="'.DataUtil::formatForDisplay($type).'"'.$name.$value.' title="'.DataUtil::formatForDisplay($title).'"><img src="'.DataUtil::formatForDisplay($imgsrc).'" alt="'.DataUtil::formatForDisplay($alt).'" /></button>';
    } else {
        $return = '<input'.$id.$class.' type="image"'.$name.$value.' title="'.DataUtil::formatForDisplay($title).'" src="'.DataUtil::formatForDisplay($imgsrc).'" alt="'.DataUtil::formatForDisplay($alt).'" />';
    }

    if (isset($assign)) {
        $smarty->assign($assign, $return);
    } else {
        return $return;
    }
}
