<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.pncalendarinput.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display a calendar input control
 *
 * This function displays a javascript (jscalendar) calendar control
 *
 * Available parameters:
 *   - objectname     The name of the object the field will be placed in
 *   - htmlname:      The html fieldname under which the date value will be submitted
 *   - dateformat:    The dateformat to use for displaying the chosen date
 *   - defaultstring: The String to display before a value has been selected
 *   - defaultdate:   The Date the calendar should to default to (format: Y/m/d)
 *   - hidden:        Boolean to show a hidden input or not
 *   - display:       Boolean to show a display output (when date is added in a hidden field)
 *   - class:         The class to apply to the html elements
 *   - time:          If set show time selection
 *
 * Example
 * <!--[pncalendarinput objectname=myobject htmlname=from dateformat='%Y-%m-%d' defaultdate='2005/12/31']-->
 *
 * @author       Mark West
 * @author       Robert Gasch
 * @since        25/10/2005
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @param        assign      The smarty variable to assign the resulting menu HTML to
 * @return       string      the language constant
 */
function smarty_function_pncalendarinput($params, &$smarty)
{
    if (!isset($params['objectname'])) {
        $smarty->trigger_error('pncalendarinput: parameter [objectname] required');
        return false;
    }
    if (!isset($params['htmlname'])) {
        $smarty->trigger_error('pncalendarinput: parameter [htmlname] required');
        return false;
    }
    if (!isset($params['dateformat'])) {
        $smarty->trigger_error('pncalendarinput: parameter [dateformat] required');
        return false;
    }

    // start of old pncalendarinit
    // pagevars make an extra pncalendarinit obsolete, they take care about the fact
    // that the styles/jsvascript do not get loaded multiple times
    static $langFile; 
    
    if (!isset($langFile)) {
        $currentLang = SessionUtil::getVar('lang');
        if (!file_exists ($langFile = 'javascript/jscalendar/lang/calendar-' . convcallang($currentLang) . '.js')) {
            $langFile = 'javascript/jscalendar/lang/calendar-en.js';
        }
    }

    PageUtil::addVar('stylesheet', 'javascript/jscalendar/calendar-win2k-cold-2.css');
    PageUtil::addVar('javascript', 'javascript/jscalendar/calendar.js');
    PageUtil::addVar('javascript', $langFile);
    PageUtil::addVar('javascript', 'javascript/jscalendar/calendar-setup.js');
    // end of old pncalendarinit

    if (!isset($params['defaultstring'])) $params['defaultstring'] = null;
    if (!isset($params['defaultdate'])) $params['defaultdate'] = null;

    $html = '';

    $fieldKey    = $params['htmlname'];
    if ($params['objectname']) {
        $fieldKey  = $params['objectname'] . '[' . $params['htmlname'] . ']';
    }

    $triggerName = 'trigger_' . $params['htmlname'];
    $displayName = 'display_' . $params['htmlname'];

    if (isset($params['class']) && !empty($params['class'])) {
        $params['class'] = ' class="' . DataUtil::formatForDisplay($params['class']) . '"';
    }

    if (isset($params['display']) && $params['display']) {
        $html .= '<span id="'.$displayName.'"'.$params['class'].'>'.$params['defaultstring'].'</span>';
    }
    $html .= '&nbsp;';
    if (isset($params['hidden']) && $params['hidden']) {
        $html .= '<input type="hidden" name="'.$fieldKey.'" id="'.$params['htmlname'].'" value="'.$params['defaultdate'].'" />';
    }

    $html .= '<img src="javascript/jscalendar/img.gif" id="'.$triggerName.'" style="cursor: pointer; border: 0px solid blue;" title="' . DataUtil::formatForDisplay(_DATE_SELECTOR) . '"  alt="' . DataUtil::formatForDisplay(_DATE_SELECTOR) . '" onmouseover="this.style.background=\'blue\';" onmouseout="this.style.background=\'\'" />';

    $html .= '<script type="text/javascript"> Calendar.setup({';
    //$html .= 'ifFormat    : "%Y-%m-%d %H:%M:00",'; // universal format, don't change this!
    $html .= 'ifFormat    : "'.$params['dateformat'].'",';
    $html .= 'inputField  : "'.$params['htmlname'].'",';
    $html .= 'displayArea : "'.$displayName.'",';
    $html .= 'daFormat    : "'.$params['dateformat'].'",';
    $html .= 'button      : "'.$triggerName.'",';
    $html .= 'defaultDate : "'.$params['defaultdate'].'",';
    $html .= 'align       : "Tl",';

    if (isset($params['defaultdate']) && $params['defaultdate']) {
        $d = strtotime ($params['defaultdate']);
        $d = date ('Y/m/d', $d);
        $html .= 'date : "'.$d.'",';
    }

    if (isset($params['time'])) {    
        $html .= 'showsTime      :    true,';
        $html .= 'timeFormat     :    "' . DataUtil::formatForDisplay(_TIMEFORMAT) . '",';
    }

    $html .= 'singleClick : true }); </script>';

    return $html;

}

function convcallang($currentlang) 
{
    $alllangs = array('afr' => 'af',
                      'sqi' => 'al',
                      '???' => 'bg',
                      'por' => 'br', // ??
                      '???' => 'ca',
                      'ces' => 'cs-win',
                      'dan' => 'da',
                      'deu' => 'de',
                      '???' => 'du',
                      '???' => 'el',
                      'eng' => 'en',
                      'soa' => 'es',
                      'fin' => 'fi',
                      'fra' => 'fr',
                      'scr' => 'hr',
                      'hun' => 'hu',
                      'ita' => 'it',
                      '???' => 'jp',
                      '???' => 'ko',
                      '???' => 'lt',
                      '???' => 'lv',
                      'nld' => 'nl',
                      'nor' => 'no',
                      'pol' => 'pl',
                      'por' => 'pt',  // ??
                      'ron' => 'ro',
                      'rus' => 'ru',
                      'slv' => 'si',
                      'slk' => 'sk',
                      '???' => 'sp',
                      'swe' => 'sv',
                      'tur' => 'tr',
                      'zho' => 'zh');
    if (array_key_exists($currentlang, $alllangs)) {
        return $alllangs[$currentlang];
    }
    return 'en';
}
