<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.pnthemegetvar.php 24342 2008-06-06 12:03:14Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to get a colour definition from the theme
 *
 * This function returns the corresponding color define from the theme
 *
 * Available parameters:
 *   - name:    Name of the colour definition
 *   - default: If set, the default value to return if the variable is not set
 *   - assign:  If set, the results are assigned to the corresponding variable instead of printed out
 *
 * Example
 * <!--[pnthemegetvar name="bgcolor"]-->
 *
 * @author       Jrg Napp
 * @since        16. Sept. 2003
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      the colour definition
 */
function smarty_function_pnthemegetvar($params, &$smarty)
{
    $assign  = isset($params['assign'])  ? $params['assign']  : null;
    $default = isset($params['default']) ? $params['default'] : null;
    $name    = isset($params['name'])    ? $params['name']    : null;

    if (!$name) {
        $smarty->trigger_error('pnthemegetvar: variable [name] required');
        return false;
    }

    $result = ThemeUtil::getVar($name, $default);

    if ($assign) {
        $smarty->assign($assign, $result);
    } else {
        return $result;
    }
}
