<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: outputfilter.pagevars.php 19321 2006-06-29 13:15:10Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Smarty function to display the jsquicktags pseudo visual editor
 *
 * The quick tags template tag MUST appear after the textarea in the template
 *
 * Available parameters:
 *   - id:        CSS ID of the control to act on
 *   - assign:    If set, the results are assigned to the corresponding variable instead of printed out
 *
 * Example
 *	<div class="pn-adminformrow">
 *	    <label for="pages_content"><!--[pnml name="_CONTENT"]--></label>
 *      <textarea id="pages_content" name="section[content]" rows="10" cols="50"></textarea>
 *      <!--[quicktags id=pages_content]-->
 *  </div> *
 * @author       Mark West
 * @since        24/04/2006
 * @param        array       $params      All attributes passed to this function from the template
 * @param        object      &$smarty     Reference to the Smarty object
 * @return       string      the page breaker markup
 */
function smarty_function_quicktags($params, &$smarty)
{
    extract($params);
    unset($params['id']);
    unset($params['assign']);

    if (!isset($id)) {
        $smarty->trigger_error('quicktags: parameter [id] required');
        return false;
    }

    if (!pnConfigGetVar('jsquicktags')) {
        return;
    }

    // load the javascript into the header
    PageUtil::addVar('javascript', 'javascript/js_quicktags/js_quicktags.js');
    PageUtil::addVar('stylesheet', 'javascript/js_quicktags/js_quicktags.css');

    $quicktags = '<script type="text/javascript">edToolbar();</script>';
    $quicktags .= '<script type="text/javascript">var edCanvas = document.getElementById(\''.DataUtil::formatForDisplay($id).'\');</script>';

    if (isset($assign)) {
        $smarty->assign($assign, $quicktags);
    } else {
        return $quicktags;
    }
}
