<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: function.v4b_selector_object_array.php 19260 2006-06-12 13:08:15Z markwest $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @author Robert Gasch
 * @package Zikula_Template_Plugins
 * @subpackage Functions
 */

/**
 * Available parameters:
 *   - btnText:  If set, the results are assigned to the corresponding variable instead of printed out
 *   - cid:      category ID
 *
 * Example
 * <!--[v4b_rbs_getcategory cid="1" assign="category"]-->
 *
 */
function smarty_function_selector_category ($params, &$smarty)
{
    $categoryRegistryModule   = isset($params['categoryRegistryModule'])   ? $params['categoryRegistryModule']   : '';
    $categoryRegistryTable    = isset($params['categoryRegistryTable'])    ? $params['categoryRegistryTable']    : '';
    $categoryRegistryProperty = isset($params['categoryRegistryProperty']) ? $params['categoryRegistryProperty'] : '';

    $category         = isset($params['category'])         ? $params['category']         : 0;
    $path             = isset($params['path'])             ? $params['path']             : '';
    $pathfield        = isset($params['pathfield'])        ? $params['pathfield']        : 'path';
    $field            = isset($params['field'])            ? $params['field']            : 'id';
    $fieldIsAttribute = isset($params['fieldIsAttribute']) ? $params['fieldIsAttribute'] : null;
    $selectedValue    = isset($params['selectedValue'])    ? $params['selectedValue']    : 0;
    $defaultValue     = isset($params['defaultValue'])     ? $params['defaultValue']     : 0;
    $defaultText      = isset($params['defaultText'])      ? $params['defaultText']      : '';
    $allValue         = isset($params['allValue'])         ? $params['allValue']         : 0;
    $allText          = isset($params['allText'])          ? $params['allText']          : '';
    $lang             = isset($params['lang'])             ? $params['lang']             : SessionUtil::getVar('lang');
    $name             = isset($params['name'])             ? $params['name']             : 'defautlselectorname';
    $submit           = isset($params['submit'])           ? $params['submit']           : false;
    $recurse          = isset($params['recurse'])          ? $params['recurse']          : true;
    $relative         = isset($params['relative'])         ? $params['relative']         : true;
    $includeRoot      = isset($params['includeRoot'])      ? $params['includeRoot']      : false;
    $includeLeaf      = isset($params['includeLeaf'])      ? $params['includeLeaf']      : true;
    $all              = isset($params['all'])              ? $params['all']              : false;
    $displayPath      = isset($params['displayPath'])      ? $params['displayPath']      : false;
    $attributes       = isset($params['attributes'])       ? $params['attributes']       : null;
    $assign           = isset($params['assign'])           ? $params['assign']           : null;
    $editLink         = isset($params['editLink'])         ? $params['editLink']         : true;
    $multipleSize     = isset($params['multipleSize'])     ? $params['multipleSize']     : 1;
    $sortField        = isset($params['sortField'])        ? $params['sortField']        : null;
    $doReplaceRootCat = isset($params['doReplaceRootCat']) ? $params['doReplaceRootCat'] : null;

    Loader::loadClass('CategoryUtil');

    if (!$category && !$path && $categoryRegistryModule && $categoryRegistryTable && $categoryRegistryProperty) {
        Loader::loadClass('CategoryRegistryUtil');
        $category = CategoryRegistryUtil::getRegisteredModuleCategory ($categoryRegistryModule, $categoryRegistryTable, $categoryRegistryProperty);
    }

    $allCats = array();
    // if we don't have a category-id we see if we can get a category by path
    if (!$category && $path) {
        $category = CategoryUtil::getCategoryByPath ($path, $pathfield);
        $allCats  = CategoryUtil::getSubCategoriesForCategory ($category, $recurse, $relative, $includeRoot, 
                                                               $includeLeaf, $all, '', '', $attributes, $sortField);
    }
    else
    // check if we have an actual category object with a numeric ID set
    if (is_array($category) && isset($category['id']) && is_integer($category['id'])) {
        $allCats  = CategoryUtil::getSubCategoriesForCategory ($category, $recurse, $relative, $includeRoot, 
                                                               $includeLeaf, $all, '', '', $attributes, $sortField);
    }
    else
    // check if we have a numeric category 
    if (is_numeric($category)) {
        $category = CategoryUtil::getCategoryByID ($category);
        $allCats  = CategoryUtil::getSubCategoriesForCategory ($category, $recurse, $relative, $includeRoot, 
                                                               $includeLeaf, $all, '', '', $attributes, $sortField);
    }
    else
    // check if we have a string/path category 
    if (is_string($category) && strpos($category, '/')===0) {
        $category = CategoryUtil::getCategoryByPath ($category, $pathfield);
        $allCats  = CategoryUtil::getSubCategoriesForCategory ($category, $recurse, $relative, $includeRoot, 
                                                               $includeLeaf, $all, '', '', $attributes, $sortField);
    }

    $html = CategoryUtil::getSelector_Categories ($allCats, $field, $selectedValue, $name, $defaultValue, $defaultText, 
                                                  $allValue, $allText, $submit, $displayPath, $doReplaceRootCat, $multipleSize, $fieldIsAttribute);

    if ($editLink && !empty($category) && SecurityUtil::checkPermission( 'Categories::', "$category[id]::", ACCESS_EDIT)) {
        $url = DataUtil::formatForDisplay(pnModURL ('Categories', 'user', 'edit', array('dr' => $category['id'])));
        $html .= "&nbsp;&nbsp;<a href=\"$url\"><img src=\"".pnGetBaseURL()."images/icons/extrasmall/xedit.gif\" title=\"" . _CATEGORIES_EDIT_SUBCATEGORIES . '" alt="' . _CATEGORIES_EDIT_SUBCATEGORIES . '" /></a>';
    }

    if ($assign) {
        $smarty->assign($assign, $html);
    } else {
        return $html;
    }
}

