<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2004, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: modifier.userprofilelink.php 24953 2008-12-05 20:22:26Z Landseer $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Template_Plugins
 * @subpackage Modifiers
 */

/**
 * Smarty modifier to create a link to a users profile
 *
 * Example
 *
 *   Simple version, shows $username
 *   <!--[$username|userprofilelink]-->
 *   Simple version, shows $username, using class="classname"
 *   <!--[$username|userprofilelink:classname]-->
 *   Using profile.gif instead of username, no class
 *   <!--[$username|userprofilelink:'':'images/profile.gif']-->
 *
 *   Using language depending image from pnimg. Note that we pass
 *   the pnimg result array to the modifier as-is
 *   <!--[ pnimg src='profile.gif' assign=profile]-->
 *   <!--[$username|userprofilelink:'classname':$profile]-->
 *
 * @author       Frank Schummertz
 * @author       The pnForum team
 * @version      $Id: modifier.userprofilelink.php 24953 2008-12-05 20:22:26Z Landseer $
 * @param        $string    string       the users name
 * @param        $class     string       the class name for the link (optional)
 * @param        $image     string/array the image to show instead of the username (optional)
 *                                       may be an array as created by pnimg
 * @param        $maxLength integer      if set then user names are truncated to x chars
 * @return       string   the output
 */
function smarty_modifier_userprofilelink($string, $class = '', $image = '', $maxLength = 0)
{
    $string = DataUtil::formatForDisplay($string);
    $profileModule = pnConfigGetVar('profilemodule', '');
    if (!empty($profileModule) && pnModAvailable($profileModule)) {
        pnModLangLoad($profileModule, 'user');
        if (!empty($class)) {
            $class = ' class="' . DataUtil::formatForDisplay($class) . '"';
        }

        if (!empty($image)) {
            if (is_array($image)) {
                // if it is an array we assume that it is an pnimg array
                $show = '<img src="' . DataUtil::formatForDisplay($image['src']) . '" alt="' . DataUtil::formatForDisplay($image['alt']) . '" width="' . DataUtil::formatForDisplay($image['width']) . '" height="' . DataUtil::formatForDisplay($image['height']) . '" />';
            } else {
                $show = '<img src="' . DataUtil::formatForDisplay($image) . '" alt="' . $string . '" />';
            }
        } elseif($maxLength > 0) {
            // truncate the user name to $maxLength chars
            $showLength = strlen($show);
            $truncEnd = ($maxLength > $showLength) ? $showLength : $maxLength;
            $show = substr($string, 0, $truncEnd);
        } else {
            $show = $string;
        }
        
        if (strpbrk($string, '/:#')=='') {
            $string = '<a' . $class . ' title="' . DataUtil::formatForDisplay(_PROFILE) . ': ' . $string . '" href="' . pnGetBaseURL() . DataUtil::formatForDisplay(pnModURL($profileModule, 'user', 'view', array('uname' => $string))) . '">' . $show . '</a>';
        } else {
            $uid = pnUserGetIDFromName($string);
            $string = '<a' . $class . ' title="' . DataUtil::formatForDisplay(_PROFILE) . ': ' . $string . '" href="' . pnGetBaseURL() . DataUtil::formatForDisplay(pnModURL($profileModule, 'user', 'view', array('uid' => $uid))) . '">' . $show . '</a>';
        }
    }
    return $string;
}

// code taken from http://de2.php.net/manual/de/function.strpbrk.php#76169
// original author is unknown
//
if (!function_exists('strpbrk')) {
    function strpbrk( $haystack, $char_list )
    {
        $strlen = strlen($char_list);
        $found = false;
        for( $i=0; $i<$strlen; $i++ ) {
            if( ($tmp = strpos($haystack, $char_list{$i})) !== false ) {
                if( !$found ) {
                    $pos = $tmp;
                    $found = true;
                    continue;
                }
                $pos = min($pos, $tmp);
            }
        }
        if( !$found ) {
            return false;
        }
        return substr($haystack, $pos);
    }
}
