<?php
/**
 * Zikula Application Framework
 *
 * @copyright Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: upgrade.php 24737 2008-10-18 18:19:13Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @category Zikula_Core
 * @package Install_Upgrade
 * @subpackage upgrade10xto11x
 * @author "Frank Schummertz <frank@zikula.org>"
 */

/* Upgrade Zikula from 1.0.x to 1.1.1 */

// file_put_contents emulation for php 4 - code from http://uk2.php.net/manual/en/function.file-put-contents.php#68329
if ( !function_exists('file_put_contents') && !defined('FILE_APPEND') ) {
    define('FILE_APPEND', 1);
    function file_put_contents($n, $d, $flag = false) {
        $mode = ($flag == FILE_APPEND || strtoupper($flag) == 'FILE_APPEND') ? 'a' : 'w';
        $f = @fopen($n, $mode);
        if ($f === false) {
            return 0;
        } else {
            if (is_array($d)) $d = implode($d);
            $bytes_written = fwrite($f, $d);
            fclose($f);
            return $bytes_written;
        }
    }
}

ini_set('max_execution_time' , 86400);
define('_PNINSTALLVER', '1.1.1');
define('_ZIKULAUPGRADE', 'running');

// include config file for retrieving name of temporary directory
include 'config/config.php';
$tempDir = (isset($GLOBALS['System']['temp']) ? $GLOBALS['System']['temp'] : 'pnTemp');

$action = (isset($_GET['action']) ? $_GET['action'] : (isset($_POST['action']) ? $_POST['action'] : false));
$lang = (isset($_GET['lang']) ? $_GET['lang'] : (isset($_POST['lang']) ? $_POST['lang'] : 'eng'));

if (file_exists($tempDir . '/upgrade_languages')) {
    $installed_languages = unserialize(file_get_contents($tempDir . '/upgrade_languages'));
    $lang = (in_array($lang, $installed_languages)) ? $lang : 'eng';
    if (file_exists($langfile = 'install/pnlang/' . $lang . '/upgrade.php')) {
        include_once $langfile;
        include "language/$lang/core.php";
    } else {
        include_once 'install/pnlang/eng/upgrade.php';
        include_once 'language/eng/core.php';
        $lang = 'eng';
    }
} else {
    $lang = 'eng';
    include_once 'install/pnlang/eng/upgrade.php';
    include_once 'language/eng/core.php';
}

//unset($PNConfig);

switch ($action)
{
    case 'intro':
        _upg_intro_to_110($lang);
        break;
    case 'start':
        // do the real work
        _upg_upgrade_to_110($lang);
        break;
    case 'learnmore':
        // show some news about 1.1.1
        _upg_learnmoreabout110();
        break;
    default:
        // select language
        _upg_selectlanguage();
}

function _upg_intro_to_110($lang)
{
    _upg_header(_UPG_TITLE10X, _UPG_SUBTITLE10X, _UPG_DESCRIPTION10X);
    
    echo '<h2>'._UPG_BACKUPNOTICE.'</h2>';
    echo '<p>'._UPG_BACKUPHINT.'</p>';
    echo '<p style="margin-top: 4em;"><a href="upgrade10xto11x.php?action=start&lang='.$lang.'">'._UPG_PROCEEDTOUPGRADE."</a></p>\n";
    
    _upg_footer();
}

function _upg_upgrade_to_110($lang)
{
    _upg_header(_UPG_TITLE10X, _UPG_SUBTITLE10X, _UPG_DESCRIPTION10X);

    // Need to use $_SESSION here because PN cleans out all globals
    $_SESSION['_ZikulaUpgrader']['_ZikulaUpgradeFrom10x'] = true;
    include 'includes/pnAPI.php';

    if (!pnInit(PN_CORE_ALL & ~PN_CORE_THEME & ~PN_CORE_AJAX & ~PN_CORE_SESSIONS & ~PN_CORE_TOOLS)) {
        Loader::requireOnce('includes/templates/dbconnectionerror.htm');
        pnShutDown();
    }

    echo '<ul id="upgradelist">'."\n";

    // upgrade modules table to work with the 1.1.1 framework
    Loader::requireOnce('system/Modules/pninit.php');
    modules_init_changestructure();

    // reload new modules pntables info
    unset($_SESSION['_ZikulaUpgrader']['_ZikulaUpgradeFrom10x']);
    pnModDBInfoLoad('Modules', 'Modules', true);

    // upgrade db entry for Modules module
    modules_updatedefaultdata($lang);
    echo '<li class="upgradedone">' . _UPG_MODULEDBSTRUCTUREUPGRADED . '</li>';

    // force load the modules admin API
    pnModAPILoad('Modules', 'admin', true);

    // regenerate the modules list
    pnModAPIFunc('Modules', 'admin', 'regenerate');

    // array with system modules
    $modules = array('Admin', 'Admin_Messages', 'AuthPN', 'Blocks', 'Categories', 'Errors', 'Groups', 'Header_Footer',
                     'legal', 'Mailer', 'ObjectData', 'PageLock', 'Permissions', 'pnForm', 'pnRender', 'Search',
                     'SecurityCenter', 'Settings', 'SysInfo', 'Theme', 'Users', 'Workflow');

    foreach ($modules as $module) {
        // $_SESSION['_PNUpgrader']['_PNUpgradeFrom10x'] = true;
        $modinfo = pnModGetInfo(pnModGetIDFromName($module));
        if ($modinfo['state'] == 5) {
            // 5 = version has changed
            if (pnModAPIFunc('Modules', 'admin', 'upgrade', array('id' => pnModGetIDFromName($module)))) {
                pnModAPIFunc('Modules', 'admin', 'setstate', array('id' => pnModGetIDFromName($module), 'state' => PNMODULE_STATE_ACTIVE));
                echo "<li class=\"upgradedone\">upgrade $module: " . _UPG_SUCCESSFUL . '</li>';
            } else {
                echo "<li class=\"upgradeerror\">upgrade $module: " . _UPG_FAILED . '</li>';
            }
        }
    }

    // regenerate the modules list to pick up any final changes
    pnModAPIFunc('Modules', 'admin', 'regenerate');

    echo '</ul>'."\n";
    unset($_SESSION['_ZikulaUpgrader']['_ZikulaUpgradeFrom10x']);

    // configure Admin module to show the installercheck
    pnModSetVar('Admin', 'ignoreinstallercheck', 0);
    
    echo '<p>' . _UPG_THANKSFORUPGRADINGTO110 . "</p>\n";
    echo '<p style="margin-top: 2em;">'._UPG_DONOTFORGETTOREMOVEFILES.'</p>';
    echo pnML('_UPG_CONTINUETO110NEWS', array('lang' => $lang), true);
}

function _upg_selectlanguage()
{
    _upg_header(_UPG_TITLE10X, _UPG_SUBTITLE10X, _UPG_DESCRIPTION10X);

    echo '<h2>Please select your language</h2>';
    // the following text is not a define by intention
    echo '<p>If you are missing your language here or see some strange lines like _UPG_DESCRIPTION you should install the most recent version of your language files. Please contact your local support site for more information about this topic. You can continue the installation in English and add your languages files later if you want.</p>';
    echo '<ul style="text-align: center; list-style-type: none;">' . "\n";
    $langbasedir = 'install/pnlang/';
    $dh = opendir($langbasedir);
    while(($lang = readdir($dh)) !== false) {
        if ($lang != '.' && $lang != '..' && $lang != '.svn' && $lang != 'CVS' && $lang != 'index.html' && is_dir("$langbasedir/$lang")) {
            $path = "$langbasedir/$lang/upgrade.php";
            if (file_exists($path)) {
                $languages[] = $lang;
                echo '<li style="margin: 1em 0;"><a href="upgrade10xto11x.php?action=intro&amp;lang=' . $lang . '"><img src="images/flags/flag-' . $lang . '.png" alt="" /></a></li>';
            }
        }
    }
    closedir($dh);
    $tempDir = (isset($PNConfig['System']['temp']) ? $PNConfig['System']['temp'] : 'pnTemp');
    file_put_contents($tempDir . '/upgrade_languages', serialize($languages));
    echo '</ul>'."\n";

    _upg_footer();
}

function _upg_learnmoreabout110()
{
    _upg_header(_UPG_TITLE10X, _UPG_THANKSFORUPGRADINGTO110, _UPG_NEWFEATURESINTRO);

    echo "<ul id=\"upgradelist\" style=\"margin-top: 2em;\">\n";
    echo '<li>' . _UPG_110FEATURE01 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE02 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE03 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE04 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE05 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE06 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE07 . "</li>\n";
    echo '<li>' . _UPG_110FEATURE08 . "</li>\n";
    echo "</ul>\n";
    echo '<p style="margin-top: 4em;">'._UPG_DONOTFORGETTOREMOVEFILES.'</p>';
    echo '<p style="margin-top: 4em;"><a href="admin.php">'._UPG_CONTINUETOYOURSITE."</a></p>\n";

    _upg_footer();
}


function _upg_header($title, $subtitle, $desc)
{
    echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">'."\n";
    echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-us">'."\n";
    echo '<head>'."\n";
    echo '<meta http-equiv="Content-Type" content="text/html; charset=ISO-8859-1" />'."\n";
    echo '<title>' . $title .'</title>'."\n";
    echo '<link rel="stylesheet" href="install/pnstyle/upgrade.css" type="text/css" />' . "\n";
    echo '</head>' . "\n";
    echo '<body>'."\n";
    echo '<div id="nonFooter"><div id="header">'."\n";
    echo '<div id="headertopleft"><img src="install/pnimages/top1.jpg" alt="" /></div>'."\n";
    echo '<div id="headertopright"><img src="install/pnimages/top2.jpg" alt="" /></div>'."\n";
    echo '</div>'."\n";
    echo '<div id="contents">';
    echo '<h1>' . $subtitle . '</h1>'."\n";
    echo '<p>' . $desc . '</p>';
}

function _upg_footer()
{
    echo '</div></div>'."\n";
    echo '<div id="footer">'."\n";
    echo '<p>'._UPG_FOOTERINFO.'</p>';
    echo '</div>'."\n";
    echo '</body>'."\n";
    echo '</html>';
}
